datatype spi_term = SPI_VAR of string
       | SPI_APPL of string * spi_term list
	 ;

type spi_pattern = spi_term;

datatype spi_fact = SPI_EQ of spi_term * spi_term
       | SPI_DIFF of spi_term * spi_term
	 ;

datatype spi_process = SPI_ZERO
       | SPI_CALL of string * spi_term list
	 (* f (X1, ..., Xn) *)
       | SPI_BANG of string * located_process (* name of var holding pid, process *)
	 (* ![X] P *)
       | SPI_NEW of string * located_process (* name of fresh var, process *)
	 (* new X; P *)
       | SPI_IF of spi_term * spi_term * located_process * located_process
	 (* if X=Y then P else Q *)
       | SPI_DECRYPT_SYM of spi_term * string * spi_term * located_process * located_process
	 (* case e1 of {X}_e2 => P else Q *)
       | SPI_DECRYPT_ASYM of spi_term * string * spi_term * located_process * located_process
	 (* case e1 of [X]_e2 => P else Q *)
       | SPI_DESTRUCT of spi_term * string * string list * located_process * located_process
	 (* case e1 of f (X1,...,Xn) => P else Q *)
       | SPI_IN of spi_term * string * located_process
	 (* in (c, x); P *)
       | SPI_OUT of spi_term * spi_term * located_process
       | SPI_LET of string * spi_term * located_process
       | SPI_EVT of spi_term * located_process
       | SPI_PAR of located_process list
withtype located_process = spi_process * int * int * int * int
	 ;

fun locate_process_end (loc, proc) =
    (proc, isub (loc, 2), isub (loc, 3), isub (loc, 2), isub (loc, 3));

fun mk_let (loc, var :: othervars, arg :: otherargs, p) =
    if arg=SPI_VAR var
	then mk_let (loc, othervars, otherargs, p)
    else locate_process_end (loc, SPI_LET (var, arg,
					   mk_let (loc, othervars, otherargs, p)))
  | mk_let (_, _, _, p) = p;
