(* Some definitions of useful dimensions and scales.
   Copyright (C) 1992-1995 Jean Goubault and Bull S.A.
   All rights reserved.

   Redistribution and use in source and binary forms are permitted provided
   that: (1) source distributions retain this entire copyright notice and
   comment, and (2) distributions including binaries display the following
   acknowledgement:  ``This product includes software developed by Jean
   Goubault as a contributor of Bull S.A.'' in the
   documentation or other materials provided with the distribution and in
   all advertising materials mentioning features or use of this software.
   Neither the name of Bull S.A. or of its corporate affiliates,
   nor the names of its contributors may be used to endorse or promote
   products derived from this software without specific prior written permission.
   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*)

dimension mass(kg) and distance(m) and intensity(A); (* MKSA system *)
	(* time(s) is predefined *)
dimension memory(byte);

(* scales used as multiples: *)
scale   1`deca = 10.0 and
	1`hecto = 100.0 and
	1`kilo = 1000.0 and
	1`myria = 10000.0 and
	1`mega = 1E6 and
	1`giga = 1E9 and
	1`tera = 1E12 and
	1`peta = 1E15 and
	1`exa = 1E18 and
	1`zetta = 1E21 and
	1`yotta = 1E24
scale   1`deci = 0.1 and
	1`centi = 0.01 and
	1`milli = 0.001 and
	1`micro = 1E~6 and
	1`nano = 1E~9 and
	1`pico = 1E~12 and
	1`femto = 1E~15 and
	1`atto = 1E~18 and
	1`zepto = 1E~21 and
	1`yocto = 1E~24
scale   1`Kilo = 1024.0
scale   1`Mega = 1`Kilo^2 and
	1`Giga = 1`Kilo^3 and
	1`Tera = 1`Kilo^4

(* scales of memory: *)
scale   1`Kb = 1`Kilo`byte and
	1`Mb = 1`Mega`byte and
	1`Gb = 1`Giga`byte and
	1`Tb = 1`Tera`byte

(* scales with no dimension: *)
type angle = num
scale   1`r = 1.0 and (* radian *)
	180`deg = 3.14159265358979324847905
scale   1`turn = 360`deg	(* 1 full rotation, or the length of the scale circle *)
scale   100`grad = 90`deg   (* grade *)
scale   60`arcmin = 1`deg and   (* minute of arc *)
	3600`arcs = 1`deg   (* second of arc *)

type solid_angle = num
scale   1`sr = 1.0	(* steradian *)
scale   1`sturn = 800`grad^2	(* 1 "solid turn", or the area of the scale sphere *)

type quant = num
scale   1`mol = 6.023E23	(* Avogadro number *)

(* mass: *)
scale   1`g = 1`milli`kg
scale   1`t = 1`mega`g and  (* metric ton *)
	1`ton = 907.18474`kg and	(* short ton *)
	1`tonUK = 1016.0469088`kg and   (* long ton *)
	1`ct = 0.0002`kg and	(* carat *)
	1`grain = 0.00006479891`kg and
	1`lb = 0.45359237`kg and	(* pound *)
	1`lbt = 0.3732417`kg and	(* troy pound *)
	1`oz = 0.028349523125`kg and	(* ounce *)
	1`ozt = 0.031103475`kg and  (* troy ounce *)
	1`slug = 14.5939029372`kg and
	1`u = 1.66057E~27`kg	(* atomic mass *)

(* distance, length: *)
scale   1`km = 1`kilo`m and
	1`dm = 1`deci`m and
	1`cm = 1`centi`m and
	1`mm = 1`milli`m and
	1`um = 1`micro`m and
	1`angstrom = 1E~10`m and
	1`fermi = 1E~15`m
scale   1`au = 149597900000`m and   (* astronomical scale *)
	1`chain = 20.1168402337`m and   (* chain *)
	1`fathom = 1.82880365761`m and
	1`ft = 0.3048`m and (* international foot *)
	1`ftUS = 0.304800609601`m and   (* survey foot *)
	1`inch = 0.0254`m and   (* international inch *)
	1`lyr = 9.46052840488E15`m and  (* light year *)
	1`mi = 1609.344`m and   (* international mile *)
	1`mil = 0.0000254`m and
	1`miUS = 1609.34721869`m and	(* statute mile *)
	1`nmi = 1852`m and  (* nautical mile *)
	1`pc = 3.08567818585E16`m and   (* parsec *)
	1`rd = 5.02921005842`m and  (* rod *)
	1`yd = 0.9144`m (* international yard *)

(* area: *)
type	surface = distance^2
scale   1`are = 100`m^2 and
	1`acre = 4046.87260987`m^2 and
	1`barn = 1E~28`m^2

(* volume: *)
type	volume = distance^3
scale   1`bbl = 0.158987294928`m^3 and  (* barrel of oil *)
	1`bu = 0.03523907`m^3 and   (* bushel *)
	1`cu = 2.365882365E~4`m^3 and   (* cup *)
	1`fbm = 0.002359737216`m^3 and  (* board feet *)
	1`gal = 0.003785411784`m^3 and  (* US gallon *)
	1`galC = 0.00454609`m^3 and (* canadian gallon *)
	1`galUK = 0.004546092`m^3 and   (* UK gallon *)
	1`l = 1`dm^3 and	(* liter *)
	1`ozfl = 2.95735295625E~5`m^3 and   (* US fluid ounce *)
	1`ozUK = 0.000028413075`m^3 and (* UK fluid ounce *)
	1`pk = 0.0088097675`m^3 and (* peck *)
	1`pt = 0.000473176473`m^3 and   (* pint *)
	1`qt = 0.000946352946`m^3 and   (* quart *)
	1`st = 1`m^3 and	(* stere *)
	1`tbsp = 1.47867647813E~5`m^3 and   (* tablespoon *)
	1`tsp = 4.92892159375E~6`m^3	(* teaspoon *)

(* time, duration: *)
scale   1`ms = 1`milli`s and
	1`us = 1`micro`s and
	1`min = 60`s
scale   1`h = 60`min
scale   1`j = 24`h
scale   1`sem = 7`j
scale   1`yr = 365.24219878125`j	(* tropical year *)

(* frequency, period: *)
type frequency = time^~1
scale   1`Hz = 1`s^~1 and
	1`Bq = 1`s^~1 and  (* becquerel *)
	1`Ci = 37000000000`s^~1	(* curie *)

(* speed: *)
type speed = distance`time^~1
scale   1`c = 299792459`m`s^~1 and (* speed of light *)
	1`knot = 1`nmi`h^~1 and
	1`kph = 1`km`h^~1 and  (* kilometers per hour *)
	1`mph = 1`mi`h^~1  (* miles per hour *)

(* acceleration: *)
type acceleration = distance`time^~2
scale   1`ga = 9.80665`m`s^~2  (* standard free fall acceleration *)

(* force: *)
type force = mass`acceleration
scale   1`N = 1`kg`m`s^~2  (* newton *)
scale   1`dyn = 1E~5`N and  (* dyne *)
	1`gf = 1`g`ga and   (* gram-force *)
	1`kip = 1`kilo`lb`ga and
	1`lbf = 1`lb`ga and (* pound-force *)
	1`pdl = 1`lb`ft`s^~2   (* poundal *)

(* work, energy: *)
type work = force`distance
type energy = work
scale   1`J = 1`kg`m^2`s^~2	(* joule *)
scale   1`Btu = 1055.05585262`J and
	1`cal = 4.1868`J and	(* calorie *)
	1`erg = 1E~7`J and
	1`eV = 1.60219E~19`J and	(* electron volt *)
	1`therm = 105506000`J   (* EEC therm *)

(* power: *)
type power = work`time^~1
scale   1`W = 1`J`s^~1 and (* watt *)
	1`hp = 745.699871582`kg`m^2`s^~3   (* horsepower *)

(* pressure: *)
type pressure = force`surface^~1
scale   1`Pa = 1`N`m^~2	(* pascal *)
scale   1`atm = 101325`Pa and   (* standard atmosphere *)
	1`bar = 100000`Pa and
	1`psi = 1`lbf`inch^~2 and  (* pound-force per square inch *)
	1`inHg = 3386.38815789`Pa and   (* inches of a column of mercury at 0 deg C *)
	1`inH2O = 248.84`Pa and (* inches of a column of water at 60 deg F *)
	1`mmHg = 133.322368421`Pa   (* millimeters of a column of mercury at O deg C *)
scale   1`torr = 1`mmHg

(* luminance: *)
type luminance = energy`time^~1	(* to check !!! *)
scale   1`cd = 1`N`s^~1	(* candela *)   (* to check !!! *)
scale   1`lm = 7.95774715459E~2`cd  (* lumen *)

(* illumination: *)
type illumination = luminance`surface^~1   (* to check !!! *)
scale   1`fc = 1`cd`m^~2 and   (* footcandle *)
	1`flam = 3.42625909964`cd`m^~2 and (* foot lambert *)
	1`lam = 3183.09886184`cd`m^~2 and  (* lambert *)
	1`lx = 7.95774715459E~2`cd`m^~2 and	(* lux *)
	1`ph = 795.774715459`cd`m^~2 and   (* phot *)
	1`sb = 10000`cd`m^~2   (* stilb *)

(* eletrical charge: *)
type charge = intensity`time
scale   1`C = 1`A`s (* coulomb *)
scale   1`Fdy = 96487`C (* faraday *)

(* tension, electrical potential: *)
type tension = power`intensity^~1  (* P = U.I *)
scale   1`V = 1`W`A^~1 (* volt *)

(* electrical capacity: *)
type capacity = charge`tension^~1  (* q = C.U *)
scale   1`F = 1`C`V^~1 (* farad *)
scale   1`mF = 1`milli`F and
	1`uF = 1`micro`F and
	1`nF = 1`nano`F and
	1`pF = 1`pico`F

(* inductance: *)
type inductance = tension`time`intensity^~1	(* U = L.dI/dt *)
scale   1`H = 1`V`s`A^~1   (* henry *)
scale   1`mH = 1`milli`H and
	1`uH = 1`micro`H and
	1`nH = 1`nano`H and
	1`pH = 1`pico`H

(* electrical resistance: *)
type resistance = tension`intensity^~1 (* U = R.I *)
scale   1`ohm = 1`V`A^~1
scale   1`mohm = 1`milli`ohm and
	1`kohm = 1`kilo`ohm and
	1`Mohm = 1`mega`ohm

type conductance = resistance^~1
scale   1`mho = 1`ohm^~1 and
	1`S = 1`ohm^~1 (* siemens *)

(* electric field: *)
type electric_field = tension`distance^~1  (* kg`m`s^~3`A^~1 *)

(* magnetic field: *)
type magnetic_field = inductance`intensity`surface^~1
scale   1`T = 1`H`A`m^~2   (* tesla *)

(* magnetic flow: *)
type magnetic_flow = magnetic_field`surface
scale   1`Wb = 1`T`m^2  (* weber *)

scale   1`P = 1`kg`m^~1`s^~1  (* poise *)

scale   1`St = 1`m^2`s^~1  (* stokes *)

scale   1`R = 0.000258`C`kg^~1 (* roentgen *)

scale   1`Gy = 1`m^2`s^~2 and  (* gray *)
	1`rad = 0.01`m^2`s^~2 and
	1`rem = 0.01`m^2`s^~2 and
	1`Sv = 1`m^2`s^~2  (* sievert *)

(* other things to put in scales:
 fundamental constants (eps0, mu0, hbar, density of air, water, etc.)
*)

	(* typography: *)
scale   72.27`didot = 1`inch and	(* didot point *)
	72`point = 1`inch   (* large point *)
