(* Universal random number generator
   Copyright (C) 1992-1995 Jean Goubault and Bull S.A.
   All rights reserved.

   Redistribution and use in source and binary forms are permitted provided
   that: (1) source distributions retain this entire copyright notice and
   comment, and (2) distributions including binaries display the following
   acknowledgement:  ``This product includes software developed by Jean
   Goubault as a contributor of Bull S.A.'' in the
   documentation or other materials provided with the distribution and in
   all advertising materials mentioning features or use of this software.
   Neither the name of Bull S.A. or of its corporate affiliates,
   nor the names of its contributors may be used to endorse or promote
   products derived from this software without specific prior written permission.
   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
   WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*)

(* Based upon George Marsaglia's (FSU) algorithm
 from Ada Letters v. VIII, n. 2 1988.
 I have tested it and do not find it very random, however.
 (It alternates between numbers above and below 0.5, making
  them decrease; when one goes under 0.0 or 0.5 respectively,
  then two numbers get on the same side of 0.5,
  then it does the same pattern again.)
*)

local
    val m1 = 179
    val m2 = m1-10
    val default_i = 12 and default_j = 34 and default_k = 56 and default_l = 78
    val m3 = 97
    val init_c = 362436.0 #/ 16777216.0
    val cd = 7654321.0 #/ 16777216.0
    val cm = 16777213.0 #/ 16777216.0
    val ir = ref default_i
    val jr = ref default_j
    val kr = ref default_k
    val lr = ref default_l
    val ni = ref m3
    val nj = ref (m3 div 3+1)
    val c = ref init_c
    val u = array (m3+1, 0.0 : num)
    fun iter (i,j,f) =
	let val k = ref i
	in
	    while !k<=j
		do (f (!k); inc k)
	end
in
    fun ur_start (i,j,k,l) =
	(* i, j, k must be in the range 1 to 178, and not all 1, while l may be an integer
	 ranging from 0 to 168.  if nonnegative integer values are assigned to i, j, k or l
	 outside the specified ranges, the generator will still perform satisfactorily but
	 not product exactly the same bit patterns on different computers. *)
	(* This procedure initializes the table for
	 f (m3,  m3/ 3 + 1, - mod 1.0), a lagged-fibonacci sequence
	 generator, and produces values for the arithmetic sequence. *)

	(ir := i;
	 jr := j;
	 kr := k;
	 lr := l;
	 ni := m3;
	 nj := m3 div 3+1;
	 c := init_c;
	 iter (1,m3,fn ii =>
	       let val s = ref 0.0 : num ref
		   val t = ref 0.5 : num ref
	       in
		   iter (1,24,fn jj =>
			 let val m = (((!jr * !ir) mod m1) * !kr) mod m1
			 in
			     ir := !jr;
			     jr := !kr;
			     kr := m;
			     lr := (53 * !lr + 1) mod m2;
			     (if (((!lr * m) mod 64) >= 32)
				  then s := !s #+ !t
			      else ());
			     t := 0.5 #* !t
			 end);
		   update (u,ii,!s)
	       end)
	 )

    fun ur_next () =
	let val t1 = sub (u, !ni) #- sub (u, !nj)
	    val temp = if t1 #< 0.0 then t1 #+ 1.0 else t1
	in
	    update (u,!ni,temp);
	    ni := (if !ni=1 then m3 else !ni-1);
	    nj := (if !nj=1 then m3 else !nj-1);
	    let val c' = !c #- cd
	    in
		c := (if c' #< 0.0 then c' #+ cm else c')
	    end;
	    let val res = temp #- !c
	    in
		if res #< 0.0
		    then res #+ 1.0
		else res
	    end
	end
end
