/* Opcodes for abstract syntax.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

	/*  codes for abstract syntax.
	Certain conditions should hold:
	- for every *_OF, *_MAP should be *_OF+1
	*/

#define A_STRING	0
#define A_NUMBER	1
#define A_VARIABLE  2
#define A_UNIT_POWER	3
#define A_UNIT_STAR 4
#define A_SET_UNIT  5
#define A_LABEL	6
#define A_RECORD_MAPLET 7
#define A_RECORD	8
#define A_CODEPOINT 9
#define A_TUPLE	10
#define A_PROGN	11
#define A_LIST	12
#define A_LIST_OF   13
#define A_LIST_MAP  14
#define A_IN_SET	15
#define A_IN_LIST   16
#define A_MAPLET	17
#define A_MAP	18
#define A_MAP_OF	19
#define A_MAP_MAP   20
#define A_MAP_UNDER	21
#define A_MAP_UNDER_OF  22
#define A_MAP_UNDER_MAP 23
#define A_PROG_OF   24
#define A_PROG_MAP  25
#define A_ALL_OF	26
#define A_ALL_MAP   27
#define A_EXISTS_OF 28
#define A_EXISTS_MAP	29
#define A_SOME_OF   30
#define A_SOME_MAP  31
#define A_LET   32
#define A_APPLY 33
#define A_CAST  34
#define A_AND   35
#define A_OR	36
#define A_HANDLE	37
#define A_RAISE 38
#define A_FN	39
#define A_LAMBDA	40
#define A_IF	41
#define A_WHILE 42
#define A_WHILE_LIST	43
#define A_WHILE_MAP 44
#define A_WHILE_MAP_UNDER   45
#define A_WHILE_ALL 46
#define A_WHILE_EXISTS  47
#define A_WHILE_SOME	48
#define A_DECL_LIST 49
#define A_LOCAL 50
#define A_VAL   51
#define A_VALREC	52
#define A_BIND  53
#define A_PAT_TUPLE 54
#define A_QUANTITY  55
#define A_NEWQUAN   56
#define A_UNIT  57
#define A_NEWUNIT   58
#define A_TYPE  59
#define A_TYPE_BIND 60
#define A_DATATYPE  61
#define A_DATATYPE_BIND 62
#define A_ABSTYPE   63
#define A_TYPE_VAR_SEQ  64
#define A_CONSTRUCTOR   65
#define A_EXCEPTION 66
#define A_NEWEXC	67
#define A_EXC_BIND  68
#define A_MAPLET_TYPE   69
#define A_RECORD_TYPE   70
#define A_TYPE_APPLY	71
#define A_TYPE_POWER	72
#define A_TYPE_STAR	73
#define A_TUPLE_TYPE	74
#define A_FUNCTION_TYPE 75
#define A_MAP_TYPE	76
#define A_PAT_ANY	77
#define A_PAT_RECORD_ELLIPSIS   78
#define A_PAT_TUPLE_ELLIPSIS	79
#define A_PAT_RECORD_MAPLET 80
#define A_PAT_RECORD	81
#define A_PAT_AS	82
#define A_PAT_CAST  83
#define A_PAT_NIL   84
#define A_PAT_ANYLIST   85
#define A_PAT_APPLY 86
#define A_PAT_EMPTY 87
#define A_PAT_ANYMAP	88
#define A_PAT_MAPLET	89
#define A_PAT_MAPADD	90
#define A_PRIME_DATABIND	91
#define A_UNIT_NAME 92
#define A_TYPE_VAR  93
#define A_REAL_VAR  94
#define A_PAT_VAR   95
#define A_PAT_CONST 96
#define A_SELECT_FN 97
#define A_MATCH 98
#define A_MU	99
#define A_SUB_MAP   100
#define A_DELAY 101
#define A_NONSENSE 102
#define A_PACK  103
#define A_PAT_PACK	104
#define A_INTEGER 105
#define A_TYPE_ELLIPSIS 106
#define A_EXT_DATATYPE 107
#define A_EXT_EQTYPE 108
#define A_EXT_QUANTITY 109
#define A_EXT_VAL 110
#define A_MEMO_FN   111
#define	A_CONS_INFO	112
#define	A_EQTYPE_INFO	113
#define A_EXT_EXC 114
#define A_BASIS 115
#define A_ENTER_FILE 116
#define A_EXIT_FILE 117
#define A_SET_LINE 118
#define A_PRAGMA 119
#define A_FUNBIND 120

#define	A_MODULES	121
#define	A_SIG	(A_MODULES+0)
#define	A_LOCAL_SPEC	(A_MODULES+1)
#define	A_VAL_SPEC	(A_MODULES+2)
#define	A_VAL_BIND_SPEC	(A_MODULES+3)
#define	A_TYPE_SPEC	(A_MODULES+4)
#define	A_EQTYPE_SPEC	(A_MODULES+5)
#define	A_NUMTYPE_SPEC	(A_MODULES+6)
#define	A_TYPE_BIND_SPEC	(A_MODULES+7)
#define	A_DATATYPE_DESC	(A_MODULES+8)
/*#define	A_EXCEPTION_DESC	(A_MODULES+9) FREE */
/*#define	A_NEWEXC_DESC	(A_MODULES+10) FREE */
#define	A_STRUCT_DESC	(A_MODULES+11)
#define	A_STRUCT_BIND_DESC	(A_MODULES+12)
#define	A_SHARING_DESC	(A_MODULES+13)
#define	A_STRUCT_SHARE_DESC	(A_MODULES+14)
#define	A_TYPE_SHARE_DESC	(A_MODULES+15)
#define	A_FUNCTOR_DESC	(A_MODULES+16)
#define	A_FUNCTOR_BIND_DESC	(A_MODULES+17)
#define	A_OPEN_DESC	(A_MODULES+18)
#define	A_UNIT_SPEC	(A_MODULES+19)
#define	A_INCLUDE_DESC	(A_MODULES+20)
#define	A_SIG_BIND	(A_MODULES+21)
#define	A_SIG_DECL	(A_MODULES+22)
#define	A_OPEN_DECL	(A_MODULES+23)
#define	A_STRUCT_DECL	(A_MODULES+24)
#define	A_STRUCT_BIND	(A_MODULES+25)
#define	A_STRUCT_CAST	(A_MODULES+26)
#define	A_FUNCTOR_APPLY	(A_MODULES+27)
#define	A_LET_STRUCT	(A_MODULES+28)
#define	A_FUNCTOR_DECL	(A_MODULES+29)
#define	A_FUNCTOR_BIND	(A_MODULES+30)

IMPORT  ExprPtr tc P((INOUT ExprPtr expr,INOUT ExprPtr assum,INOUT ExprPtr state));
