/* Prototype of an assembler syntax for expressing bytecode.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

%{
#include <stdio.h>
%}

%token kw_fun kw_open_paren kw_close_paren kw_end kw_comma kw_semicolon kw_colon
%token kw_goto kw_equal kw_nil kw_unbox kw_box kw_int kw_real
%token identifier integer_constant

%start file

%%

file :
     | file fun_decl
     ;

fun_decl : kw_fun identifier kw_open_paren formal_arg_list kw_close_paren
		body kw_end
	 ;

formal_arg_list :
		| formal_args
		;

formal_args : identifier
	    | formal_args kw_comma identifier
	    ;

body :
     | body stmt
     ;

stmt : kw_semicolon /* empty statement */
     | kw_open_paren body kw_close_paren /* group */
     | identifier kw_colon /* label */
     | kw_goto identifier
     | identifier kw_equal expr /* assignment */
     ;

expr : kw_open_paren expr kw_close_paren /* group */
     | kw_nil /* Z_NIL, the empty list */
     | integer_constant /* Z_UNBOXED_INT, an integer (unboxed) */
     | kw_unbox type expr
     | kw_box type expr
     ;

type : kw_int
     | kw_real
     ;

%%
