/* Macros needed to compile GimML.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

	/* Operating systems: */
#define UNIX_SYS_V 1
#define UNIX_BSD 2

	/* The maximal number of generations in the system */
#define MAX_GENERATIONS 5

#define STD_ML_TYVAR 1
#define STD_ML_NJ_TYVAR 2
#define EFFECT_TYVAR 3 /* not fully implemented */

#define CALLCC 1
#define PROMPT 2 /* Felleisen and Friedman's prompts, not implemented */
#define CALLCT 4 /* Danvy and Filinski's reset/callct, not implemented */
#define SPLITTER 8 /* Queinnec and Serpette's splitter, not implemented */
#define CATCH 16

#define NUMERICAL_MODEL_SET SetAdd(KSTR("complex floating point"),NewMap())
#define NO_MODULE 0
#define CAML_LIKE_MODULE 1
#define STD_ML_MODULE 2
#define MODULE_SYSTEM CAML_LIKE_MODULE /* no choice here for now */
#define MODULE_SET SetAdd(KSTR("separate compilation a la CaML"),NewMap())

/* PROFILING may contain:
   - PROFILING_CALL_COUNTS: code is profiled to determine
   number of calls to each function
   - PROFILING_TOTAL_TIMES: code is profiled to determine the time
   spent in each function, between entry and exit, including subfunctions
   - PROFILING_PROPER_TIMES: code is profiled to determine the time
   spent in each function, not including its subfunctions
   - PROFILING_TOTAL_MEM: code is profiled to determine
   number of bytes allocated, including subfunctions
   - PROFILING_PROPER_MEM: code is profiled to determine
   number of bytes allocated, not including subfunctions
   */
#define PROFILING_CALL_COUNTS 1
#define PROFILING_TOTAL_TIMES 2
#define PROFILING_PROPER_TIMES 4
#define PROFILING_TOTAL_MEM 8
#define PROFILING_PROPER_MEM 16
#define PROFILING (PROFILING_CALL_COUNTS | PROFILING_TOTAL_TIMES | PROFILING_PROPER_TIMES) /* no choice, but not implemented either */
#define PROFILING_SET SetAdd(KSTR("proper times"),SetAdd(KSTR("total times"),SetAdd(KSTR("call counts"),NewMap())))

#define STD_DEBUGGING 1
#define REPLAY_DEBUGGING 2 /* not implemented */

#define IEEE754 1
#define OTHER_FLOAT 2

#define STATUS_ALPHA 1
#define STATUS_BETA 2
#define STATUS_GAMMA 3
#define STATUS_RELEASE 5
