/* Definitions for parsing functions.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef YYLTYPE
typedef
struct yyltype
{
  int first_line;
  int first_column;
  int last_line;
  int last_column;
  char *text;
} yyltype;

#define YYLTYPE yyltype
#endif

#if CORE_TRACE
EXTERN hFILE *coreTraceFile;
IMPORT void resetTracingYyLex P((void));
IMPORT int tracingYylex P((void));
#endif

EXTERN Boolean unit_recognized;
EXTERN COUNTER star_recognized,union_recognized;
EXTERN SymPtr lexId,lexPath;
EXTERN ExprPtr lexEnv,		/* maps ids to precedences as E_INT */
	       consEnv;		/* sets of constructors */
#if MODULE_SYSTEM & STD_ML_MODULE
EXTERN ExprPtr strEnv,
	       sigEnv;
#endif
struct lexOpenToks {
  char *begin;
  char *end;
  struct lexOpenToks *next;
};
EXTERN struct lexOpenToks *lex_open_toks;
EXTERN SymPtr consSym,itSym,it2Sym;
EXTERN char *yytext;
EXTERN int yylineno,yyposno;
EXTERN stackDesc toplevelJmpBuf;
EXTERN Boolean toplevelJmpBufInstalled;

IMPORT void InitMarkerStack P((void));
IMPORT void pushMarker P((IN CONST char *begin, IN CONST char *end));
PUBLIC ExprPtr popMarker P((ExprPtr keep));

IMPORT void ResetFiles P((IN CONST char *mark,IN Boolean flushRemText));
IMPORT char *getMarkFiles P((void));
IMPORT void CloseFile P((void));
IMPORT Boolean OpenFile P((IN CONST CHAR *name));
IMPORT Boolean PushFile P((void));
IMPORT Boolean OpenString P((IN CONST CHAR *str,IN COUNTER len));

IMPORT void exportML P((IN CONST ExprPtr name));
IMPORT void importML P((IN CONST ExprPtr name));

IMPORT void setLocalEnv P((void));
IMPORT void setTopEnv P((void));

#if MODULE_SYSTEM & STD_ML_SYSTEM
IMPORT void keepEnv P((void));
IMPORT void restoreEnv P((void));

IMPORT void lexOpen P((IN CONST struct yyltype *loc,IN CONST SymPtr id,IN Boolean overwritep,IN CONST ExprPtr env));
IMPORT void popEnvMakeStruct P((IN CONST SymPtr strname));
IMPORT void popEnvMakeSig P((IN CONST SymPtr signame));
IMPORT void poppopEnvMakeFunctor P((IN CONST SymPtr funname,IN CONST SymPtr inname));
IMPORT void poppopEnvMakeFunctorSig P((IN CONST SymPtr funname,IN CONST SymPtr inname));

IMPORT void longLexOpen P((IN CONST ExprPtr longsym,IN Boolean sigp));
IMPORT void longLexOpenList P((IN CONST ExprPtr longsymlist,IN Boolean sigp));

IMPORT void saveLexDummy P((INOUT ExprPtr env));
IMPORT void initSaveLexDummy P((void));
IMPORT void endSaveLexDummy P((INOUT ExprPtr env));
IMPORT void popEnvMakeStructMatch P((IN CONST TZone *zone,IN CONST SymPtr strname,IN CONST SymPtr signame));
IMPORT void popEnvStructMatch P((IN CONST TZone *zone,IN CONST SymPtr signame));
IMPORT void popOtherEnv P((void));
PUBLIC void popEnvApplyFunctor P((IN CONST ExprPtr longsym));
#else
#define keepEnv()
#define restoreEnv()
#define lexOpen(loc,id,overwritep,env)
#endif

#if FLOAT_FORMAT==IEEE754
EXTERN ExprPtr *NaNs;
#endif


IMPORT ExprPtr DupCompAssum P((IN CONST ExprPtr table));
IMPORT COUNTER nLocals P((IN CONST ExprPtr table));
IMPORT void setNLocals P((IN CONST ExprPtr assum,IN COUNTER n));
EXTERN ExprPtr basisAssum,basisState,basisTable,basisEnv;

IMPORT ExprPtr genVar P((IN CONST struct yyltype *start,IN CONST struct yyltype *end));

IMPORT void tildeToMinus P((CHAR *text));
IMPORT ExprPtr getReal P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CHAR *text));
IMPORT ExprPtr getComplex P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CHAR *text));
IMPORT ExprPtr setFix P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CONST ExprPtr ll,IN COUNTER p));
IMPORT void makeZone P((OUT struct TZone *zone,IN CONST struct yyltype *yylsp1,IN CONST struct yyltype *yylsp2));
IMPORT ExprPtr Build P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr info));
IMPORT ExprPtr build1 P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a));
IMPORT ExprPtr build2 P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr b));
IMPORT ExprPtr build3 P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr b,IN CONST ExprPtr c));
IMPORT ExprPtr buildrev P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr list));
IMPORT ExprPtr build1rev P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr list));
IMPORT ExprPtr build2rev P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr b,IN CONST ExprPtr list));
IMPORT ExprPtr build1list P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr list));
IMPORT ExprPtr build2list P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER f,IN CONST ExprPtr a,IN CONST ExprPtr b,IN CONST ExprPtr list));

#define BUILD(start,end,f,info) Build(start,end,f,info)

IMPORT ExprPtr applyBin P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CONST ExprPtr f,IN CONST ExprPtr arg1,IN CONST ExprPtr arg2));
IMPORT ExprPtr patApplyBin P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CONST ExprPtr f,IN CONST ExprPtr arg1,IN CONST ExprPtr arg2));
IMPORT ExprPtr lexVar P((IN CONST struct yyltype *start,IN Boolean longp));
IMPORT ExprPtr buildFun P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN COUNTER n,IN CONST ExprPtr list));
IMPORT ExprPtr buildFunBind P((IN CONST struct yyltype *start,IN CONST struct yyltype *end,IN CONST ExprPtr list));
IMPORT void patchMemoFunList P((INOUT ExprPtr ll,IN CONST ExprPtr tyvarseq,IN CONST ExprPtr rest));
IMPORT void installMapPattern P((IN CONST struct yyltype *yylsp,IN CONST ExprPtr mapadds,IN CONST ExprPtr rest));
IMPORT ExprPtr zeroTuple P((IN CONST struct yyltype *yylsp));
IMPORT ExprPtr zeroPatTuple P((IN CONST struct yyltype *yylsp));
IMPORT ExprPtr patNil P((IN CONST struct yyltype *yylsp));
IMPORT ExprPtr makeNum P((IN CONST struct yyltype *yylsp,IN CONST ExprPtr num));
IMPORT ExprPtr makeInt P((IN CONST struct yyltype *yylsp,IN CONST ExprPtr num));

IMPORT void checkValRec P((IN CONST ExprPtr valrec));
IMPORT void checkUniqueCon P((IN CONST ExprPtr con,IN CONST ExprPtr conbind));
IMPORT void checkUniqueConstructors P((IN CONST ExprPtr datbind,IN CONST ExprPtr datbindlist));
IMPORT void checkUniqueIdInType P((IN CONST ExprPtr typbind,IN CONST ExprPtr typbindlist));
IMPORT void checkUniqueTypeSeq P((IN CONST ExprPtr typevar,IN CONST ExprPtr typeseq));
IMPORT void checkUniqueUnit P((IN CONST ExprPtr unit,IN CONST ExprPtr unitlist));
IMPORT void checkUniqueQuantityAndUnit P((IN CONST ExprPtr quan,IN CONST ExprPtr quanlist));
IMPORT void checkUniqueLabel P((IN CONST ExprPtr lexpr,IN CONST ExprPtr lrow));
IMPORT void checkQuantifiedType P((IN CONST ExprPtr tyvarseq,IN CONST ExprPtr type));
IMPORT void checkQuantifiedDatatype P((IN CONST ExprPtr tyvarseq,IN CONST ExprPtr conbindlist));
IMPORT void checkWithType P((IN CONST ExprPtr datbind,IN CONST ExprPtr typbind));
IMPORT void checkUniqueException P((IN CONST ExprPtr exc,IN CONST ExprPtr exclist));
IMPORT void checkEqTypes P((IN CONST ExprPtr eqtylist,IN CONST ExprPtr datbind));

IMPORT void checkUniqueValDesc P((IN CONST ExprPtr val,IN CONST ExprPtr vallist));

#if MODULE_SYSTEM & STD_ML_MODULE
IMPORT void checkUniqueSig P((IN CONST ExprPtr sig,IN CONST ExprPtr siglist));
IMPORT void checkUniqueStruct P((IN CONST ExprPtr str,IN CONST ExprPtr strlist));
IMPORT void checkUniqueFunctor P((IN CONST ExprPtr fun,IN CONST ExprPtr funlist));
#endif

IMPORT ExprPtr exprTyVars P((IN CONST ExprPtr expr,IN CONST ExprPtr tyvars));
IMPORT ExprPtr listTyVars P((IN CONST ExprPtr ll,IN CONST ExprPtr tyvars));
IMPORT ExprPtr exprToDecls P((IN CONST struct yyltype *start,
			      IN CONST struct yyltype *end,
			      IN CONST ExprPtr expr,
			      IN CONST ExprPtr sym));

IMPORT void yyinit P((void));
IMPORT NORETURN void ml_done P((int code)) _NORETURN;
IMPORT void cutoffInitials P((void));

typedef struct Basis {
  ExprPtr parseMarker;
  ExprPtr parseMarkerStack;
  Boolean syntax_analysis;
#if DEBUGGING
  Boolean evaluatingp;
  Boolean stopwanted;
#endif
  Boolean unit_recognized;
  COUNTER star_recognized;
  COUNTER union_recognized;
  ExprPtr lexEnv;
  ExprPtr consEnv;
  ExprPtr strEnv;
  ExprPtr sigEnv;
  excHandler *excStack;
  ExprPtr basisState;
  ExprPtr basisAssum;
  ExprPtr basisTable;
  ExprPtr basisEnv;
  ExprPtr bindings;
} Basis;

#define CHECKSUM_WIDTH 8
typedef CHAR Checksum[CHECKSUM_WIDTH];

IMPORT void CheckSum P((IN CONST ExprPtr e,OUT Checksum cs));

struct cslist {
  struct cslist *next;
  Checksum cs;
  ExprPtr basis;
  INT age;
};

IMPORT struct cslist *Checksums;

IMPORT ExprPtr basisNew P((IN Boolean copy));
IMPORT void basisInstall P((IN CONST ExprPtr e,IN Boolean errp,IN Boolean copy));

IMPORT void StoreInitialBasis P((void));
IMPORT struct cslist *CurrentChecksum P((void));
IMPORT struct cslist *GetChecksum P((IN CONST Checksum cs));
IMPORT void ResetCurrentAge P((IN INT age));

EXTERN ExprPtr theTypes,theBindings;

DECL_ACTION(precompile)
IMPORT int GimMLParse1 P((IN compiledAction action));

IMPORT ExprPtr preCompile P((IN CONST CHAR *filename,OUT CHAR **realname));
IMPORT ExprPtr moduleLoad P((IN CONST TZone *zone,IN CONST CHAR *name,IN Boolean recompile));
IMPORT ExprPtr moduleCompile P ((IN CONST CHAR *name));

IMPORT void checkConsEnv P((IN CONST TZone *zone,IN CONST ExprPtr basis));
IMPORT void augmentLexicalEnv P((IN CONST ExprPtr basis));

#if DEBUGGING
IMPORT void setCurrentFile P((void));
#endif
