# template file contents (makefile.tpl), as included in makefile by makemake:
#  Makefile for GimML.
#  Copyright (C) 2021 Jean Goubault-Larrecq
#  and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
#  Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.
#
#  This file is part of GimML.
#
#  GimML is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2, or (at your option)
#  any later version.
#
#  GimML is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GimML; see the file COPYING.  If not, write to
#  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

STACK =
PORTABLE = portable.h config.h system.h file.h
EVAL = eval.h $(PORTABLE) refcount.h
EXPR = expr.h
ABSSYN = abssyn.h
COMPILE = compile.h
CONS = cons.h
SPECIAL = special.h
TYPE = type.h
MEM = mem.h
APPLMEM = applmem.h
INPUT = input.h linenoise.h
EXECINIT = execinit.h
ACTIONS = actions.h
PRINT = print.h gimml.tab.h
TIMING = timing.h
DEBUGGER = debug.h $(TIMING) linenoise.h
TRANSFER = transfer.h $(DEBUGGER)
GRAM = gram.h gimml.tab.h $(TRANSFER)
MESSAGE = messages
BYTECODE = bc.h $(TRANSFER)
BCAUX = $(BYTECODE) bcaux.h
REGEXP = re.h
BCI = rbci.h
LIP = lip.h lipact.h

OBJS =  ml.o arith.o exec.o lists.o map.o mapinit.o compile.o compaux.o compinit.o \
	cons.o consinit.o errors.o evalinit.o execinit.o glexinit.o gyaccinit.o \
	lipinit.o print.o symbols.o \
	expr.o gc.o gramaux.o init.o malloc.o mem.o mathutil.o stack.o modules.o \
	type.o typeaux.o typeinit.o gimml.tab.o gimml.yy.o debug.o \
	profile.o dbginit.o random.o file.o inline.o lip.o table.o \
	dir.o transfer.o bc.o bcaux.o bciaux.o bci0.o bci.o bcinit.o re.o glex.o gyacc.o linenoise.o readline.o OTHEROBJS
RUNOBJS = mlrun.o gimmlx.a
GIMMLXOBJS = arith.o exec.o lists.o map.o mapinit.o \
	compile2.o compaux2.o compinit.o cons.o consinit.o errors.o \
	evalinit.o execinit.o glexinit.o gyaccinit.o lipinit.o print.o symbols.o \
	expr.o gc.o gramaux2.o init.o malloc.o mem.o mathutil.o stack.o \
	modules2.o type2.o typeaux2.o typeinit.o \
	profile.o dbginit2.o random.o file.o lip.o table.o dir.o transfer.o \
	bc.o bcaux.o bciaux.o bci0.o bci2.o bcinit.o re.o \
	glex.o gyacc.o linenoise.o readline.o OTHEROBJS
LIBGIMMLOBJS = arith.o exec.o lists.o map.o mapinit.o \
	compile2.o compaux2.o compinit.o cons.o consinit.o errors.o \
	evalinit.o execinit.o glexinit.o gyaccinit.o lipinit.o print.o symbols.o \
	expr.o gc.o gramaux2.o init.o malloc.o mem.o mathutil.o stack.o \
	modules2.o type2.o typeaux2.o typeinit.o \
	profile.o dbginit2.o random.o file.o lip.o table.o dir.o transfer.o \
	bc.o bcaux.o bciaux.o bci0.o bci2.o bcinit.o re.o \
	glex.o gyacc.o linenoise.o readline.o OTHEROBJS
XOBJS = mlx.o cxaux.o arith.o exec.o lists.o map.o mapinit.o \
	compile2.o compaux2.o compinit2.o cons.o consinit.o errors.o \
	evalinit.o execinit2.o glexinit2.o gyaccinit2.o lipinit2.o print.o symbols.o \
	expr.o gc2.o gramaux2.o init.o malloc.o mem.o mathutil.o stack.o \
	modules2.o type2.o typeaux2.o typeinit.o \
	profile.o dbginit2.o random.o file.o lip.o table.o dir.o transfer2.o \
	bc.o bcaux.o bciaux.o bci0.o bci2.o bcinit.o re.o \
	glex.o gyacc.o linenoise.o readline.o OTHEROBJS

.c.o :
	$(CC) $(CFLAGS) $<

.s.o :
	$(AS) $(AFLAGS) $<

TARGET = all

all :   makefile gimml Emacs/gimml-mode.el gimmldep gimmlpack gimmllnk gimmlrun gimmlx gimmlx.h libgimml.a Glex/glex Gyacc/gyacc

clean : makefile
	find . -name '*.o' -exec rm {} \;
	find . -name '*.u' -exec rm {} \;
	find . -name 'core' -exec rm {} \;
	find Emacs -name '*.elc' -exec rm {} \;
	rm -f gimml mksys proto gmon.out system.h
	rm -f gimmldep gimmlrun gimmlx gimmlx.h
	(cd Glex; make clean)
	(cd Gyacc; make clean)

cleanall : clean
	find . -name '*.mlx' -exec rm {} \;
	find . -name '*~' -exec rm {} \;
	find . -name GimML.trace -exec rm {} \;
	-rm gimml.pkg
	rm -f `cat protofiles`

distrib : cleanall
	(cd Doc; make)
	cp fakemake makefile
	$(SH) makedistrib

gimml.pkg : gimml Emacs/gimml-mode.el gimmldep gimmlpack gimmllnk gimmlrun gimmlx gimmlx.h libgimml.a Glex/glex Gyacc/gyacc
	$(SH) mkdist.macosx  
	 
lha : cleanall
	(cd Doc; make)
	cp fakemake makefile
	$(SH) makelha

install : all
	install -d $(DESTDIR)$(INSTALL_DIR)
	install gimml gimmlx gimmlrun gimmldep gimmlpack gimmllnk Glex/glex Gyacc/gyacc $(DESTDIR)$(INSTALL_DIR)
	install -d $(DESTDIR)$(EMACS_DIR)
	install Emacs/gimml-mode.el Emacs/gimml-site.el $(DESTDIR)$(EMACS_DIR)
	install -d $(DESTDIR)$(INCLUDE_DIR)
	install gimmlx.h $(DESTDIR)$(INCLUDE_DIR)
	install -d $(DESTDIR)$(LIB_DIR)
	install libgimml.a $(DESTDIR)$(LIB_DIR)
	install -d $(DESTDIR)$(MAN_DIR)
	install gimml.1 gimmldep.1 gimmllnk.1 gimmlpack.1 gimmlrun.1 gimmlx.1 Glex/glex.1 Gyacc/gyacc.1 $(DESTDIR)$(MAN_DIR)

Glex/glex:
	(cd Glex; make)

Gyacc/gyacc:
	(cd Gyacc; make)

makefile : makefile.tpl config.h
	$(SH) makemake
	exec make

makedefs : OPTIONS
	@echo >makedefs "#!$(SH)"
	@echo >>makedefs "CC=\"$(CC)\";export CC"
	@echo >>makedefs "LD=\"$(LD)\";export LD"
	@echo >>makedefs "BISON=\"$(BISON)\";export BISON"
	@echo >>makedefs "FLEX=\"$(FLEX)\";export FLEX"
	@echo >>makedefs "AS=\"$(AS)\";export AS"
	@echo >>makedefs "CFLAGS=\"$(CFLAGS)\";export CFLAGS"
	@echo >>makedefs "AFLAGS=\"$(AFLAGS)\";export AFLAGS"
	@echo >>makedefs "LFLAGS=\"$(LFLAGS)\";export LFLAGS"
	@echo >>makedefs "OTHERLFLAGS=\"$(OTHERLFLAGS)\";export OTHERLFLAGS"
	@cat makedef0 >>makedefs

prof : gimml
	pixie -o gimml.pixie gimml
	gimml.pixie -path Utils -path Prover -cmd "use\"init.ml\";use\"pack.ml\";quit 0;"
	prof -pixie gimml gimml.Addrs gimml.Counts >gimml.prof

config.h : OPTIONS makeconfig.c
	$(LD) -o makeconfig makeconfig.c
	./makeconfig <OPTIONS >config.h
	rm -f makeconfig

gimml : $(OBJS) mksys patch makedefs
	-rm -f gimml # to avoid ulimit headaches from the linker
	$(LD) -o gimml $(OBJS) $(LFLAGS) $(OTHERLFLAGS)
	$(SH) makedefs patch gimml

gimmlx : $(XOBJS) mksys patch makedefs
	-rm -f gimmlx # to avoid ulimit headaches from the linker
	$(LD) -o gimmlx $(XOBJS) $(LFLAGS) $(OTHERLFLAGS)
	$(SH) makedefs patch gimmlx

gimmlrun : $(RUNOBJS) mksys patch makedefs
	-rm -f gimmlrun # to avoid ulimit headaches from the linker
	$(LD) -o gimmlrun $(RUNOBJS) $(LFLAGS) $(OTHERLFLAGS)
	$(SH) makedefs patch gimmlrun

gimmlx.a : $(GIMMLXOBJS)
	-rm -f gimmlx.a # to avoid complaints about file having a different architecture if compiling for multiple archs
	ar cru gimmlx.a $(GIMMLXOBJS)
	ranlib gimmlx.a

libgimml.a : $(LIBGIMMLOBJS)
	-rm -f libgimml.a # to avoid complaints about file having a different architecture if compiling for multiple archs
	ar cru libgimml.a $(LIBGIMMLOBJS)
	ranlib libgimml.a

gimmldep : dep.o
	$(LD) -o gimmldep dep.o $(LFLAGS) $(OTHERLFLAGS)

dep.o : dep.c
dep.c : dep.l
	$(FLEX) -s -8 -odep.c dep.l

mksys : mksys.c random.src
	$(LD) -o mksys mksys.c -lm

proto :
	(cd Protoize;make;mv proto ..)

system.h random.tbl :	mksys makedefs mksyscc
	$(SH) makedefs mksyscc

sarith.h : arith.c proto
	$(SH) genproto arith.c sarith.h

slists.h : lists.c proto
	$(SH) genproto lists.c slists.h

smap.h : map.c proto
	$(SH) genproto map.c smap.h

smapinit.h : mapinit.c proto
	$(SH) genproto mapinit.c smapinit.h

sevlinit.h : evalinit.c proto
	$(SH) genproto evalinit.c sevlinit.h

arith.o : $(EVAL) $(EXPR) sarith.h

lists.o : $(EVAL) $(EXPR) $(MEM) slists.h

map.o : $(EVAL) $(EXPR) smap.h

table.o : $(EVAL) $(EXPR) table.h

mapinit.o : $(EVAL) $(EXPR) smapinit.h

evalinit.o : $(EVAL) $(EXPR) $(BYTECODE) sevlinit.h

sbc.h : bc.c proto
	$(SH) genproto bc.c sbc.h

bc.o : $(EVAL) $(EXPR) $(BCAUX) $(MEM) sbc.h rbc.h

sbcaux.h : bcaux.c proto
	$(SH) genproto bcaux.c sbcaux.h

bcaux.o : $(EVAL) $(EXPR) $(BCAUX) $(MEM) sbcaux.h rbc.h

hastag.c labs.c : genlab.c bci.c
	$(LD) -o genlab genlab.c
	./genlab
	rm genlab

bciaux.o : $(MEM) $(EVAL) $(EXPR) bciaux.h

sbci.h : bci.c proto hastag.c labs.c
	$(SH) genproto bci.c sbci.h

bci0.o : $(EVAL) $(EXPR) $(BYTECODE) rbc.h

bci.o : $(MEM) $(EVAL) $(BYTECODE) $(EXPR) $(TIMING) $(DEBUGGER) $(BCI) sbci.h

bci2.o : bci.c $(MEM) $(EVAL) $(BYTECODE) $(EXPR) $(TIMING) $(DEBUGGER) $(BCI) sbci.h
	$(CC) $(CFLAGS) $< -DGIMMLRUN -o bci2.o

bcinit.o : $(EVAL) $(BYTECODE) $(EXPR)

sexec.h : exec.c proto
	$(SH) genproto exec.c sexec.h

exec.o : $(MEM) $(EVAL) $(EXPR) $(TIMING) $(TRANSFER) sexec.h

scompile.h : compile.c proto
	$(SH) genproto compile.c scompile.h

compile.o : $(MEM) $(EVAL) $(ABSSYN) $(GRAM) $(COMPILE) $(TYPE) $(EXPR) $(DEBUGGER) $(REGEXP) scompile.h rcompile.h

compile2.o : $(MEM) $(EVAL) $(ABSSYN) $(GRAM) $(COMPILE) $(TYPE) $(EXPR) $(DEBUGGER) $(REGEXP) scompile.h rcompile.h

scompaux.h : compaux.c proto
	$(SH) genproto compaux.c scompaux.h

compaux.o :  $(MEM) $(EVAL) $(ABSSYN) $(COMPILE) $(TYPE) $(EXPR) $(DEBUGGER) scompaux.h rcompaux.h

compaux2.o :  $(MEM) $(EVAL) $(ABSSYN) $(COMPILE) $(TYPE) $(EXPR) $(DEBUGGER) scompaux.h rcompaux.h

sinline.h : inline.c proto
	$(SH) genproto inline.c sinline.h

inline.o : $(MEM) $(EVAL) $(ABSSYN) $(GRAM) $(COMPILE) $(TYPE) $(EXPR) $(DEBUGGER) sinline.h rinline.h

scmpinit.h : compinit.c proto
	$(SH) genproto compinit.c scmpinit.h

compinit.o : compinit.c $(EVAL) $(COMPILE) $(EXPR) scmpinit.h

compinit2.o : compinit.c $(EVAL) $(COMPILE) $(EXPR) scmpinit.h
	$(CC) $(CFLAGS) $< -DGIMMLX -o compinit2.o

scons.h : cons.c proto
	$(SH) genproto cons.c scons.h

cons.o : $(MEM) $(EVAL) $(EXPR) $(SPECIAL) $(CONS) scons.h

scnsinit.h : consinit.c proto
	$(SH) genproto consinit.c scnsinit.h

consinit.o : $(EVAL) $(EXPR) $(SPECIAL) $(CONS) scnsinit.h

sdbginit.h : dbginit.c proto
	$(SH) genproto dbginit.c sdbginit.h

dbginit.o : $(EVAL) $(EXPR) $(DEBUGGER) $(TYPE) $(BCAUX) sdbginit.h

dbginit2.o : $(EVAL) $(EXPR) $(DEBUGGER) $(TYPE) $(BCAUX) sdbginit.h

sdebug.h : debug.c config.h proto
	$(SH) genproto debug.c sdebug.h

debug.o : $(MEM) $(EVAL) $(EXPR) $(DEBUGGER) $(TYPE) $(COMPILE) $(PRINT) $(BYTECODE) $(BCI) sdebug.h rdebug.h

sprofile.h : profile.c config.h proto
	$(SH) genproto profile.c sprofile.h

profile.o : $(MEM) $(EVAL) $(EXPR) $(DEBUGGER) $(TYPE) $(COMPILE) $(PRINT) $(BYTECODE) $(BCI) sprofile.h

serrors.h : errors.c proto
	$(SH) genproto errors.c serrors.h

errors.o : $(EVAL) $(MESSAGE) serrors.h

sprint.h : print.c proto
	$(SH) genproto print.c sprint.h

print.o :   $(EVAL) $(PRINT) $(TYPE) $(EXPR) sprint.h rprint.h

ssymbols.h : symbols.c proto
	$(SH) genproto symbols.c ssymbols.h

symbols.o : $(EVAL) $(EXPR) ssymbols.h

sexcinit.h : execinit.c proto
	$(SH) genproto execinit.c sexcinit.h

execinit.o : execinit.c $(EVAL) $(COMPILE) $(EXPR) $(EXECINIT) $(TYPE) sexcinit.h
	$(CC) $(CFLAGS) $< -DDATE="\"`date`\n\""

execinit2.o : execinit.c $(EVAL) $(COMPILE) $(EXPR) $(EXECINIT) $(TYPE) sexcinit.h
	$(CC) $(CFLAGS) $< -DGIMMLX -DDATE="\"`date`\n\"" -o execinit2.o

sexpr.h : expr.c proto
	$(SH) genproto expr.c sexpr.h

expr.o :	$(EVAL) $(EXPR) $(SPECIAL) $(MEM) sexpr.h

sgc.h : gc.c proto
	$(SH) genproto gc.c sgc.h

gc.o :  $(MEM) $(EVAL) $(TYPE) $(COMPILE) $(GRAM) $(EXPR) $(SPECIAL) $(CONS) $(DEBUGGER) $(TIMING) sgc.h

gc2.o :  gc.c $(MEM) $(EVAL) $(TYPE) $(COMPILE) $(GRAM) $(EXPR) $(SPECIAL) $(CONS) $(DEBUGGER) $(TIMING) sgc.h
	$(CC) $(CFLAGS) $< -DXLATE -o gc2.o

sgramaux.h : gramaux.c proto
	$(SH) genproto gramaux.c sgramaux.h

gramaux.o : gramaux.c $(MEM) $(EVAL) $(ABSSYN) $(EXPR) $(GRAM) $(COMPILE) $(INPUT) $(TYPE) $(DEBUG) sgramaux.h rgramaux.h
	$(CC) $(CFLAGS) $< -DINSTALL_DIR="\"$(INSTALL_DIR)\""

gramaux2.o : gramaux2.c $(MEM) $(EVAL) $(ABSSYN) $(EXPR) $(GRAM) $(COMPILE) $(INPUT) $(TYPE) $(DEBUG) sgramaux.h rgramaux.h

sinit.h : init.c proto
	$(SH) genproto init.c sinit.h

init.o : $(MEM) $(APPLMEM) $(EXECINIT) $(ACTIONS) $(EVAL) $(TIMING) sinit.h

smalloc.h : malloc.c proto
	$(SH) genproto malloc.c smalloc.h

smem.h : mem.c proto
	$(SH) genproto mem.c smem.h

malloc.o : $(PORTABLE) $(MEM) smalloc.h

mem.o : $(PORTABLE) $(MEM) smem.h

smthutil.h : mathutil.c proto
	$(SH) genproto mathutil.c smthutil.h

mathutil.o : $(PORTABLE) smthutil.h

linenoise.o : $(PORTABLE) linenoise.h

sreadline.h : readline.c proto
	    $(SH) genproto readline.c sreadline.h

readline.o : $(EVAL) $(MEM) $(TYPE) exeproto.h linenoise.h sreadline.h

sml.h : ml.c proto
	$(SH) genproto ml.c sml.h

ml.o :  $(PORTABLE) $(INPUT) $(DEBUGGER) $(EVAL) sml.h

mlrun.o :  $(PORTABLE) $(INPUT) $(DEBUGGER) $(EVAL) $(GRAM) $(INPUT) $(TYPE) $(ABSSYN) $(EXPR) sml.h

mlx.o : mlx.c $(PORTABLE) $(INPUT) $(DEBUGGER) $(EVAL) $(GRAM) $(INPUT) $(TYPE) $(ABSSYN) $(EXPR) sml.h cxaux.h mlstart.inc.c
	$(CC) $(CFLAGS) -DCC="\"$(CC)\"" -DCFLAGS="\"$(STACK_CFLAGS)\"" -DLD="\"$(LD)\"" -DLFLAGS="\"$(LFLAGS) $(OTHERLFLAGS)\"" -DLIB_DIR="\"$(LIB_DIR)\"" mlx.c

wrap: wrap.c
	$(CC) -o wrap wrap.c

mlstart.inc.c : wrap mlstart.c
	./wrap mlstart.c >mlstart.inc.c

smodules.h : modules.c proto
	$(SH) genproto modules.c smodules.h

modules.o : $(EVAL) $(TYPE) $(EXPR) smodules.h rmodules.h

modules2.o : $(EVAL) $(TYPE) $(EXPR) smodules.h rmodules.h

sstack.h : stack.c proto
	$(SH) genproto stack.c sstack.h

stack.o : stack.c $(STACK) $(EVAL) $(EXPR) $(MEM) $(DEBUG) sstack.h
	$(CC) $(STACK_CFLAGS) stack.c

strans.h : transfer.c proto
	$(SH) genproto transfer.c strans.h

transfer.o : $(EVAL) $(EXPR) $(GRAM) $(COMPILE) $(TYPE) $(MEM) $(TRANSFER) $(DEBUGGER) strans.h

transfer2.o : transfer.c $(EVAL) $(EXPR) $(GRAM) $(COMPILE) $(TYPE) $(MEM) $(TRANSFER) $(DEBUGGER) strans.h
	$(CC) $(CFLAGS) $< -DXLATE -o transfer2.o

stype.h : type.c proto
	$(SH) genproto type.c stype.h

type.o :  $(EVAL) $(ABSSYN) $(GRAM) $(TYPE) $(EXPR) $(MEM) stype.h rtype.h

type2.o :  $(EVAL) $(ABSSYN) $(GRAM) $(TYPE) $(EXPR) $(MEM) stype.h rtype.h

stypeaux.h : typeaux.c proto
	$(SH) genproto typeaux.c stypeaux.h

typeaux.o : $(MEM) $(EVAL) $(ABSSYN) $(TYPE) $(EXPR) stypeaux.h rtypeaux.h

typeaux2.o : $(MEM) $(EVAL) $(ABSSYN) $(TYPE) $(EXPR) stypeaux.h rtypeaux.h

stypinit.h : typeinit.c proto
	$(SH) genproto typeinit.c stypinit.h

typeinit.o : $(EVAL) $(TYPE) $(EXPR) stypinit.h

sre.h : re.c config.h proto
	$(SH) genproto re.c sre.h

re.o : re.c $(REGEXP) $(MEM) $(EVAL) $(EXPR) $(TRANSFER) sre.h

sgram.h : gimml.tab.c proto
	$(SH) genproto gimml.tab.c sgram.h

gimml.tab.o : $(EVAL) $(ABSSYN) $(EXPR) $(GRAM) $(TYPE) $(COMPILE) $(INPUT) $(DEBUGGER) sgram.h

slex.h : gimml.yy.c proto
	$(SH) genproto gimml.yy.c slex.h

gimml.yy.o : $(MEM) $(EVAL) $(GRAM) $(INPUT) $(ABSSYN) $(EXPR) gimml.tab.c slex.h

srandom.h : random.c proto
	$(SH) genproto random.c srandom.h

random.o : $(PORTABLE) random.tbl srandom.h

sfile.h : file.c proto
	$(SH) genproto file.c sfile.h

file.o : $(PORTABLE) $(MEM) $(TIMING) sfile.h

sdir.h : dir.c proto
	$(SH) genproto dir.c sdir.h

dir.o : $(MEM) $(TIMING) $(EVAL) sdir.h

sglexinit.h : glexinit.c proto
	$(SH) genproto glexinit.c sglexinit.h

glexinit.o : glexinit.c $(EVAL) $(EXPR) $(TYPE) sglexinit.h

glexinit2.o : glexinit.c $(EVAL) $(EXPR) $(TYPE) sglexinit.h
	$(CC) $(CFLAGS) $< -DGIMMLX -o glexinit2.o

sglex.h : glex.c proto
	$(SH) genproto glex.c sglex.h

glex.o : $(MEM) $(TIMING) $(EVAL) $(EXPR) $(TRANSFER) sglex.h

sgyaccinit.h : gyaccinit.c proto
	$(SH) genproto gyaccinit.c sgyaccinit.h

gyaccinit.o : gyaccinit.c $(EVAL) $(EXPR) $(TYPE) sgyaccinit.h

gyaccinit2.o : gyaccinit.c $(EVAL) $(EXPR) $(TYPE) sgyaccinit.h
	$(CC) $(CFLAGS) $< -DGIMMLX -o gyaccinit2.o

sgyacc.h : gyacc.c proto
	$(SH) genproto gyacc.c sgyacc.h

gyacc.o : $(MEM) $(TIMING) $(EVAL) $(EXPR) $(TRANSFER) sgyacc.h

lipinit.o : lipinit.c $(EVAL) $(TYPE) $(LIP) $(EXPR)

lipinit2.o : lipinit.c $(EVAL) $(TYPE) $(LIP) $(EXPR)
	$(CC) $(CFLAGS) $< -DGIMMLX -o lipinit2.o

lip.o : $(EVAL) $(LIP) $(TRANSFER) $(EXPR) $(MEM)

messages :
	touch messages # because Sun make thinks it can build it from messages.r

gimml.tab.c gimml.tab.h : gram.y
	$(BISON) -dl gram.y
	$(SH) bisonpatch gimml.tab.c
	mv gram.tab.h gimml.tab.h

gimml.yy.c :  lex.l
	$(FLEX) -s -L -8 lex.l
	$(SH) flexpatch gimml.yy.c

#Emacs/gimml-mode.elc : Emacs/gimml-mode.el
#	(cd Emacs; emacs -batch --eval "(byte-compile-file \"gimml-mode.el\")")

gimmlpack: gimmlpack.ml gimml
	./gimml -c gimmlpack.ml
	chmod a+x gimmlpack.mlx
	mv gimmlpack.mlx gimmlpack

gimmllnk:

gimmlx.h: $(EVAL) $(GRAM) $(INPUT) $(TYPE) $(ABSSYN) $(TRANSFER) gimmlx1.h
	$(SH) ./mkgimmlx.sh >gimmlx.h

cxaux.o : $(EVAL) $(EXPR) $(CONS) $(BYTECODE) cxaux.h

