#! /bin/sh
#  Build include file gimmlx.h.
#  Copyright (C) 2005 Jean Goubault-Larrecq
#  and LSV, CNRS UMR 8643 & INRIA Futurs projet Secsi & ENS Cachan.
#
#  This file is part of GimML.
#
#  GimML is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2, or (at your option)
#  any later version.
#
#  GimML is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GimML; see the file COPYING.  If not, write to
#  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

echo "/* Auxiliary definitions for GimML compiled functions."
echo "   Generated automatically by mkgimmlx.sh."
echo "   Copyright (C) 2021 Jean Goubault-Larrecq"
echo "   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,"
echo "   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France."
echo
echo "   This file is part of GimML."
echo "   As an exception to the fact that GimML is licensed under the"
echo "   GNU General Public License, the present file is subject to"
echo "   the following license:"
echo
echo "   Permission is granted to anyone to use this software for any"
echo "   purpose on any computer system, and to redistribute it freely,"
echo "   subject to the following restrictions:"
echo
echo "   1. The author is not responsible for the consequences of use of"
echo "   this software, no matter how awful, even if they arise"
echo "   from defects in it."
echo
echo "   2. The origin of this software must not be misrepresented, either"
echo "   by explicit claim or by omission."
echo
echo "   3. Altered versions must be plainly marked as such, and must not"
echo "   be misrepresented as being the original software."
echo "*/"

# Concatenate general.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN general.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" general.h
echo "/* END general.h */"
echo

# Concatenate system.h
echo "/* BEGIN system.h */"
cat system.h
echo "/* END system.h */"
echo

# Concatenate perks.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h), and excluding #include lines.
echo "/* BEGIN perks.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" perks.h | grep -v "#include"
echo "/* END perks.h */"
echo

# Concatenate config.h, except for #include lines.
echo "/* BEGIN config.h */"
grep -v "#include" config.h
echo "/* END config.h */"
echo

# Concatenate portable.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h), and excluding #include lines, at least those
# of the form #include "...", not #include <...>.
echo "/* BEGIN portable.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" portable.h | grep -v "#include.*\""
echo "/* END portable.h */"
echo

# Concatenate file.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h).
echo "/* BEGIN file.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" file.h
echo "/* END file.h */"
echo

# Concatenate refcount.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN refcount.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" refcount.h
echo "/* END refcount.h */"
echo

# Concatenate eval.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h), and excluding #include lines, at least those
# of the form #include "...", not #include <...>.
echo "/* BEGIN eval.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" eval.h | grep -v "#include.*\""
echo "/* END eval.h */"
echo

# Concatenate expr.h, removing GPL license comment (superseded by the above
# for inclusion in expr.h).
echo "/* BEGIN expr.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" expr.h
echo "/* END expr.h */"
echo

# Concatenate bciaux.h, removing GPL license comment (superseded by the above
# for inclusion in bciaux.h).
echo "/* BEGIN bciaux.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" bciaux.h
echo "/* END bciaux.h */"
echo

# Concatenate gimmlx.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h).
echo "/* BEGIN gimmlx.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" gimmlx1.h
echo "/* END gimmlx.h */"
echo

# Concatenate gram.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN gram.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" gram.h
echo "/* END gram.h */"
echo

# Concatenate gimml.tab.h
echo "/* BEGIN gimml.tab.h */"
cat gimml.tab.h
echo "/* END gimml.tab.h */"
echo

# Concatenate input.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN input.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" input.h
echo "/* END input.h */"
echo

# Concatenate type.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN type.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" type.h
echo "/* END type.h */"
echo

# Concatenate abssyn.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN abssyn.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" abssyn.h
echo "/* END abssyn.h */"
echo

# Concatenate transfer.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN transfer.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" transfer.h
echo "/* END transfer.h */"
echo

# Concatenate evlproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN evlproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" evlproto.h
echo "/* END evlproto.h */"
echo

# Concatenate errproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN errproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" errproto.h
echo "/* END errproto.h */"
echo

# Concatenate grmproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN grmproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" grmproto.h
echo "/* END grmproto.h */"
echo

# Concatenate expproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN expproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" expproto.h
echo "/* END expproto.h */"
echo

# Concatenate stkproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN stkproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" stkproto.h
echo "/* END stkproto.h */"
echo

# Concatenate prtproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN prtproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" prtproto.h
echo "/* END prtproto.h */"
echo

# Concatenate iniproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN iniproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" iniproto.h
echo "/* END iniproto.h */"
echo

# Concatenate dirproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN dirproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" dirproto.h
echo "/* END dirproto.h */"
echo

# Concatenate cmpproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN cmpproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" cmpproto.h
echo "/* END cmpproto.h */"
echo

# Concatenate typproto.h, removing GPL license comment (superseded by the above
# for inclusion in gimmlx.h)
echo "/* BEGIN typproto.h */"
awk "BEGIN { s=0; };\
    {if (s==2) print; }\
    /\/\*/ { if (s==0) s=1; };\
    /\*\// { if (s==1) s=2; };" typproto.h
echo "/* END typproto.h */"
echo

# Now include all DECL_ACTIONs.
echo "/* Declare all global actions. */"
for i in bciaux.h evlproto.h ex1proto.h ex2proto.h exeproto.h exgproto.h glexproto.h gyaccproto.h lipact.h regproto.h
do
  grep DECL_ACTION $i
done
echo "/* End declare all global actions. */"
echo
