/* gimmlrun main().
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <fcntl.h>
#if OS==UNIX_SYS_V || OS==UNIX_BSD
#include <sys/errno.h>
#else
#include <errno.h>
#endif
#include "portable.h"
#include "eval.h"
#include "gram.h"
#include "gimml.tab.h"
#include "input.h"
#include "type.h"
#include "abssyn.h"
#include "expr.h"
#include "evlproto.h"
#include "errproto.h"
#include "grmproto.h"
#include "expproto.h"
#include "stkproto.h"
#include "prtproto.h"
#include "iniproto.h"
#include "dirproto.h"
#include "cmpproto.h"
#include "typproto.h"
#if OS==UNIX_SYS_V || OS==UNIX_BSD
#include <signal.h>
#endif
#ifdef HAS_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#ifdef HAS_PROTOTYPES
#include "sml.h"
#endif

#define OPT_EXPANDED

EXTERN COUNTER crtab_maxlines;

EXTERN int gcTraceFd;

EXTERN struct Expression *features,*featuresType;
EXTERN char maintenance[],creator[];
EXTERN long replay_size;
EXTERN COUNTER theRmargin;

EXTERN INT maxCells;
EXTERN unsigned long ktuple2_hash_size,kons_hash_size,
  mapletkons_hash_size,mapkons_hash_size,
  int_hash_size;
EXTERN unsigned long data_hash_size,real_hash_size;
EXTERN unsigned long string_hash_size,array_hash_size;

EXTERN EXTENDED fone;

EXTERN struct Expression *nihil;

#if DEBUGGING
EXTERN COUNTER debugMode;
EXTERN Boolean debugEnabled;
#endif

PUBLIC int theArgc,theArgi,theArgi0=2;
PUBLIC char **theArgv;

PRIVATE void ml_getopt P((int argc,char **argv));
PRIVATE void usage P((IN Boolean fatalp));

/* Dummy variables needed by the rest of the GimML system: */
PUBLIC Boolean evaluatingp=false,stopwanted=false;
PUBLIC CHAR *yyfilename;
PUBLIC char *yytext = "";
PUBLIC int yynerrs = 0;
PUBLIC long yyoffset=0;
PUBLIC int yy_input_device=0; /* different from YY_INPUT_STDIN so that symbol
				 table is not printed after the bytecode is loaded. */

PUBLIC ExprPtr currentFile;
PUBLIC void setCurrentFile() {}
PUBLIC COUNTER initCommandMode = 1;
PUBLIC ExprPtr lexEnv, /* maps ids to precedences as E_INT */
  consEnv; /* sets of constructors */
PUBLIC SymPtr lexId,lexPath;
PUBLIC ExprPtr keepLexEnv,keepConsEnv;
PUBLIC Boolean unit_recognized=false;
PUBLIC COUNTER star_recognized=0;
PUBLIC COUNTER union_recognized=0;
PUBLIC void resetTracingYyLex() {}
PUBLIC void setTopEnv() {}
PUBLIC void ResetFiles(mark,flushRemText)
IN CONST char *mark;
IN Boolean flushRemText;
{
}
PUBLIC void yyinit() {}
PUBLIC void CloseFile() {}
/* End of dummy variables. */

PUBLIC void setConstructor(name)
IN CONST ExprPtr name;
{
  setcarMapPut(name,UNIT,consEnv);
}

PRIVATE ExprPtr copyEnv1(env)
IN CONST ExprPtr env;
{
  env = Nreverse(env);
  return Nreverse(env);
}

PUBLIC ExprPtr copyLexEnv()
{
  ExprPtr e;
  ENTER;

  e = copyEnv1(lexEnv);
  KEEPONLY(e);
  return e;
}

PUBLIC ExprPtr copyConsEnv()
{
  ExprPtr e;
  ENTER;

  e = copyEnv1(consEnv);
  KEEPONLY(e);
  return e;
}

PUBLIC char *getMarkFiles()
{
  return NULL;
}

struct bcframes;
struct bcstacks;

PUBLIC OPCODE DoBreak(bc, pc, top, bcframes, stacks, exc)
IN CONST struct byteCodeChunk *bc;
IN CONST OPCODE *pc;
IN CONST ExprPtr *top;
IN CONST struct bcframes *bcframes;
IN CONST struct bcstacks *stacks;
IN CONST ExprPtr exc;
{
  exit(2);
}

PUBLIC void restoreBreaks () {}
PUBLIC void reinstallBreaks () {}

PUBLIC compiledAction __gimmlx_code [1];
PUBLIC COUNTER _n__gimmlx_code = 0;

EXTERN ExprPtr theSyntax;

PUBLIC int yyparse P((void))
{
  TZone *zone;
  ExprPtr res,aux,l;
  ENTER;

  zone = (TZone *)alloctmp_anchored(sizeof(TZone),(char *)&res);
  zone->line1 = zone->line2 = zone->pos1 = zone->pos2 = 0;
  yyfilename = (CHAR *)theArgv[1];
  res = moduleLoad (zone, yyfilename, false);
  if (res==NIL)
    {
      int len = strlen (theArgv[1]);
      char *buf = alloctmp_anchored(len+64,(char *)&res);

      sprintf(buf,message(921),theArgv[1]);
      type_error(zone,(CHAR *)buf);
      return 1;
    }
  res = MakePair (E_CONS, res, NIL);
  KEEP(res);
  aux = ZBuild(zone,A_BASIS,res);
  KEEP(aux);
  l = MakePair (E_CONS, aux, NIL);
  KEEP(l);
  res = ZBuild(zone,A_DECL_LIST,l);
  KEEPONLY(res);
  theSyntax = res;
  return 0;
}

EXTERN SymPtr funSym;

PRIVATE void MLRun()
{
#if DEBUGGING
  currentFile = NIL;
#endif
  lexEnv = consEnv = NIL;
  pushEnv ();
  lexId = lexPath = NIL;
  keepLexEnv = keepConsEnv = NIL;
  (void) GimMLParse();
  {
    SymPtr mainSym;
    ExprPtr type;

    mainSym = Ksymbol(message(927));
    type = varType (mainSym,basisAssum,basisState
#if IMPERATIVE_TYPES!=EFFECT_TYVAR
		    ,false
#endif
		    );
    if (type==NOEXPR)
      berror ("cannot find main ().\n");
    if (type->kind==(CHAR)E_TYPE_APPL && fxSym(APPLFUN(type))==funSym && CAR(APPLARG(type))==NIL)
      { /* of type unit -> ? */
	ExprPtr i_l;

	i_l = MapGetSure (mainSym, IdLocs(basisTable));
	if (i_l->kind==(CHAR)E_INT)
	  {
	    COUNTER i;
	    ExprPtr prog;

	    i = IVAL(i_l);
	    prog = EVARS(GLOBENV)->expr[i];
	    evaluatingp = true;
	    (void) (*ACTION(prog))(ENV_CONS(QENV(prog),0));
	    exit (0);
	  }
	else berror ("internal bug, cannot find main in toplevel environment.\n");
      }
    else berror ("main() should be of type unit -> t for some type t.\n");
  }
  /*!!! now run "main (); quit 0;" */
}

PRIVATE NORETURN void loop P((int argc, char *argv[])) _NORETURN;

PRIVATE NORETURN void loop(argc,argv)
int argc;
char *argv[];
{
  int i;

  debugEnabled = false;
#if OS==UNIX_SYS_V || OS==UNIX_BSD
  signal(2,ml_break);
  signal(3,ml_done);
  signal(SIGTERM,ml_done);
#ifdef SIGFPE
  signal(SIGFPE,ml_fpe);
#endif
#endif
  MLRun();
  ml_done(1);
}

int main(argc,argv)
int argc;
char *argv[];
{
  ml_getopt(argc,argv);
  InitML();
  loop(argc,argv);
  return 0;
}

typedef struct Command {
  char *key;
  void (*do_) P((void));
  char *doc;
  long xmask; /* the command (number #j) cannot be used if the command #i
		    has been used before, for 1<<i in xmask;
		    in particular, if j=i, the command may be reused
		    only if 1<<i is not in xmask */
}   Command;

PRIVATE void do_initmemsize()
{
  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&replay_size)!=1)
    {
      fprintf(stderr,message(98));
      ml_done(10);
    }
}

PRIVATE void do_initusage()
{
  usage(false);
  ml_done(0);
}

PRIVATE void do_rmargin()
{
  int n = theRmargin;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%d",&n)!=1)
    {
      fprintf(stderr,message(103));
      ml_done(10);
    }
  theRmargin = (COUNTER)n;
}

PRIVATE void do_growfactor()
{
  double grow;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%lf",&grow)!=1 || grow<=fone)
    {
      fprintf(stderr,message(104));
      ml_done(10);
    }
  installGrowFactor(&grow);
}

PRIVATE void do_maxcells()
{
  long i = maxCells;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1024)
    {
      fprintf(stderr,message(793));
      ml_done(10);
    }
  maxCells = i;
}

PRIVATE void do_gctrace()
{
  char *file;

  if (theArgi>=theArgc)
    {
      fprintf(stderr,message(613));
      ml_done(10);
    }
  file = theArgv[theArgi++];
  gcTraceFd = open(file,O_CREAT | O_TRUNC | O_WRONLY, 0666);
  if (gcTraceFd==-1)
    {
      fprintf(stderr,message(614),file);
      ml_done(10);
    }
}

PRIVATE void do_pair_hash_size()
{
  unsigned long i = ktuple2_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(655),(unsigned long)KTUPLE2_HASH_SIZE);
      ml_done(10);
    }
  ktuple2_hash_size = i;
  kons_hash_size = i;
  mapletkons_hash_size = i;
  mapkons_hash_size = i;
}

PRIVATE void do_data_hash_size()
{
  unsigned long i = data_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(673),(unsigned long)DATA_HASH_SIZE);
      ml_done(10);
    }
  data_hash_size = i;
}

PRIVATE void do_int_hash_size()
{
  unsigned long i = int_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(657),(unsigned long)INT_HASH_SIZE);
      ml_done(10);
    }
  int_hash_size = i;
}

PRIVATE void do_real_hash_size()
{
  unsigned long i = real_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(658),(unsigned long)REAL_HASH_SIZE);
      ml_done(10);
    }
  real_hash_size = i;
}

PRIVATE void do_string_hash_size()
{
  unsigned long i = string_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(659),(unsigned long)STRING_HASH_SIZE);
      ml_done(10);
    }
  string_hash_size = i;
}

PRIVATE void do_array_hash_size()
{
  unsigned long i = array_hash_size;

  if (theArgi>=theArgc || sscanf(theArgv[theArgi++],"%ld",&i)!=1 || i<1)
    {
      fprintf(stderr,message(660),(unsigned long)ARRAY_HASH_SIZE);
      ml_done(10);
    }
  array_hash_size = i;
}

PRIVATE Command command[] = {
/* 0 */ { "-mem", do_initmemsize, " memory-size", 0x00000001L },
/* 1 */	{ "-col", do_rmargin, " number-of-columns", 0x00000002L },
/* 2 */	{ "-grow", do_growfactor, " memory-grow-factor", 0x00000004L },
/* 3 */ { "-maxcells", do_maxcells, " max-cells", 0x00000008L },
/* 4 */ { "-gctrace", do_gctrace, " file-name", 0x00000010L },
/* 5 */ { "-pair-hash-size", do_pair_hash_size, " #entries", 0x00000020L },
/* 6 */ { "-int-hash-size", do_int_hash_size, " #entries", 0x00000040L },
/* 7 */ { "-real-hash-size", do_real_hash_size, " #entries", 0x00000080L },
/* 8 */ { "-string-hash-size", do_string_hash_size, " #entries", 0x00000100L },
/* 9 */ { "-array-hash-size", do_array_hash_size, " #entries", 0x00000200L },
/*10 */ { "-data-hash-size", do_data_hash_size, " #entries", 0x00000400L, },
	{ NULL, NULL }
};

PRIVATE long requiredMask = 0L; /* the xmask of mandatory options */

PRIVATE char errbuf[1024];

PRIVATE void ml_getopt(argc,argv)
int argc;
char *argv[];
{
  Command *com;
  char *s;
  long xmask,mask,m;
  int i;
  Boolean start;

  theArgc = argc;
  theArgv = argv;
  /*replay_mode = false;*/
  xmask = 0L;
  for (theArgi = 2; theArgi<theArgc; )
    {
      if (!strcmp(theArgv[theArgi], "--"))
	{
	  while (++theArgi < theArgc)
	    {
	      theArgv[theArgi0++] = theArgv[theArgi];
	    }
	  break;
	}
      for (com=command, mask=1L; com->key!=NULL; com++, mask <<= 1)
	if (!strcmp(theArgv[theArgi],com->key))
	  {
	    if ((m = (com->xmask & xmask)))
	      {
		start = true;
		if (xmask & mask)
		  sprintf(errbuf,message(105),theArgv[theArgi]);
		else
		  {
		    sprintf(errbuf,message(106),theArgv[theArgi]);
		    s = errbuf+strlen(errbuf);
		    for (i=0; m!=0L; i++)
		      {
			if (m & 1)
			  {
			    if (start)
			      start = false;
			    else if (m!=0L && (m & (-m))==m)
			      strcpy(s,message(107));
			    else
			    strcpy(s,message(108));
			    s += strlen(s);
			    sprintf(s,"%s",command[i].key);
			    s += strlen(s);
			  }
			m >>= 1;
		      }
		  }
		{
		  fprintf(stderr,errbuf);
		  fprintf(stderr,"\n");
		  ml_done(10);
		}
	      }
	    theArgi++;
	    xmask |= mask;
	    (*com->do_)();
	    break;
	  }
      if (com->key==NULL)
	{
	  theArgv[theArgi0++] = theArgv[theArgi++];
	}
    }
  if ((m = (requiredMask & (~xmask))))
    {
      start = true;
      
      sprintf(errbuf,
	      ((m & (-m))==m)?message(110):message(111));
      s = errbuf+strlen(errbuf);
      for (i=0; m!=0L; i++)
	if (m & 1)
	  {
	    m >>= 1;
	    if (start)
	      start = false;
	    else if (m!=0L && (m & (-m))==m)
	      strcpy(s,message(107));
	    else
	      strcpy(s,message(108));
	    s += strlen(s);
	    sprintf(s,"%s",command[i].key);
	    s += strlen(s);
	  }
      {
	fprintf(stderr,errbuf);
	ml_done(10);
      }
    }
  theArgc = theArgi0;
  if (theArgc<2)
    usage (true);
  theArgi = 2;
}

PRIVATE void usage(fatalp)
IN Boolean fatalp;
{
  char *s;
  Command *com;
  long mask;

  sprintf(errbuf,message(926));
  s = errbuf+strlen(errbuf);
  for (com=command, mask = 1L; com->key!=NULL; com++, mask <<= 1)
    {
      sprintf(s,(requiredMask & mask)?" %s%s":" [%s%s]",com->key,com->doc);
      s += strlen(s);
    }
  sprintf (s, " file [-- arguments...]");
  s += strlen (s);
  *s++ = '\n';
  *s = 0;
  fprintf(stderr,errbuf);
  if (fatalp)
    ml_done(10);
}

#ifdef SIGFPE
void ml_fpe(code)
int code;
{
  while (true)
    Raise(ARITHEXC); /* cannot get back result,
			so raise exception ad infinitum;
			this is not very smart, but what
			else can we do ? */
}
#endif
