/* A few tests for GimML's random number generator.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"portable.h"
#include	<math.h>

#undef INIT_RANDOM
#include "random.src"

#define	NTESTS	100000L
#define	NCATEGORIES	16

void	main()
{
	REAL	y,sum,V;
	EXTENDED	x;
	long	i,j;
	long	sf2;
	unsigned short	seed,nuseed;
	EXTENDED	scale;
	short	fall[NCATEGORIES];
	short	fallbyte[256];

	InitRandom();
	x	=	0.0;
	scale	=	(1.0+ldexp(1.0,33));
	for	(j=0; j<3; j++)
	{
	for	(i=0; i<NCATEGORIES; i++)	fall[i]	=	0;
	sum	=	0.0;
	for	(i=0; i<NTESTS; i++)
	{
		randReal(&y);
		sum	+=	y;
		fall[(short)(y*NCATEGORIES)]++;
	}
	sum	=	sum/NTESTS;	/*	sum is the mean	*/
	printf("mean is %f.\n",sum);
	sf2	=	0;	/*	V = NCATEGORIES/NTESTS.=fall[i]^2 - NTESTS	*/
	for	(i=0; i<NCATEGORIES; i++)
	{
		printf("%d .",fall[i]);
		sf2	+=	((long)fall[i]*(long)fall[i]);
	}
	V	=	(NCATEGORIES*(EXTENDED)sf2)/NTESTS - NTESTS;
	printf("\nchi-square value (%d degrees of freedom)= %f.\n",NCATEGORIES-1,V);

	for	(i=0; i<256; i++)	fallbyte[i]	=	0;
	for	(i=0; i<NTESTS; i++)
	{
		fallbyte[RANDLONG() & 0xff]++;
	}
	sf2	=	0;	/*	V = NCATEGORIES/NTESTS.=fall[i]^2 - NTESTS	*/
	for	(i=0; i<256; i++)
	{
		printf("%d|",fallbyte[i]);
		sf2	+=	((long)fallbyte[i]*(long)fallbyte[i]);
	}
	V	=	(256*(EXTENDED)sf2)/NTESTS - NTESTS;
	printf("\nchi-square value (%d degrees of freedom) = %f.\n",256-1,V);
	}
}
