/* Static structures for bci.c.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

struct bcframe {
  byteCodeChunk *bc;
  OPCODE *pc;
};

#define MAX_RECURSE (BC_STACK_SIZE/2)
/* MAX_RECURSE is computed so that the return stack never overflows;
   this works because at each recursion we must push at least two
   arguments (environment and argument). */
#define MAX_MAP_STACK (2*((BC_STACK_SIZE)/(1+MAP_CHAIN_LENGTH)+1))
/* MAX_MAP_STACK is computed so that mapstack never overflows;
   more precisely, if mapstack overflows, then the value stack _x
   has overflown first. We check the latter. */
#define MAX_EXC_NEST 50

struct bcstacks {
#if DEBUGGING!=0
  struct bcstacks *parentStack;
#endif
  ExprPtr *mapstack[MAX_MAP_STACK];
  struct _x { COUNTER n; gcStackView *next; ExprPtr p[BC_STACK_SIZE]; } _x;
};

struct xhandler {
  byteCodeChunk *bc;
  OPCODE *pc;
  COUNTER n;
  ExprPtr **maptop;
  struct bcframe *bcframe;
};

struct bcframes {
#if DEBUGGING!=0
  struct bcframes *parentBcFrames;
  struct bcframe *curBcFrame;
#endif
  struct bcframe bcframes[MAX_RECURSE];
};
