/* Static structures for Hlex scanner engine.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

typedef struct yy_buffer_state *YY_BUFFER_STATE;

typedef INT yy_state_type;

/* yydata: current state of lexer (they are globals in usual scanners
   generated by flex). */

typedef struct yydata {
  /* FILE *yyin; =NULL*/
  ExprPtr yyread; /* : int -> string */
  ExprPtr yyreadln; /* : unit -> string */
  ExprPtr yywrap; /* : unit -> bool */
  ExprPtr yy_cbuf; /* NIL or an external containing YY_BUFFER_STATE yy_current_buffer; */
  ExprPtr yylloc; /* points to a 4-entry intarray (see rgyacc.h); */
  /* = NIL initially;*/
  char *yytext; /* !!!yytext is always a pointer, never an array (change flex table
		   generator so that it does not allow %array);
		   it is NULL or points into the yy_ch_buf array of yy_cbuf. */
  COUNTER yy_n_chars; /* number of characters read into yy_ch_buf */
  COUNTER yyleng;
  char *yy_c_buf_p; /*= (char *) 0;*/ /* Points to current character in buffer. */
  COUNTER yy_start; /*= 0;*/ /* start state number */
  yy_state_type yy_last_accepting_state;
  char *yy_last_accepting_cpos; /* points into yy_ch_buf or is NULL. */
  INT yy_start_stack_ptr; /*= 0;*/ /* We consider YY_STACK_USED to be defined */
  INT yy_start_stack_depth; /*= 0;*/
  INT *yy_start_stack; /*= 0;*/
  char yy_hold_char; /* holds the character lost when yytext is formed. */
  CHAR yy_init; /*= 1;*/ /* whether we need to initialize */
  Boolean yy_did_buffer_switch_on_eof;
} yydata;

/* yytables: generated by flex. */

typedef struct yytables {
  ExprPtr yy_action_table; /* vector of pointers to closures implementing the actions
			      in the switch (yy_act) below, of type glex_data -> 'token option;
			      to return a token x, return SOME x; otherwise, return NONE. */
  COUNTER yy_num_rules;
  COUNTER yy_end_of_buffer;
  INT yy_accept_n;
  INT yy_ec_n;
  INT yy_meta_n;
  INT yy_nultrans_n;
  INT yy_base_n;
  INT yy_def_n;
  INT yy_nxt_n;
  INT yy_chk_n;
  SHORT *yy_accept; /* inlined, i.e., points to some table inside yytables itself */
  SHORT *yy_base; /* same */
  SHORT *yy_def; /* same */
  SHORT *yy_nxt; /* same */
  SHORT *yy_chk; /* same */
  INT *yy_ec; /* same */
  INT *yy_meta; /* same */
  INT *yy_NUL_trans; /* same */
  INT yy_jambase;
  INT yy_jamstate;
  INT yy_NUL_ec;
} yytables;

struct yy_buffer_state
{
  /*FILE *yy_input_file;*/
  ExprPtr yyread;
  ExprPtr yyreadln;
  /* Size of input buffer in bytes, not including room for EOB characters. */
  INT yy_buf_size;
  char *yy_ch_buf;		/* input buffer */
  char *yy_buf_pos;		/* current position in input buffer */
  /* Number of characters read into yy_ch_buf, not including EOB characters. */
  COUNTER yy_n_chars;
  /* Whether we "own" the buffer - i.e., we know we created it,
   * and can realloc() it to grow it, and should free() it to delete it. */
#define YY_IS_OUR_BUFFER_MASK (0x04)
  /* Whether this is an "interactive" input source; if so, and
   * if we're using stdio for input, then we want to use getc()
   * instead of fread(), to make sure we stop fetching input after
   * each newline.
   */
#define YY_IS_INTERACTIVE_MASK (0x08)
  /* Whether we're considered to be at the beginning of a line.
   * If so, '^' rules will be active on the next match, otherwise not. */
#define YY_AT_BOL_MASK (0x10)
  /* Whether to try to fill the input buffer when we reach the end of it. */
#define YY_FILL_BUFFER_MASK (0x20)
#define YY_BUFFER_STATUS_MASK (0x03)
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
  /* When an EOF's been seen but there's still some text to process
   * then we mark the buffer as YY_EOF_PENDING, to indicate that we
   * shouldn't try reading from the input source any more.  We might
   * still have a bunch of tokens to match, though, because of
   * possible backing-up.
   *
   * When we actually see the EOF, we change the status to "new"
   * (via yyrestart()), so that the user can continue scanning by
   * just pointing yyd->yyin at a new input file.
   */
#define YY_BUFFER_EOF_PENDING 2
  CHAR yy_flags;
};

typedef unsigned char YY_CHAR;
