/* Header file for timing functions; this is for portability.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#if OS==UNIX_SYS_V || OS==UNIX_BSD
#ifndef HAS_TIME_T
typedef long time_t;
#endif
#include <sys/types.h>
#include <time.h>
/* for HZ or CLK_TCK */
#include <sys/times.h>
/* for struct tms */
#ifndef HZ
#ifdef CLK_TCK
#define HERTZ CLK_TCK
#else
/* provide rough estimates */
#if OS==UNIX_BSD
#define HERTZ 60
#else
#define HERTZ 50
#endif
#endif
#else
#define HERTZ HZ
#endif

#else
#define HERTZ 50
/* unknown */

#endif

struct h_tms {
  time_t tms_utime;
  time_t tms_stime;
};

/* should provide at least a struct tms with fields
   tms_utime and tms_stime, and a function
   long times(struct tms *), and a tickcount HZ
*/

