/* Tests whether we have got IEEE floating-point arithmetic.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <math.h>
#include <signal.h>

extern void exit (int);

int nerrs = 0;
double z;

void do_fpe(sig)
int sig;
{
  signal (SIGFPE, do_fpe);
  nerrs++;
}

int main ()
{
  double pinf,minf,nan1,nan2;

  z = 0.0;
  signal(SIGFPE,do_fpe);
  /* Testing +infty: */
  pinf = 1.0/z;
  minf = (-1.0)/z;
  if (pinf==minf)
    nerrs++;
  nan1 = sqrt(-2.0);
  nan2 = (z+z)/z;
  if (nan1==nan2 || nan1==pinf || nan2==pinf || nan1==minf || nan2==minf)
    nerrs++;
  printf("pinf=%f, minf=%f, nan1=%f, nan2=%f, exit(%d).\n", pinf, minf, nan1, nan2, nerrs);
  fflush(stdout);
  exit (nerrs);
}
