/* Type checking, gimmlrun version.
   Copyright (C) 2003-2005 Jean Goubault-Larrecq
   and LSV, CNRS UMR 8643 & INRIA Futurs projet Secsi & ENS Cachan.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <math.h>
#include "eval.h"
#include "abssyn.h"
#include "gram.h"
#include "type.h"
#include "expr.h"
#include "mem.h"
#include "grmproto.h"
#include "typproto.h"
#include "prtproto.h"
#include "errproto.h"
#include "zproto.h"
#ifdef HAS_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif
#include "rtype.h"
#ifdef HAS_PROTOTYPES
#include "stype.h"
#endif

#undef DEBUG
#undef DEBUG1
#undef DEBUG_EXPLICIT

EXTERN SymPtr equalSym,equalitySym,discrSym;
EXTERN CHAR *yyfilename;

#if IMPERATIVE_TYPES==EFFECT_TYVAR
PUBLIC ExprPtr theFx; /* contains the computed effect of the expression */
	/* used as an accumulator */
#endif

#define TP(type) TypePrint(type,do_printfile,(char *)hstderr,NIL)
#define EP(expr) valPrint(expr,alpha,do_printfile,(char *)hstderr,0)

	/*****  type-checking functions *****/

PRIVATE void setq(var,val)
INOUT ExprPtr *var;
IN CONST ExprPtr val;
{
  SETQ(*var,val);
}

#undef SETQ
#define SETQ(var,val) setq(&var,val)

PRIVATE void setcar(cell,val)
INOUT ExprPtr cell;
IN CONST ExprPtr val;
{
  SETCAR(cell,val);
}

#undef SETCAR
#define SETCAR(cell,val) setcar(cell,val)

PRIVATE void setcdr(cell,val)
INOUT ExprPtr cell;
IN CONST ExprPtr val;
{
  SETCDR(cell,val);
}

#undef SETCDR
#define SETCDR(cell,val) setcdr(cell,val)

PRIVATE ExprPtr Cons(a,b)
IN CONST ExprPtr a,b;
{
  ExprPtr res;

  MAKECONS(a,b,res);
  return KEEP(res);
}

#if IMPERATIVE_TYPES==EFFECT_TYVAR
PUBLIC void resetFx()
{
  SETQ(theFx,NewMap());
}

PRIVATE void addFx(fx)
IN CONST ExprPtr fx;
{
  ExprPtr aux;
  ENTER;

  aux = SetAdd(fx,theFx);
  SETQ(theFx,aux);
  QUIT;
}
#endif

PRIVATE ExprPtr tcDeclList1(args,assum,state)
	/* return assumptions */
IN CONST ExprPtr args;
INOUT ExprPtr assum;
INOUT ExprPtr state;
{
  ExprPtr res,decl_assum,e;
  ENTER;

  if (args==NIL)
    {
      res = NewTypeAssum();
      A_KIND(res) = A_KIND(assum);
      A_FLAGS(res) = A_FLAGS(assum);
    }
  else
    {
      e = CAR(args);
      res = decl_assum = TC(e,assum,state);
      A_KIND(res) = A_KIND(assum);
      A_FLAGS(res) = A_FLAGS(assum);
      while (args=CDR(args), args!=NIL)
	{
	  AddTypeAssum(assum,decl_assum);
	  e = CAR(args);
	  decl_assum = TC(e,assum,state);
	  AddTypeAssum(res,decl_assum);
	}
    }
  KEEPONLY(res);
  return res;
}

PRIVATE ExprPtr tcDeclList(expr,assum,state)
	/* A_DECL_LIST, return assumptions */
IN CONST ExprPtr expr;
INOUT ExprPtr assum;
INOUT ExprPtr state;
{
  return tcDeclList1(ARGS(expr),assum,state);
}

PRIVATE ExprPtr tcIllegal(expr,assum,state)
IN CONST ExprPtr expr;
INOUT ExprPtr assum;
INOUT ExprPtr state;
{
  crash_error(ZONE(expr),(CHAR *)message(65));
  return NIL;
}

MAPCOLLECTFUNC(do_nondummytycons)
{
  if (y->kind==(CHAR)E_CONS) /* then this is a datatype description */
    {
      if (CDR(y)==NewMap()) /* then this is a dummy datatype description */
	return NewMap();
    }
  return MapletKons(x,y);
}

PRIVATE MapPtr nonDummyTyCons(state)
IN CONST ExprPtr state;
{
  return MapCollect(S_TYCONS(state),do_nondummytycons,(CHAR *)NULL);
}

PRIVATE void untrimState(state,local_state)
INOUT ExprPtr state;
IN CONST ExprPtr local_state;
{
  ExprPtr aux;
  ENTER;

  aux = MapMerge(S_NONEQS(state),S_NONEQS(local_state));
  SETQ(S_NONEQS(state),aux);
  aux = MapMerge(S_TYCONS(state),nonDummyTyCons(local_state));
  SETQ(S_TYCONS(state),aux);
  QUIT;
}

MAPSWEEPFUNC(do_checkimports)
{
  struct checkimport *data;
  ExprPtr local_state;
  ExprPtr type,wanted;
  Boolean found;
  ENTER;

  data = MAPDATA(struct checkimport *);
  type = MapGet(x,A_IDS(data->assum),&found);
  if (!found)
    type = MapGet(x,A_IDEXTS(data->assum),&found);
  if (!found)
    {
      if (A_KIND(data->assum)==TA_REGULAR)
	{
	  type_soft_error(ZONE(data->zone),(CHAR *)varMsg(message(806),x));
	  data->err = true;
	}
    }
  else
    {
      local_state = DupTypeState(data->state);
      wanted = MapGetSure(x,A_IDEXTS(data->new_assum));
      type = newTypeInst(type,data->assum,local_state
#if IMPERATIVE_TYPES!=EFFECT_TYVAR
			 ,false
#endif
			 );
      (void) coerceTypesImport(ZONE(data->zone),type,wanted,
			       data->assum,local_state,x,false);
      /* check the type, but in the local_state, i.e. do not instantiate it
	 for outside the module. */
    }
  QUIT;
  return false;
}

MAPSWEEPFUNC(do_checkdatatypes)
{
  struct checkimport *data;
  ExprPtr tyfcn,new_tyfcn;
  Boolean found;
  ENTER;

  data = MAPDATA(struct checkimport *);
  new_tyfcn = MapGetSure(x,A_TYFCNS(data->new_assum));
  tyfcn = MapGet(x,A_TYFCNS(data->assum),&found);
  if (!found)
    {
      if (A_KIND(data->assum)==TA_REGULAR)
	type_soft_error(ZONE(data->zone),(CHAR *)varMsg(message(498),x));
    }
  else /* then we can link the datatype or dimension exactly only when
	  they were declared imported and exported from the same module,
	  i.e. when tyfcn and new_tyfcn are exactly equal (we could check
	  that they have the same type names, arities and equality
	  conditions, but this can only happen when they are declared
	  in the same .mlx). */
    {
      if (tyfcn!=new_tyfcn)
	{
	  type_soft_error(ZONE(data->zone),(CHAR *)varMsg(message(821),x));
	  data->err = true;
	}
    }
  QUIT;
  return false;
}

PRIVATE ExprPtr tcBasis(expr,assum,state)
     /* A_BASIS, return assumptions */
IN CONST ExprPtr expr;
INOUT ExprPtr assum;
INOUT ExprPtr state;
{
  Basis *basis;
  ExprPtr imports;
  ENTER;

  basis = (Basis *)EXTDATA(CAR(ARGS(expr)));
  untrimState(state,basis->basisState);
  /* compute the set of imported variables: */
  imports = A_IDEXTS(basis->basisAssum);
  /* If assum is TA_REGULAR, then check that all imports exist and have the right types;
     if it is TA_PRECOMPILE, check that all imports that exist have the right types: */
  {
    struct checkimport *data =
      (struct checkimport *)alloctmp(sizeof(struct checkimport));

    data->zone = expr;
    data->assum = assum;
    data->state = state;
    data->new_assum = basis->basisAssum;
    data->err = false;
    (void) MapSweep(imports,do_checkimports,data);
    if (data->err)
      type_error(ZONE(expr),(CHAR *)message(817));
  }
  /* Check that extern datatypes declared as admitting equality really do,
     and that they have the right type names and arities. */
  imports = A_TYEXTS(basis->basisAssum);
  {
    struct checkimport *data =
      (struct checkimport *)alloctmp(sizeof(struct checkimport));

    data->zone = expr;
    data->assum = assum;
    data->state = state;
    data->new_assum = basis->basisAssum;
    data->err = false;
    (void) MapSweep(imports,do_checkdatatypes,data);
    if (data->err)
      type_error(ZONE(expr),(CHAR *)message(817));
  }
  KEEPONLY(basis->basisAssum);
  return basis->basisAssum;
}

PUBLIC ExprPtr tc(expr,assum,state)
INOUT ExprPtr expr;
INOUT ExprPtr assum;
INOUT ExprPtr state;
{
  ExprPtr type;
  ENTER;

  switch (TYPE(expr))
    {
    case A_BASIS: type = tcBasis(expr,assum,state); break;
    case A_DECL_LIST: type = tcDeclList(expr,assum,state); break;
    default: type = tcIllegal(expr,assum,state); break;
    }
  KEEPONLY(type);
  return type;
}
