/* Produce code that prints some other piece of code.
   Copyright (C) 2021 Jean Goubault-Larrecq
   and Université Paris-Saclay, CNRS, ENS Paris-Saclay,
   Laboratoire Méthodes Formelles, 91190, Gif-sur-Yvette, France.

   This file is part of GimML.

   GimML is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   GimML is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with GimML; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

void stringify(FILE *f,char *text)
{
  char c, *s;
  char buf[5];
  
  fputc ('"', f);
  s = text;
  while (c=*s++, c!='\0')
    {
      if (c<0x20 || (signed char)c<(signed char)0xa0 || c=='\\' || c=='\"')
	switch (c)
	  {
	  case '\\': fputs ("\\\\", f); break;
	  case '"': fputs ("\\\"", f); break;
	  case '\n': fputs ("\\n", f); break;
	  case '\t': fputs ("\\t", f); break;
	  case '\v': fputs ("\\v", f); break;
	  case '\f': fputs ("\\f", f); break;
	  default:
	    buf[0] = '\\';
	    buf[3] = (c & 0x07) + '0';
	    c >>= 3;
	    buf[2] = (c & 0x07) + '0';
	    c >>= 3;
	    buf[1] = (c & 0x07) + '0';
	    buf[4] = '\0';
	    fputs (buf,f);
	    break;
	  }
      else fputc(c, f);
    }
  fputc ('"', f);
}

int main(int argc, char **argv)
{
  int i;
  FILE *f;
  char *line=NULL;
  size_t linecapp=0;
  char c;
  int state;

  for (i=1; i<argc; i++)
    {
      f = fopen (argv[1], "r");
      if (f==NULL)
	{
	  fprintf (stderr, "Cannot open file %s. stop.\n", argv[1]);
	  exit(1);
	}
    again:
      while (c = fgetc(f), isspace(c));
      if (c=='/')
	{
	  c = fgetc(f);
	  if (c=='*')
	    { /* gobble up initial comments */
	      state = 1;
	      while (c = fgetc(f), c!=EOF)
		{
		  if (c=='*' && state==1) state=2;
		  else if (c=='/' && state==2) break;
		  else state=1;
		}
	      goto again;
	    }
	  else
	    {
	      ungetc (c, f);
	      ungetc ('\\', f);
	    }
	}
      else
	{
	  ungetc (c, f);
	}
      while (getline(&line,&linecapp,f)>=0)
	{
	  fputs ("  hfputs (", stdout);
	  stringify (stdout, line);
	  fputs (", out);\n", stdout);
	}
      fflush (stdout);
      fclose (f);
    }
  exit(0);
}
