/*

Signed arbitrary length long int package.

	Copyright Arjen K. Lenstra, 1989
	Modified by Jean Goubault-Larrecq to interface nicely with GimML.

	Bugs, questions, suggestions, additions, whatever, to Jean Goubault-Larrecq:
	goubault@lsv.ens-cachan.fr.

	Original author:

		Arjen K. Lenstra
		Room 2Q334
		Bellcore
		445 South Street
		Morristown, NJ 07960

		email: lenstra@flash.bellcore.com
		tel: 201 829 4878
		fax: 201 829 2645
*/

/*!!! change all functions that act on files. */
/*!!! all static's should be put in some global array that urgent GCs could sweep
  through to free some space (except when called from an allocation function in
  this file). */
/*!!! check that uses of zmont* are enclosed in modulus saving/restoring functions. */

#include <stdio.h>
#include <math.h>
#if OS==UNIX_SYS_V || OS==UNIX_BSD
#include <sys/errno.h>
#else
#include <errno.h>
#endif
#include "eval.h"
#include "lip.h"
#include "lipact.h"
#include "transfer.h"
#include "expr.h"
#include "mem.h"
#ifdef HAS_STRING_H
#include <string.h>
#else
#include <strings.h>
#endif

#ifdef vax
#include <sys/types.h>
#endif

#ifndef vax
/*#include <malloc.h>*/
#include <stddef.h>
#endif

#if 0
#if defined(sparc) && defined(__svr4__)
#include <sys/rusage.h>
#endif /* SOLARIS */
#endif

# define ALLOCATE 1
# define PRT_REALLOC 0
# define START 1

EXTERN ExprPtr none,optttm;
IMPORT long randLong P((void));

#define BYTEL NBITS_IN_CHAR	/* 8 bits per byte */
#define NBITS NBITS_IN_LIP_LONG	/* must be even and 0 < NBITS <BITSOFLONG */
#define RADIXM          (RADIX-1)
#define RADIXROOTM      (RADIXROOT-1)

#define NPBITS ((NBITS_IN_LIP_LONG > 31)?31:NBITS_IN_LIP_LONG)

PRIVATE unsigned char log10rad_bytes[] = { LOG10RAD_BYTES };
#undef LOG10RAD
#define LOG10RAD (*(EXTENDED *)log10rad_bytes)
/* #define LOG10RAD        (EXTENDED)9.03089986991943585644*/	/* base 10 log(RADIX) */

#define MAXDEPTH        50
#define MULT_CROSS      30
#define SQUARE_CROSS    30

#define SIZE            LIP_SIZE	/* SIZE*NBITS must be >= BITSOFLONG */

#if (!(NBITS&1)&&(NBITS>0)&&(NBITS<BITSOFLONG)&&(SIZE*NBITS>=BITSOFLONG))

#define LINE            68	/* approximate bound # digits per line */
/*#define BUFSIZE         2048*/	/* at most 2048 characters per line */

PUBLIC long *thePrimes = 0;
PUBLIC long nPrimes = 0;
PRIVATE long primbnd;
/*PRIVATE long primbndsqr;*/

PUBLIC int liperr = 0;
PUBLIC ExprPtr lipErrExc;

/*Function prototypes for static internal functions.*/
PRIVATE void zabs P((verylong *pa));
PRIVATE void zsubmul P((long r, verylong a, verylong b));
#if 0
PRIVATE void hidetimer P((FILE *f, long w));
#endif
#if 0
PRIVATE void zhalt P((char *c));
#endif
PRIVATE void zaddls P((verylong a, verylong b, verylong c));
PRIVATE void zmstartint P((verylong n));
PRIVATE void zmkeep P((verylong n));
PRIVATE void zmback P((void));
PRIVATE void kar_mul P((verylong a, verylong b, verylong *c, long shi));
PRIVATE void kar_sq P((verylong a, verylong *c, long shi));
PRIVATE long zxxeucl P((verylong ain, verylong nin, verylong *invv, verylong *uu));
PRIVATE void mont_init_odd_power P((verylong a, verylong *a_sq, long m,
				    verylong **odd_a_power));
PRIVATE void init_odd_power P((verylong a, verylong n, verylong *a_sq,long m,
			       verylong **odd_a_power));
PRIVATE void zsmexp P((long a, verylong e, verylong *bb));
PRIVATE void zpshift P((struct smallprimegen *spg));
PRIVATE void zrootnewton P((verylong a, verylong *b, long n));
PRIVATE long ph1set P((verylong n, verylong *rap, verylong *alpha, verylong *mu,
		       verylong *x, verylong *f));
PRIVATE long ph1exp P((verylong n, verylong *x, verylong e, verylong rap, verylong *f));
PRIVATE long ph1 P((verylong n, verylong *x, long m, verylong rap, verylong *f,
		    IN struct smallprimegen *spg));
PRIVATE long ph1toph2 P((verylong n, verylong *x, verylong *y, verylong *ra,
			 verylong alpha, verylong mu, verylong *f));
PRIVATE long ph2squ P((verylong n, verylong x1s, verylong y1s, verylong *x2, verylong *y2,
		       verylong ra, verylong *f));
PRIVATE long ph2mul P((verylong n, verylong x1s, verylong y1s, verylong x2s, verylong y2s,
		       verylong *x3, verylong *y3, verylong ra, verylong *f));
PRIVATE long ph2exp P((verylong n, verylong x1, verylong yy1, verylong *x2, verylong *y2,
		       long e, verylong ra, verylong *f));
PRIVATE long ph2set P((verylong n, verylong x, verylong y, long te, verylong ra,
		       verylong *f));
PRIVATE void ph2prec P((verylong n, long n1, long j));
PRIVATE void ph2eval P((verylong n, long ind, long n1, long j, verylong *result));
PRIVATE long ph2 P((verylong n, verylong inx, verylong iny, long m, long te,
		    verylong ra, verylong *f));

PRIVATE void lipSweep (extdata, doext)
INOUT char *extdata;
IN CONST doExtDesc *doext;
{
  switch (doext->xtag)
    {
    case X_SAVE:
      {
	INT len, i, *ip;

	len = ((long *)extdata)[0];
	outInt (len);
	if (errno) return;
	if (len<0)
	  len = -len;
	for (i=0, ip = (long *)extdata; i<len; i++)
	  {
	    outInt (*++ip);
	    if (errno) return;
	  }
	break;
      }
    case X_MARK:
    case X_MARKDATA:
      break;
    case X_FREE:
      MARKFREE ((char *)(((long *)extdata)-1));
      break;
    }
}

PRIVATE ExprPtr lipLoad (desc,shared,sharedref,load)
IN CONST extDesc *desc;
IN Boolean shared;
INOUT ExprPtr sharedref;
IN CONST loadAction load;
{
  long len, abslen;
  long *ip, *ipp, i;
  ExprPtr e;

  len = inInt ();
  if (errno) return NIL;
  abslen = (len<0)?(-len):len;
  ipp = ip = (long *)MALLOC ((abslen+2) * sizeof(long));
  *ipp++ = abslen;
  ipp[0] = len;
  for (i=0; i<abslen; i++)
    {
      *++ipp = inInt ();
      if (errno)
	{
	  MARKFREE (ip);
	  return NIL;
	}
    }
  e = MakeExternal (desc, (char *)(ip+1));
  (void) KEEP (e);
  return e;
}

PRIVATE unsigned long lipHash (extdata)
IN CONST char *extdata;
{
  REGISTER verylong n = (verylong)extdata, nend;
  REGISTER long len = *n++;
  REGISTER unsigned long hash = (unsigned long)len;

  if (len<0)
    len = -len;
  nend = n+len;
  for (; n<nend; )
    hash += (unsigned long)*n++;
  return hash;
}

PRIVATE Boolean lipEq (ext1, ext2)
IN CONST char *ext1, *ext2;
{
  REGISTER verylong n1 = (verylong)ext1, n2 = (verylong)ext2;
  REGISTER long len = *n1++;
  REGISTER verylong n1end;

  if (len!=*n2++)
    return false;
  if (len<0)
    len = -len;
  n1end = n1+len;
  for (; n1<n1end; )
    if (*n1++ != *n2++)
      return false;
  return true;
}

PRIVATE void lipPrint (ext,doprint,printdata)
IN CONST char *ext;
IN printFunc doprint;
INOUT char *printdata;
{
  (*doprint) ((CHAR *)"<Int:", 5, printdata);
  zfwrite ((verylong)ext, MAX_SIGNED_LONG, (CHAR *)"", 0, (CHAR *)"", 0, doprint, printdata);
  (*doprint) ((CHAR *)">", 1, printdata);
}

PUBLIC extDesc lipExtDesc = {
  (CHAR *)"Int",
  { 'I','n','t',' ' }, NULL,
  lipHash,
  lipEq,
  lipPrint,
  lipSweep,
  lipLoad
};

PRIVATE ExprPtr lipMake (n)
verylong n;
{
  ExprPtr e;

  e = Kexternal (&lipExtDesc, (CONST char *)n);
  return e;
}

#if LIPZAMP==1
#include "lipzamp1.h"
#else
#include "lipzamp2.h"
#endif

#define zaddmulone(ama, amb) \
{ \
	register long lami; \
	register long lams = 0; \
	register verylong lama = (ama); \
	register verylong lamb = (amb); \
 \
	for (lami = (*lamb++); lami > 0; lami--) \
	{ \
		lams += (*lama + *lamb++); \
		*lama++ = lams & RADIXM; \
		lams >>= NBITS; \
	} \
	*lama += lams; \
}

#define zaddmul(ams, ama, amb) \
{ \
	register long lami; \
	register long lams = (ams); \
	register verylong lama = (ama); \
	register verylong lamb = (amb); \
	long lamcarry = 0; \
 \
	for (lami = (*lamb++); lami > 0; lami--) \
	{ \
		zaddmulp(lama, *lamb, lams, &lamcarry); \
	/* Be careful, the last lama is unnormalized */ \
		lama++; \
		lamb++; \
	} \
        *lama += lamcarry; \
}

#define zaddmulsq(sql, sqa, sqb) \
{ \
	register long lsqi = (sql); \
	register long lsqs = *(sqb); \
	register verylong lsqa = (sqa); \
	register verylong lsqb = (sqb); \
	long lsqcarry = 0; \
 \
	lsqb++; \
	for (; lsqi > 0; lsqi--) \
	{ \
		zaddmulp(lsqa, *lsqb, lsqs, &lsqcarry); \
		lsqa++; \
		lsqb++; \
	} \
	*lsqa += lsqcarry; \
/* Be careful, the last lama is unnormalized */ \
}

#define zmmulp(mmpa, mmpb) \
{ \
	register verylong lmmpa = (mmpa); \
	register verylong lmmpb = (mmpb); \
	register long lmmi = (*lmmpa) >> NBITSH; \
	register long lmmd = (*lmmpa) & RADIXROOTM; \
	long zmmtemp = 0; \
 \
	lmmd = (zminv1 * lmmd + (((zminv1 * lmmi + zminv2 * lmmd) & RADIXROOTM) << NBITSH)) & RADIXM; \
	for (lmmi = *lmmpb++; lmmi > 0; lmmi--) \
	{ \
		zaddmulp(lmmpa, *lmmpb, lmmd, &zmmtemp); \
		lmmpa++; \
		*lmmpb++; \
	} \
	if (((*lmmpa += zmmtemp) & RADIX) > 0) \
	{ \
		(*lmmpa++) &= RADIXM; \
		(*lmmpa)++; \
	} \
}

#if 0
/*
	If you prefer you can replace the above macro's by C-routines ,
	(or better, replace them by assembly versions). Here are some
	examples, the rest follow easily.
*/

void
zaddmulp(
	verylong a,
	long b,
	long d,
	verylong t
	)
{
	register long b1 = b & RADIXROOTM;
	register long d1 = d & RADIXROOTM;
	register long aa = *a + b1 * d1;

	b1 = b1 * (d >>= NBITSH) + d1 * (b >>= NBITSH) + (aa >> NBITSH);
	aa = (aa & RADIXROOTM) + ((b1 & RADIXROOTM) << NBITSH) + *t;
	*t = d * b + (b1 >> NBITSH) + (aa >> NBITS);
	*a = (aa & RADIXM);
}

void
zaddmul(
	long d,
	verylong a,
	verylong b
	)
{
	register long i;
	long carry = 0;
	verylong carrya = &carry;

	for (i = (*b++); i > 0; i--)
		zaddmulp(a++, *b++, d, carrya);
	*a += carry;
}

void
zaddmulone(
	verylong a,
	verylong b
	)
{
	zaddmul(1, a, b);
}

void
zmmulp(
	verylong pa,
	verylong pb
	)
{
	register long i = (*pa) >> NBITSH;
	register long d = (*pa) & RADIXROOTM;
	long temp = 0;
	extern long zminv1;
	extern long zminv2;

	d = (zminv1 * d + (((zminv1 * i + zminv2 * d) & RADIXROOTM) << NBITSH)) & RADIXM;
	for (i = *pb++; i > 0; i--)
		zaddmulp(pa++, *pb++, d, &temp);
	if (((*pa += temp) & RADIX) > 0)
	{
		(*pa++) &= RADIXM;
		(*pa)++;
	}
}
#endif


/* a[s:] -= d*b, optimize this one, a and b not at overlapping addresses */

PRIVATE void zsubmul(r, a, b)
long r;
verylong a;
verylong b;
{
	register long rd = RADIX - r;
	register long i;
	long carry = RADIX;

	for (i = (*b++); i > 0; i--)
	{
		zaddmulp(a, *b, rd, &carry);
		a++;
		carry += RADIXM - (*b++);
	}
	*a += carry - RADIX;	/* unnormalized */
}

/*
	zdiv21 returns quot, rem so

	quot = (numhigh*RADIX + numlow)/denom;
	rem  = (numhigh*RADIX + numlow)%denom;

Assumes 0 <= numhigh < denom < RADIX and 0 <= numlow < RADIX.
*/

#if LIPZDIV==1
#include "lipzdiv1.h"
#else
#include "lipzdiv2.h"
#endif

PUBLIC long zmulmods(a, b, n)
long a;
long b;
long n;
{
#if LIPZMM==1
#include "lipzmm1.h"
#else
#include "lipzmm2.h"
#endif
	return (lr);
}

PRIVATE ExprPtr raiseLip (COUNTER n)
{
  ExprPtr arg, e;

  arg = KINT (n);
  e = MakePair (E_CONS, lipErrExc, arg);
  (void) KEEP (e);
  return Raise (e);
}

DEF_ACTION_3ARGS(zmulmods) /* zmsmul : int * int * int -> int */
{
  INT m = IVAL(arg3);

  if (m==0)
    return raiseLip (LIP_ZERO_MOD);
  return Kint (zmulmods (IVAL(arg1), IVAL(arg2), m));
}
ACTION_END

/* global variables */

/* for long division */
PRIVATE EXTENDED epsilon;
PRIVATE EXTENDED fradix = (EXTENDED)RADIX, fudge = -1.0;
/* for random generator */
PRIVATE verylong zseed = 0;
PRIVATE verylong zranp = 0;
PRIVATE verylong zprroot = 0;
/* for Montgomery multiplication */
PRIVATE verylong zn = 0;
PRIVATE verylong zoldzn = 0;
PRIVATE verylong zr = 0;
PRIVATE verylong zrr = 0;
PRIVATE verylong zrrr = 0;
PRIVATE verylong znm = 0;
PRIVATE long znotinternal = 0;
PRIVATE long zntop;
PRIVATE long zminv1;
PRIVATE long zminv2;
/* for small prime generation */
/*PRIVATE struct smallprimegen *spg = 0;*/
/* for convenience */
PRIVATE long oner[] = {1, 1, 1};
PRIVATE long glosho[] = {1, 1, 0};
PRIVATE verylong bigone = &oner[1];
/* for m_ary exponentiation */
PRIVATE verylong **exp_odd_powers = 0;
/* for karatsuba */
PRIVATE verylong kar_mem[5*MAXDEPTH];
PRIVATE long kar_mem_initialized = 0;

#if 0
EXTENDED 
getutime()
{
	struct rusage used;

#ifndef __hpux
	getrusage(RUSAGE_SELF, &used);
#else	/* maybe delete in HPUX 9. */
#include <syscall.h>
	syscall(SYS_getrusage, RUSAGE_SELF, &used);
#endif
#if defined(sparc) && defined(__svr4__)
	return (used.ru_utime.tv_sec + (used.ru_utime.tv_nsec) / 1e9);
#else
	return (used.ru_utime.tv_sec + (used.ru_utime.tv_usec) / 1e6);
#endif /* SOLARIS */
}

EXTENDED 
getstime()
{
	struct rusage used;

#ifndef __hpux
	getrusage(RUSAGE_SELF, &used);
#else	/* maybe delete in HPUX 9. */
#include <syscall.h>
	syscall(SYS_getrusage, RUSAGE_SELF, &used);
#endif
#if defined(sparc) && defined(__svr4__)
	return (used.ru_stime.tv_sec + (used.ru_stime.tv_nsec) / 1e9);
#else
	return (used.ru_stime.tv_sec + (used.ru_stime.tv_usec) / 1e6);
#endif /* SOLARIS */
}

EXTENDED 
gettime()
{
	struct rusage used;

#ifndef __hpux
	getrusage(RUSAGE_SELF, &used);
#else	/* maybe delete in HPUX 9. */
#include <syscall.h>
	syscall(SYS_getrusage, RUSAGE_SELF, &used);
#endif
	return (used.ru_utime.tv_sec + used.ru_stime.tv_sec +
#if defined(sparc) && defined(__svr4__)
		(used.ru_utime.tv_nsec + used.ru_stime.tv_nsec) / 1e9);
#else
		(used.ru_utime.tv_usec + used.ru_stime.tv_usec) / 1e6);
#endif /* SOLARIS */
}

PRIVATE void
hidetimer(
	FILE *f,
	long what
	)
{
	PRIVATE EXTENDED keep_time;
	if (what)
	{
		fprintf(f,"%8.5lf sec.\n",gettime()-keep_time);
		fflush(f);
	}
	else
		keep_time = gettime();
}

void
starttime()
{
	hidetimer(stderr,0);
}

void
printtime(
	FILE *f
	)
{
	hidetimer(f,1);
}

PRIVATE void
zhalt(
        char *c
	)
{
#ifdef NOHALT
	fprintf(stderr,"error:\n   %s\ncontinue...\n",c);
	fflush(stderr);
#else
	fprintf(stderr,"fatal error:\n   %s\nexit...\n",c);
	fflush(stderr);
	(void)exit((int)0);
#endif
}
#endif

PUBLIC void zsetlength(v, len)
verylong *v;
long len;
{
  verylong x = *v, xx;
  size_t size;

  if (x)
    {
      if (len <= x[-1])
	return;
      x[-1] = len;
      size = (size_t)(len + 2) * SIZEOFLONG;
      xx = (verylong)REALLOC(((void *)(&x[-1])), size);
      if (xx==NULL)
	{
	  xx = (verylong)MALLOC(size);
	  memcpy (xx, x, (size_t)(x[-1] + 2) * SIZEOFLONG);
	  MARKFREE(((void *)(&x[-1])));
	}
      x = xx;
    }
  else /*if (len >= 0)*/
    {
      if (len < SIZE)
	len = SIZE;
      size = (size_t)(len + 2) * SIZEOFLONG;
      x = (verylong)MALLOC(size);
      x[0] = len;
      x[1] = 1;
      x[2] = 0;
    }
  /*
  else
    zhalt("negative size allocation in zsetlength");
    */
  *v = x+1;
}

PUBLIC void zfree (x)
verylong *x;
{
  if (!(*x))
    return;
  {
    verylong y = (*x - 1);
    MARKFREE(((void*)y));
    *x = 0;
  }
}

PUBLIC EXTENDED zdoub(n)
verylong n;
{
  EXTENDED res;
  register long i;

  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    return ((EXTENDED) 0);
  if ((i = n[0]) < 0)
    i = -i;
  res = (EXTENDED) (n[i--]);
  for (; i; i--)
    res = res * fradix + (EXTENDED) (n[i]);
  if (n[0] > 0)
    return (res);
  return (-res);
}

DEF_ACTION_1ARG(zdoub) /* znum : Int -> num */
{
  EXTENDED x = zdoub ((verylong)EXTDATA(arg));
  return Kext (&x);
}
ACTION_END

PUBLIC void zstart ()
{
  EXTENDED local_one = (EXTENDED) 1;
  EXTENDED local_half = 1 / (EXTENDED) 2;
  epsilon = local_one;
  fudge = local_one + epsilon;

 /* the following loop is sick, but we have to: */
 /* comparing one to one + epsilon does not have */
 /* the desired effect on some machines, because */
 /* one + epsilon is Not cast to EXTENDED (as it should) */
  while (local_one != fudge)
    {
      epsilon = epsilon * local_half;
      fudge = local_one + epsilon;
    }
  epsilon += epsilon;
  /* The following should not happen (see mksys.c):
  if ((epsilon * RADIX) > local_one)
    {
      zhalt("recompile with smaller NBITS");
    }
  else
    */
      epsilon *= 3;
  fudge = fradix + epsilon * fradix;
}

PUBLIC void z_zero(aa)
verylong *aa;
{
  if (!(*aa))
    zsetlength(aa, (long)SIZE);
  (*aa)[0] = (long) 1;
  (*aa)[1] = (long) 0;
}

PUBLIC void z_one(aa)
verylong *aa;
{
  if (!(*aa))
    zsetlength(aa, (long)SIZE);
  (*aa)[0] = (long) 1;
  (*aa)[1] = (long) 1;
}

PUBLIC void zcopy(a,bb)
verylong a;
verylong *bb;
{
  register long i;
  verylong b = *bb;

  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if (a != b)
    {
      if ((i = *a) < 0)
	i = (-i);
      zsetlength(&b, i);
      *bb = b;
      for (; i >= 0; i--)
	*b++ = *a++;
    }
}

PUBLIC void zintoz(d,aa)
long d;
verylong *aa;
{
  register long i = 0;
  register long anegative = 0;
  verylong a = *aa;

  if (!(*aa))
    zsetlength(&a, (long)SIZE);
  *aa = a;
  if (d < 0)
    {
      anegative = 1;
      d = -d;
    }
  a[1] = 0;
  while (d > 0)
    {
      a[++i] = d & RADIXM;
      d >>= NBITS;
    }
  if (i > 0)
    a[0] = i;
  else
    a[0] = 1;
  if (anegative)
    a[0] = (-a[0]);
}

DEF_ACTION_1ARG(zintoz) /* Int : int -> Int */
{
  verylong n = 0;

  zintoz (IVAL(arg), &n);
  return lipMake (n);
}
ACTION_END

PUBLIC long ztoint(a)
verylong a;
{
  register long d;
  register long sa;

  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    return (0);
  if ((sa = *a) < 0)
    sa = -sa;
  d = *(a += sa);
  while (--sa)
    {
      d <<= NBITS;
      d += *(--a);
    }
  if ((*(--a)) < 0)
    return (-d);
  return (d);
}

DEF_ACTION_1ARG(ztoint) /* zint : Int -> int */
{
  verylong a = (verylong)EXTDATA(arg);
  return Kint (ztoint (a));
}
ACTION_END

PUBLIC void zsbastoz(base,row,n)
long base;
ExprPtr row;
verylong *n;
{
  if (row==NIL)
    {
      z_zero (n);
      return;
    }
  zintoz (IVAL(CAR(row)), n);
  for (;;)
    {
      row = CDR(row);
      if (row==NIL)
	break;
      zsmul(*n, base, n);
      zsadd(*n, IVAL(CAR(row)), n);
    }
}

DEF_ACTION_2ARGS(zsbastoz) /* zsbastoz : int * int list -> Int */
{
  verylong n = 0;

  zsbastoz (IVAL(arg1), arg2, &n);
  return lipMake (n);
}
ACTION_END

PUBLIC void zbastoz(base,row,n)
verylong base;
ExprPtr row;
verylong *n;
{
  if (row==NIL)
    {
      z_zero (n);
      return;
    }
  zcopy ((verylong)EXTDATA(CAR(row)), n);
  for (;;)
    {
      row = CDR(row);
      if (row==NIL)
	break;
      zmulin(base, n);
      zadd(*n, (verylong)EXTDATA(CAR(row)), n);
    }
}

DEF_ACTION_2ARGS(zbastoz) /* zbastoz : Int * Int list -> Int */
{
  verylong n = 0;

  zbastoz ((verylong)EXTDATA(arg1), arg2, &n);
  return lipMake (n);
}
ACTION_END

PUBLIC ExprPtr zstobas(n,base)
verylong n;
long base;
{
  /* n >= 0, base > 1 */
  register ExprPtr row;
  static verylong q = 0;
  static verylong nn = 0;
  long dig;
  ENTER;

  row = NIL;
  zcopy(n, &nn);
  zabs(&nn);
  for (;;)
    {
      if ((nn[1] == 0) && (nn[0] == 1))
	{
	  QUIT;
	  return row;
	}
      dig = zsdiv(nn, base, &q);
      row = Kons (KINT (dig), row);
      KEEPONLY(row);
      zswap(&q, &nn);
    }
}

DEF_ACTION_2ARGS(zstobas) /* zstobas : int * Int -> int list */
{
  long base = IVAL(arg1);

  if (base < 0)
    base = -base;
  if (base<2)
    return raiseLip (LIP_WRONG_BASE);
  return zstobas ((verylong)EXTDATA(arg2), base);
}
ACTION_END

PUBLIC ExprPtr zstosymbas(n,base)
verylong n;
long base;
{
  register ExprPtr row;
  register long hbase;
  static verylong q = 0;
  static verylong nn = 0;
  long dig;
  ENTER;

  hbase = (base >> 1);
  row = NIL;
  zcopy(n, &nn);
  zabs(&nn);
  for (;;)
    {
      if ((nn[1] == 0) && (nn[0] == 1))
	{
	  QUIT;
	  return row;
	}
      dig = zsdiv(nn, base, &q);
      if (dig > hbase)
	{
	  dig -= base;
	  zsadd(q, 1, &q);
	}
      row = Kons (KINT (dig), row);
      KEEPONLY(row);
      zswap(&q, &nn);
    }
}

DEF_ACTION_2ARGS(zstosymbas) /* zstosymbas : int * Int -> int list */
{
  long base = IVAL(arg1);

  if (base < 0)
    base = -base;
  if (base<2)
    return raiseLip (LIP_WRONG_BASE);
  return zstosymbas ((verylong)EXTDATA(arg2), base);
}
ACTION_END

PUBLIC ExprPtr ztobas(n,base)
verylong n;
verylong base;
{
 /* n >= 0, base > 1 */
  register ExprPtr row;
  static verylong q = 0;
  static verylong nn = 0;
  static verylong lbase = 0;
  verylong dig;
  ExprPtr longdig;
  ENTER;

  zcopy(base, &lbase);
  zabs(&lbase);
  if (zscompare(base, 1) <= 0)
    return NOEXPR;
  row = NIL;
  zcopy(n, &nn);
  zabs(&nn);
  for (;;)
    {
      if ((nn[1] == 0) && (nn[0] == 1))
	{
	  QUIT;
	  return row;
	}
      dig = 0;
      zquot(nn, lbase, &q, &dig);
      longdig = lipMake (dig);
      (void) KEEP (longdig);
      row = Kons (longdig, row);
      KEEPONLY(row);
      zswap(&q, &nn);
    }
}

DEF_ACTION_2ARGS(ztobas) /* ztobas : Int * Int -> Int list */
{
  verylong base = (verylong)EXTDATA(arg1);
  ExprPtr e;

  e = ztobas ((verylong)EXTDATA(arg2),base);
  if (e==NOEXPR)
    return raiseLip (LIP_WRONG_BASE);
  return e;
}
ACTION_END

PUBLIC ExprPtr ztosymbas(n,base)
verylong n;
verylong base;
{
  /* n >= 0, base > 1 */
  register ExprPtr row;
  static verylong q = 0;
  static verylong nn = 0;
  static verylong lbase = 0;
  static verylong hbase = 0;
  verylong dig;
  ExprPtr longdig;
  ENTER;

  zcopy(base, &lbase);
  zabs(&lbase);
  if (zscompare(base, 1) <= 0)
    return NOEXPR;
  z2div(lbase, &hbase);
  row = NIL;
  zcopy(n, &nn);
  zabs(&nn);
  for (;;)
    {
      if ((nn[1] == 0) && (nn[0] == 1))
	{
	  QUIT;
	  return row;
	}
      dig = 0;
      zquot(nn, lbase, &q, &dig);
      if (zcompare(dig, hbase) > 0)
	{
	  zsub(dig, lbase, &dig);
	  zsadd(q, 1, &q);
	}
      longdig = lipMake (dig);
      (void) KEEP (longdig);
      row = Kons (longdig, row);
      KEEPONLY(row);
      zswap(&q, &nn);
    }
}

DEF_ACTION_2ARGS(ztosymbas) /* ztosymbas : Int * Int -> Int list */
{
  verylong base = (verylong)EXTDATA(arg1);
  ExprPtr e;

  e = ztosymbas ((verylong)EXTDATA(arg2),base);
  if (e==NOEXPR)
    return raiseLip (LIP_WRONG_BASE);
  return e;
}
ACTION_END

PUBLIC long zcompare(a,b)
verylong a;
verylong b;
{
  register long sa;
  register long sb;

  if (ALLOCATE && !a)
    {
      if (!b)
	return (0);
      if (b[0] < 0)
	return (1);
      if (b[0] > 1)
	return (-1);
      if (b[1])
	return (-1);
      return (0);
    }
  if (ALLOCATE && !b)
    {
      if (a[0] < 0)
	return (-1);
      if (a[0] > 1)
	return (1);
      if (a[1])
	return (1);
      return (0);
    }
  if (a==b)
    return (0);
  if ((sa = *a) > (sb = *b))
    return (1);
  if (sa < sb)
    return (-1);
  if (sa < 0)
    sa = (-sa);
  a += sa;
  b += sa;
  for (; sa; sa--)
    {
      if (*a > *b)
	{
	  if (sb < 0)
	    return (-1);
	  return (1);
	}
      if (*a-- < *b--)
	{
	  if (sb < 0)
	    return (1);
	  return (-1);
	}
      /*
       * depending on the relative speed of various operations it might be
       * better to do the following: if ((aux=(*a--)-(*b--))>0) { if (sb<0)
       * return (-1); return (1); } else if (aux<0) { if (sb<0) return (1);
       * return (-1); } where aux is a register long
       */
    }
  return (0);
}

DEF_ACTION_2ARGS(zcompare) /* zcompare : Int * Int -> int */
{
  return Kint (zcompare ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2)));
}
ACTION_END

PUBLIC void znegate (aa)
verylong *aa;
{
  verylong a = *aa;

  if (!a)
    return;
  if (a[0]!=1 || a[1]!=0)
    a[0] = (-a[0]);
}

DEF_ACTION_1ARG(znegate) /* zneg : Int -> Int */
{
  verylong a = (verylong)EXTDATA(arg);
  verylong aa = 0;

  zcopy (a, &aa);
  znegate (&aa);
  return lipMake (aa);
}
ACTION_END

PUBLIC void zsadd(a,d,b)
verylong a;
long d;
verylong *b;
{
  static verylong x = 0;

  zintoz(d, &x);
  zadd(a, x, b);
}

DEF_ACTION_2ARGS(zsadd) /* zsadd : Int * int -> Int */
{
  verylong b = 0;

  zsadd ((verylong)EXTDATA(arg1), IVAL(arg2), &b);
  return lipMake (b);
}
ACTION_END

PRIVATE void zaddls(a,b,c)
verylong a;
verylong b;
verylong c;
{
  /* low level c = a+b, b shorter than a, output can be input */
  register long sa = *a;
  register long sb = *b;
  register long carry = 0;
  register long i;
  register verylong pc;

  /* we know that sa and sb are both >0 or both <0 */
  pc = &c[0];
  if (sa < 0)
    {
      sa = -sa;
      sb = -sb;
    }
  for (i = 1; i <= sb; i++)
    {
      if ((*(++pc) = (*(++a)) + (*(++b)) + carry) < RADIX)
	carry = 0;
      else
	{
	  *pc -= RADIX;
	  carry = 1;
	}
    }
  for (; i <= sa; i++)
    {
      if ((*(++pc) = (*(++a)) + carry) < RADIX)
	carry = 0;
      else
	{
	  *pc -= RADIX;
	  carry = 1;
	}
    }
  if (carry)
    {
      c[0] = sa + 1;
      *(++pc) = 1;
    }
  else
    c[0] = sa;
}

PUBLIC void zsubpos(a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  register long sa = a[0];
  register long sb = b[0];
  register long carry = 0;
  register long i;
  register verylong pc;
  verylong c = *cc;

  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      if (a)
	zcopy(a, cc);
      else
	z_zero(cc);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(cc);
      return;
    }
  zsetlength(&c, sa);
  /* if *cc == a, then nothing will happen */
  /* if *cc == b, then b might point to freed space, so */
  if (b == *cc) b = c;
  *cc = c;
  pc = &c[0];
  for (i = 1; i <= sb; i++)
    {
      if ((*(++pc) = (*(++a)) - (*(++b)) - carry) >= 0)
	carry = 0;
      else
	{
	  *pc += RADIX;
	  carry = 1;
	};
    }
  for (; i <= sa; i++)
    {
      if ((*(++pc) = (*(++a)) - carry) >= 0)
	carry = 0;
      else
	{
	  *pc += RADIX;
	  carry = 1;
	};
    }
  i = sa;
  while ((i > 1) && (!(*pc)))
    {
      i--;
      pc--;
    }
  c[0] = i;
}

PUBLIC void zadd(a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  register long sa;
  register long sb;
  register long anegative;
  verylong c = *cc;

  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      if (b)
	zcopy(b, cc);
      else
	z_zero(cc);
      return;
    }
  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      zcopy(a, cc);
      return;
    }
  if ((anegative = ((sa = a[0]) < 0)) == ((sb = b[0]) < 0))
    {
      /* signs a and b are the same */
      register long i;
      if (anegative)
	{
	  sa = -sa;
	  sb = -sb;
	}
      zsetlength(&c, (sa > sb ? sa : sb) + 1);
      if (a == *cc) a = c;
      if (b == *cc) b = c;
      *cc = c;
      if (sa == sb)
	{
	  register verylong pc;
	  pc = &c[0];
	  i = 0;
	  for (; sa; sa--)
	    {
	      if ((*(++pc) = (*(++a)) + (*(++b)) + i) < RADIX)
		i = 0;
	      else
		{
		  *pc -= RADIX;
		  i = 1;
		}
	    }
	  if (i)
	    {
	      c[0] = sb + 1;
	      *(++pc) = 1;
	    }
	  else
	    c[0] = sb;
	}
      else if (sa > sb)
	zaddls(a, b, c);
      else
	zaddls(b, a, c);
      if (anegative)
	c[0] = -c[0];
      /* if anegative, then c cannot be zero */
    }
  else
    {
      /* signs a and b are different */
      verylong old;
      verylong oldc;

      oldc = c;
      if (anegative)
	{
	  a[0] = -a[0];
	  old = a;
	}
      else
	{
	  b[0] = -b[0];
	  old = b;
	}
      /* the one that's negative cannot be zero */
      if (!(sa = zcompare(a, b)))
	{
	  z_zero(&c);
	  *cc = c;
	  if (anegative)
	    {
	      if (old != oldc)
		a[0] = -a[0];
	    }
	  else if (old != oldc)
	    b[0] = -b[0];
	}
      else if (sa > 0)
	{
	  zsubpos(a, b, &c);
	  *cc = c;
	  if (anegative)
	    {
	      c[0] = -c[0];
	      if (old != oldc)
		a[0] = -a[0];
	    }
	  else if (old != oldc)
	    b[0] = -b[0];
	}
      else
	{
	  zsubpos(b, a, &c);
	  *cc = c;
	  if (!anegative)
	    {
	      c[0] = -c[0];
	      if (old != oldc)
		b[0] = -b[0];
	    }
	  else if (old != oldc)
	    a[0] = -a[0];
	}
    }
}

DEF_ACTION_2ARGS(zadd) /* zadd : Int * Int -> Int */
{
  verylong c = 0;

  zadd ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake (c);
}
ACTION_END

PUBLIC void zsub(a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  register long sa;
  register long sb;
  register long anegative;
  verylong c = *cc;

  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      if (a)
	zcopy(a, cc);
      else
	z_zero(cc);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      zcopy(b, cc);
      znegate(cc);
      return;
    }
  if ((anegative = ((sa = a[0]) < 0)) == ((sb = b[0]) < 0))
    {
      /* signs a and b are the same */
      register long carry = 0;
      register long i;
      register verylong pc;
      register long agrb;

      if (!(agrb = zcompare(a, b)))
	{
	  z_zero(cc);
	  return;
	}
      if ((agrb > 0 && anegative) || (agrb < 0 && !anegative))
	{
	  pc = a;
	  a = b;
	  b = pc;
	  sa = *a;
	  sb = *b;
	}
      if (anegative)
	{
	  sa = -sa;
	  sb = -sb;
	}
      zsetlength(&c, sa);
      if (b == *cc) b = c;
      *cc = c;
      pc = &c[0];
      for (i = 1; i <= sb; i++)
	{
	  if ((*(++pc) = (*(++a)) - (*(++b)) - carry) >= 0)
	    carry = 0;
	  else
	    {
	      *pc += RADIX;
	      carry = 1;
	    };
	}
      for (; i <= sa; i++)
	{
	  if ((*(++pc) = (*(++a)) - carry) >= 0)
	    carry = 0;
	  else
	    {
	      *pc += RADIX;
	      carry = 1;
	    };
	}
      i = sa;
      while ((i > 1) && (!(*pc)))
	{
	  i--;
	  pc--;
	}
      if (agrb > 0)
	c[0] = i;
      else
	c[0] = -i;
    }
  else
    {
      /* signs a and b are different */
      verylong old;
      verylong oldc;

      oldc = c;
      if (anegative)
	{
	  a[0] = -a[0];
	  old = a;
	}
      else
	{
	  b[0] = -b[0];
	  old = b;
	}
      /* the one that's negative cannot be zero */
      zadd(a, b, &c);
      *cc = c;
      if (anegative)
	{
	  c[0] = -c[0];
	  if (old != oldc)
	    a[0] = -a[0];
	}
      else if (old != oldc)
	b[0] = -b[0];
    }
}

DEF_ACTION_2ARGS(zsub) /* zsub : Int * Int -> Int */
{
  verylong c = 0;

  zsub ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake (c);
}
ACTION_END

PUBLIC void zsmul(a,d,bb)
verylong a;
long d;
verylong *bb;
{
  register long sa;
  register long i;
  register verylong pb;
  register long bnegative = 0;
  verylong b = *bb;
  static verylong x = 0;

  if (( d >= RADIX) || (-d >= RADIX)) {
    zintoz(d,&x);
    zmulin(a,&x);
    zcopy(x,bb);
    return;
  }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if (ALLOCATE && !d)
    {
      z_zero(bb);
      return;
    }
  if ((sa = a[0]) < 0)
    {
      sa = (-sa);
      if (d < 0)
	d = (-d);
      else
	bnegative = 1;
    }
  else if ((bnegative = (d < 0)))
    d = (-d);
  zsetlength(&b, sa + 1);
  if (a == *bb) a = b;
  *bb = b;
  pb = &b[0];
  for (i = sa; i >= 0; i--)
    *pb++ = *a++;
  b[0] = sa;
  sa++;
  *pb = 0;
  zaddmul(d - 1, &b[1], &b[0]);
  while ((sa > 1) && (!(b[sa])))
    sa--;
  b[0] = sa;
  if (bnegative && (b[1] || b[0] != 1))
    b[0] = (-b[0]);
}

DEF_ACTION_2ARGS(zsmul) /* zsmul : Int * int -> Int */
{
  verylong b = 0;

  zsmul ((verylong)EXTDATA(arg1), IVAL(arg2), &b);
  return lipMake (b);
}
ACTION_END

PRIVATE void kar_mul(a,b,c,shi)
verylong a;
verylong b;
verylong *c;
long shi;
{
  register long al;
  register long hal;
  register long i;
  register long restoreb0 = b[0];
  register verylong pc;
  register long bbig = 1;
  verylong *a0;
  verylong *a1;
  verylong *a2;
  verylong *a3;
  verylong *a4;

  zsetlength(c, (hal = (al = a[0]) + (i = b[0])));
  if ((shi >= (5 * MAXDEPTH)) || (al < MULT_CROSS) || (i < MULT_CROSS))
    {
      pc = &(*c)[1];
      for (i = hal; i > 0; i--)
	*pc++ = 0;
      pc = &(*c)[1];
      if (al <= *b)
	for (i = al; i; i--)
	  {
	    zaddmul(*(++a), pc++, b);
	  }
      else
	for (i = *b; i; i--)
	  {
	    zaddmul(*(++b), pc++, a);
	  }
      while ((hal > 1) && (!((*c)[hal])))
	hal--;
      (*c)[0] = hal;
      return;
    }
  a0 = &(kar_mem[shi]);
  a1 = &(kar_mem[shi + 1]);
  a2 = &(kar_mem[shi + 2]);
  a3 = &(kar_mem[shi + 3]);
  a4 = &(kar_mem[shi + 4]);
  hal = ((al + 1) >> 1);
  zsetlength(a0, al);
  zsetlength(a1, al);
  zsetlength(a2, al);
  zsetlength(a3, al + hal);
  zsetlength(a4, al + 2);
  i = hal;
  while ((i > 1) && (!(a[i])))
    i--;
  a[0] = i;
  if (hal >= b[0])
    bbig = 0;
  else
    {
      i = hal;
      while ((i > 1) && (!(b[i])))
	i--;
      b[0] = i;
    }
  for (i = hal + 1; i <= al; i++)
    (*a1)[i - hal] = a[i];
  (*a1)[0] = al - hal;
  if (bbig)
    {
      for (i = hal + 1; i <= restoreb0; i++)
	(*a3)[i - hal] = b[i];
      (*a3)[0] = restoreb0 - hal;
    }
  kar_mul(a, b, a4, shi + 5);
  zadd(a, (*a1), a0);
  a[0] = al;
  if (bbig)
    {
      kar_mul((*a1), (*a3), c, shi + 5);
      zadd(b, (*a3), a2);
      b[0] = restoreb0;
      kar_mul((*a0), (*a2), a3, shi + 5);
    }
  else
    kar_mul((*a0), b, a3, shi + 5);
  zsubpos((*a3), (*a4), a3);
  if (bbig)
    zsubpos((*a3), *c, a3);
  zlshift((*a3), hal * NBITS, a3);
  hal <<= 1;
  if (bbig)
    {
      for (i = (*c)[0]; i; i--)
	(*c)[i + hal] = (*c)[i];
      for (i = hal; i > (*a4)[0]; i--)
	(*c)[i] = 0;
      for (; i; i--)
	(*c)[i] = (*a4)[i];
      (*c)[0] += hal;
    }
  else
    {
      for (i = (*a4)[0]; i >= 0; i--)
	(*c)[i] = (*a4)[i];
    }
  zadd(*c, (*a3), c);
}

PUBLIC void ztimes(a,b,c)
verylong a;
verylong b;
verylong *c;
{	/* output not input */
  register long aneg;
  register long bneg;
  verylong olda;
  verylong oldb;

  if (ALLOCATE && (!a || !b || (a[0]==1 && !a[1]) || (b[0]==1 && !b[1])))
    {
      z_zero(c);
      return;
    }
  if (a == b)
    {
      zsq(a, c);
      return;
    }
  if (!kar_mem_initialized)
    {
      kar_mem_initialized = 1;
      for (aneg = (5 * MAXDEPTH) - 1; aneg >= 0; aneg--)
	kar_mem[aneg] = (verylong) 0;
    }
  olda = a;
  oldb = b;
  if ((aneg = (*a < 0)))
    a[0] = -a[0];
  if ((bneg = (*b < 0)))
    b[0] = -b[0];
  if (*a > *b)
    kar_mul(a, b, c, (long) 0);
  else
    kar_mul(b, a, c, (long) 0);
  if (aneg != bneg && ((*c)[1] || (*c)[0] != 1))
    (*c)[0] = -(*c)[0];
  if (aneg)
    olda[0] = -olda[0];
  if (bneg)
    oldb[0] = -oldb[0];
}

DEF_ACTION_2ARGS(zmul) /* zmul : Int * Int -> Int */
{
  verylong c = 0;

  ztimes ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake (c);
}
ACTION_END

PRIVATE void kar_sq(a,c,shi)
verylong a;
verylong *c;
long shi;
{
  register long al;
  register long hal;
  register long i;
  register verylong pc;
  verylong *a0;
  verylong *a1;
  verylong *a2;

  zsetlength(c, (i = ((al = a[0]) << 1)));
  if ((shi >= (3 * MAXDEPTH)) || (al < SQUARE_CROSS))
    {
      register unsigned long uncar;
      long carry = 0;
      pc = &(*c)[1];
      for (; i > 0; i--)
	*pc++ = 0;
      for (hal = 1; hal <= al; hal++)
	{
	  i += 2;
	  {
	    zaddmulsq(al - hal, &((*c)[i]), &(a[hal]));
	  }
	  uncar = ((*c)[i - 1] << 1) + carry;
	  (*c)[i - 1] = uncar & RADIXM;
	  uncar = ((*c)[i] << 1) + (uncar >> NBITS);
	  {
	    zaddmulpsq(&(*c)[i - 1], a[hal], &carry);
	  }
	  uncar += carry;
	  carry = uncar >> NBITS;
	  (*c)[i] = uncar & RADIXM;
	}
      while ((i > 1) && (!((*c)[i])))
	i--;
      (*c)[0] = i;
      return;
    }
  a0 = &(kar_mem[shi]);
  a1 = &(kar_mem[shi + 1]);
  a2 = &(kar_mem[shi + 2]);
  hal = ((al + 1) >> 1);
  zsetlength(a0, al + hal + 2);
  zsetlength(a1, al + 2);
  zsetlength(a2, al);
  i = hal;
  while ((i > 1) && (!(a[i])))
    i--;
  a[0] = i;
  for (i = hal + 1; i <= al; i++)
    (*a0)[i - hal] = a[i];
  (*a0)[0] = al - hal;
  kar_sq(a, a1, shi + 3);
  zadd(a, (*a0), a2);
  kar_sq((*a0), c, shi + 3);
  a[0] = al;
  kar_sq((*a2), a0, shi + 3);
  zsubpos((*a0), (*a1), a0);
  zsubpos((*a0), *c, a0);
  zlshift((*a0), hal * NBITS, a0);
  hal <<= 1;
  for (i = (*c)[0]; i; i--)
    (*c)[i + hal] = (*c)[i];
  for (i = hal; i > (*a1)[0]; i--)
    (*c)[i] = 0;
  for (; i; i--)
    (*c)[i] = (*a1)[i];
  (*c)[0] += hal;
  zadd(*c, (*a0), c);
}

PUBLIC void zsq(a,c)
verylong a;
verylong *c;
{	/* output is not input */
  register long aneg;

  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(c);
      return;
    }
  if (!kar_mem_initialized)
    {
      kar_mem_initialized = 1;
      for (aneg = (5 * MAXDEPTH) - 1; aneg >= 0; aneg--)
	kar_mem[aneg] = (verylong) 0;
    }
  if ((aneg = (*a < 0)))
    a[0] = -a[0];
  kar_sq(a, c, (long) 0);
  if (aneg)
    a[0] = -a[0];
}

DEF_ACTION_1ARG(zsqr) /* zsqr : Int -> Int */
{
  verylong b = 0;

  zsq ((verylong)EXTDATA(arg), &b);
  return lipMake (b);
}
ACTION_END

PUBLIC void zmulin(b,a)
verylong b;
verylong *a;
{
  static verylong mem = 0;

  if (ALLOCATE && (!*a || !b || ((*a)[0]==1 && !(*a)[1]) || (b[0]==1 && !b[1])))
    {
      z_zero(a);
      return;
    }
  zcopy(*a, &mem);
  ztimes(mem, b, a);
}

PUBLIC void zsqin(a)
verylong *a;
{
  static verylong mem = 0;

  if (ALLOCATE && (!*a || ((*a)[0]==1 && !(*a)[1])))
    {
      z_zero(a);
      return;
    }
  zcopy(*a, &mem);
  zsq(mem, a);
}

PUBLIC long zsdiv(a,d,bb)
verylong a;
long d;
verylong *bb;
{
  register long sa;
  verylong b = *bb;

#ifndef START
  if (fudge < 0)
    zstart();
#endif
  liperr = 0;
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return (0);
    }
  if (!d)
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zsdiv");*/
      return (0);
    }
  if ((sa = a[0]) < 0)
    sa = (-sa);
  zsetlength(&b, sa);
  if (a == *bb) a = b;
  *bb = b;
  if ((d >= RADIX) || (d <= -RADIX))
    {
      static verylong zd = 0;
      static verylong zb = 0;

      zintoz(d, &zb);
      zquot(a, zb, &b, &zd);
      *bb = b;
      return (ztoint(zd));
    }
  else
    {
      register long den = d;
      register long carry = 0;
      register long i;
      long flag = (*a < 0 ? 2 : 0) | (den < 0 ? 1 : 0);

      if (den < 0)
	den = -den;
      if (a[sa] < den && sa > 1)
	carry = a[sa--];
      if (den < RADIXROOT)
	{
	  register long rdivd = RADIX / den;
	  register long rmodd = RADIX - den * rdivd;
	  register long temp;

	  for (i = sa; i; i--)
	    {
	      temp = a[i] + rmodd * carry;
	      b[i] = rdivd * carry + temp / den;
	      carry = temp % den;
	    }
	}
      else
	{
	  for (i = sa; i; i--)
	    {
	      long newcarry;

	      zdiv21(carry, a[i], den, &b[i], &newcarry);
	      carry = newcarry;
	    }
	}
      while ((sa > 1) && (!(b[sa])))
	sa--;
      b[0] = sa;
      if (flag)
	{
	  if (flag <= 2)
	    {
	      if (!carry)
		znegate(&b);
	      else
		{
		  zadd(b, bigone, &b);
		  b[0] = -b[0];
		  if (flag == 1)
		    carry = carry - den;
		  else
		    carry = den - carry;
		  *bb = b;
		}
	    }
	  else
	    carry = -carry;
	}
      return (carry);
    }
}

DEF_ACTION_2ARGS(zsdiv) /* zsdivmod : Int * int -> Int * int */
{
  verylong q = 0;
  long r;

  r = zsdiv ((verylong)EXTDATA(arg1), IVAL(arg2), &q);
  if (liperr!=0)
    {
      if (q) /* added jgl 08 mar 2021 */
	{ MARKFREE (&q[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  {
    ExprPtr qq, rr;
    ENTER;

    qq = lipMake (q);
    (void) KEEP (qq);
    rr = KINT (r);
    qq = Ktuple2 (qq, rr);
    QUIT;
    return qq;
  }
}
ACTION_END

PUBLIC long zsmod(a,b)
verylong a;
long b;
{
  static verylong q = 0;

  return (zsdiv(a, b, &q));
}

DEF_ACTION_2ARGS(zsmod) /* zsmod : Int * int -> int */
{
  long r;

  r = zsmod ((verylong)EXTDATA(arg1), IVAL(arg2));
  if (liperr!=0)
    return raiseLip (liperr);
  return Kint (r);
}
ACTION_END

PUBLIC void zquot(in_a,in_b,qqq,rrr)
verylong in_a;
verylong in_b;
verylong *qqq;
verylong *rrr;
{
  register long i;
  register long qq;
  long sa;
  long sb;
  long sq;
  verylong p;
  verylong pc;
  long sign;
  static verylong a = 0;
  static verylong b = 0;
  static verylong c = 0;
  static verylong d = 0;
  EXTENDED btopinv;
  EXTENDED aux;
  verylong q = *qqq;
  verylong r = *rrr;

#ifndef START
  if (fudge < 0)
    zstart();
#endif
  liperr = 0;
  if (ALLOCATE && (!in_a || (in_a[0]==1 && !in_a[1])))
    {
      z_zero(qqq);
      z_zero(rrr);
      return;
    }
  if ((!in_b) || (((sb = in_b[0]) == 1) && (!in_b[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zquot");*/
      return;
    }
  zcopy(in_a,&a);
  zcopy(in_b,&b);
  sign = (*a < 0 ? 2 : 0) | (*b < 0 ? 1 : 0);
  if (*a < 0)
    (*a) = (-(*a));
  sa = (*a);
  if (*b < 0)
    (*b) = (-(*b));
  sb = (*b);
  zsetlength(&c, (i = (sa > sb ? sa : sb) + 1));
  zsetlength(&d, i);
  zsetlength(&q, i);
  *qqq = q;
  zsetlength(&r, sb + 1);
  *rrr = r;
  p = &b[sb];
  if ((sb == 1) && (*p < RADIX))
    zintoz(zsdiv(a, *p, &q), &r);
  else
    {
      sq = sa - sb;
      btopinv = (EXTENDED) (*p) * fradix;
      if (sb > 1)
	btopinv += (*(--p));
      btopinv *= fradix;
      if (sb > 2)
	btopinv += (*(p - 1));
      btopinv = fudge / btopinv;
      p = &a[1];
      pc = &c[0];
      for (i = sa; i > 0; i--)
	*pc++ = *p++;
      p = pc;
      sa = 1 - sb;
      for (i = (-sq); i > 0; i--)
	*p++ = 0;
      *pc = 0;
      d[1] = 0;
      p = &d[sq + 1];
      for (i = sq; i >= 0; i--)
	{
	  aux = fradix * (fradix * (*pc) + (*(pc - 1))) + 1.0;
	  if (i > sa)
	    aux += (*(pc - 2));
	  qq = (long) (btopinv * aux + 0.5);
	  /* dirty, but safe. On most machines (long)(btopinv*aux) */
	  /* is correct, or one too big; on some however it becomes */
	  /* too small. Could change zstart, but +0.5 and a while */
	  /* instead of one single if is safer */
	  if (qq > 0)
	    {
	      zsubmul(qq, &c[i], &b[0]);
	      while (*pc < 0)
		{
		  qq--;
		  {
		    zaddmulone(&c[i], &b[0]);
		  }
		}
	    }
	  pc--;
	  *p-- = qq;
	}
      sb--;
      while ((sb > 0) && (!(c[sb])))
	sb--;
      sb++;
      r[0] = sb;
      p = &r[1];
      pc = &c[0];
      for (i = sb; i > 0; i--)
	*p++ = *pc++;
      if (sq < 0)
	{
	  q[0] = 1;
	  q[1] = d[1];
	}
      else
	{
	  sq++;
	  while ((sq > 1) && (!(d[sq])))
	    sq--;
	  q[0] = sq;
	  p = &q[1];
	  pc = &d[1];
	  for (i = sq; i > 0; i--)
	    *p++ = *pc++;
	}
    }	/* non-trivial case */
  
  if (sign)
    {
      if (sign <= 2)
	{
	  if (!(r[1]) && (r[0] == 1))
	    q[0] = -(q[0]);
	  else
	    {
	      zadd(q, bigone, &q);
	      q[0] = -q[0];
	      if (sign == 1)
		zsub(r, b, &r);
	      else
		zsub(b, r, &r);
	    }
	}
      else
	znegate(&r);
    }
  *qqq = q;
  *rrr = r;
}

DEF_ACTION_2ARGS(zdiv) /* zdivmod :  Int * Int -> Int * Int */
{
  verylong q = 0;
  verylong r = 0;

  zquot ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &q, &r);
  if (liperr!=0)
    {
      if (q) /* added jgl 08 mar 2021 */
	{ MARKFREE (&q[-1]); } /*@@@*/
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  {
    ExprPtr qq, rr;
    ENTER;

    qq = lipMake (q);
    (void) KEEP (qq);
    rr = lipMake (r);
    (void) KEEP (rr);
    qq = Ktuple2 (qq, rr);
    QUIT;
    return qq;
  }
}
ACTION_END

PUBLIC void zmod(in_a,in_b,rr)
verylong in_a;
verylong in_b;
verylong *rr;
{
  register long i;
  register long qq;
  verylong r = *rr;
  static verylong a = 0;
  static verylong b = 0;
  static verylong c = 0;
  long sa;
  long sb;
  long sq;
  verylong p;
  verylong pc;
  long sign;
  EXTENDED btopinv;
  EXTENDED aux;

#ifndef START
  if (fudge < 0)
    zstart();
#endif
  liperr = 0;
  if (ALLOCATE && (!in_a || (in_a[0]==1 && !in_a[1])))
    {
      z_zero(rr);
      return;
    }
  if ((!in_b) || (((sb = in_b[0]) == 1) && (!in_b[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zmod");*/
      return;
    }
  zcopy(in_a,&a);
  zcopy(in_b,&b);
  sign = (*a < 0 ? 2 : 0) | (*b < 0 ? 1 : 0);
  if (*a < 0)
    (*a) = (-(*a));
  sa = (*a);
  if (*b < 0)
    (*b) = (-(*b));
  sb = (*b);
  zsetlength(&c, (sa > sb ? sa : sb) + 1);
  zsetlength(&r, sb + 1);
  *rr = r;
  p = &b[sb];
  if ((sb == 1) && (*p < RADIX))
    zintoz(zsdiv(a, *p, &c), &r);
  else
    {
      sq = sa - sb;
      btopinv = (EXTENDED) (*p) * fradix;
      if (sb > 1)
	btopinv += (*(--p));
      btopinv *= fradix;
      if (sb > 2)
	btopinv += (*(p - 1));
      btopinv = fudge / btopinv;
      p = &a[1];
      pc = &c[0];
      for (i = sa; i > 0; i--)
	*pc++ = *p++;
      p = pc;
      sa = 1 - sb;
      for (i = (-sq); i > 0; i--)
	*p++ = 0;
      *pc = 0;
      for (i = sq; i >= 0; i--)
	{
	  aux = fradix * (fradix * (*pc) + (*(pc - 1))) + 1.0;
	  if (i > sa)
	    aux += (*(pc - 2));
	  qq = (long) (btopinv * aux + 0.5);
	  /* see comment in zquot */
	  if (qq > 0)
	    {
	      zsubmul(qq, &c[i], &b[0]);
	      while (*pc < 0)
		{
		  {
		    zaddmulone(&c[i], &b[0]);
		  }
		}
	    }
	  pc--;
	}	/* loop on i */
      sb--;
      while ((sb > 0) && (!(c[sb])))
	sb--;
      sb++;
      r[0] = sb;
      p = &r[1];
      pc = &c[0];
      for (i = sb; i > 0; i--)
	*p++ = *pc++;
    }	/* non-trivial case */
  if (sign)
    {
      if ((sign <= 2) && (!((r[0] == 1) && !(r[1]))))
	{
	  if (sign == 1)
	    zsub(r, b, &r);
	  else
	    zsub(b, r, &r);
	}
      else
	znegate(&r);	/* negating r==0 doesn't hurt */
    }
  *rr = r;
}

DEF_ACTION_2ARGS(zmod) /* zmod : Int * Int -> Int */
{
  verylong r = 0;

  zmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &r);
  if (liperr!=0)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zaddmod(a,b,n,c)
verylong a;
verylong b;
verylong n;
verylong *c;
{
  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zaddmod");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      if (b)
	zcopy(b, c);
      else
	z_zero(c);
      return;
    }
  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      zcopy(a, c);
      return;
    }
  zadd(a, b, c);
  if (zcompare(*c, n) >= 0)
    zsubpos(*c, n, c);
}

DEF_ACTION_3ARGS(zaddmod) /* zmadd : Int * Int * Int -> Int */
{
  verylong n = 0;

  zaddmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zsubmod(a,b,n,c)
verylong a;
verylong b;
verylong n;
verylong *c;
{
  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zsubmod");*/
      return;
    }
  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      if (a)
	zcopy(a, c);
      else
	z_zero(c);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      if (b[1] || (b[0] != 1))
	zsubpos(n, b, c);
      else
	z_zero(c);
      return;
    }
  zsub(a, b, c);
  if ((*c)[0] < 0)
    zadd(*c, n, c);
}

DEF_ACTION_3ARGS(zsubmod) /* zmsub : Int * Int * Int -> Int */
{
  verylong n = 0;

  zsubmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zsmulmod(a,d,n,c)
verylong a;
long d;
verylong n;
verylong *c;
{
  static verylong mem = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zsmulmod");*/
      return;
    }
  if (ALLOCATE && (!a || !d || (a[0]==1 && a[1]==0)))
    {
      z_zero(c);
      return;
    }
  zsetlength(&mem, a[0] + 3);
  zsmul(a, d, &mem);
  zmod(mem, n, c);
}

DEF_ACTION_3ARGS(zsmulmod) /* zsmmul : Int * int * Int -> Int */
{
  verylong n = 0;

  zsmulmod ((verylong)EXTDATA(arg1), IVAL(arg2),
	    (verylong)EXTDATA(arg3), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zmulmod(a,b,n,c)
verylong a;
verylong b;
verylong n;
verylong *c;
{
  static verylong mem = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zmulmod");*/
      return;
    }
  if (ALLOCATE && (!a || !b || (a[0]==1 && !a[1]) || (b[0]==1 && !b[1])))
    {
      z_zero(c);
      return;
    }
  zsetlength(&mem, a[0] + b[0] + 2);
  ztimes(a, b, &mem);
  zmod(mem, n, c);
}

DEF_ACTION_3ARGS(zmulmod) /* zmmul : Int * Int * Int -> Int */
{
  verylong n = 0;

  zmulmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zsqmod(a,n,c)
verylong a;
verylong n;
verylong *c;
{
  static verylong mem = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zsqmod");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(c);
      return;
    }
  zsetlength(&mem, 2 * a[0] + 2);
  zsq(a, &mem);
  zmod(mem, n, c);
}

DEF_ACTION_2ARGS(zsqmod) /* zmsqr : Int * Int -> Int */
{
  verylong n = 0;

  zsqmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zdivmod(a,b,n,c)
verylong a;
verylong b;
verylong n;
verylong *c;
{
  static verylong g = 0;
  static verylong na = 0;
  static verylong nb = 0;
  static verylong r = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zdivmod");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(c);
      return;
    }
  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("divisor zero in zdivmod");*/
      return;
    }
  zgcd(a, b, &g);
  if ((g[1] == 1) && (g[0] == 1))
    {
      if (zinv(b, n, &g))
	{
	  /*zhalt("undefined quotient in zdivmod");*/
	  zcopy(g,c);
	}
      else
	zmulmod(a, g, n, c);
    }
  else
    {
      zquot(a, g, &na, &r);
      zquot(b, g, &nb, &r);
      if (zinv(nb, n, &g))
	{
	  /*zhalt("undefined quotient in zdivmod");*/
	  zcopy(g,c);
	}
      else
	zmulmod(na, g, n, c);
    }
}

DEF_ACTION_3ARGS(zdivmod) /* zmdiv : Int * Int * Int -> Int */
{
  verylong n = 0;

  zdivmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PUBLIC void zinvmod(a,n,c)
verylong a;
verylong n;
verylong *c;
{
  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zinvmod");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zinvmod");*/
      return;
    }
  if (zinv(a, n, c))
    /*zhalt("undefined inverse in zinvmod")*/;
}

DEF_ACTION_2ARGS(zinvmod) /* zminv : Int * Int -> Int */
{
  verylong n = 0;

  zinvmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &n);
  if (liperr!=0)
    {
      if (n) /* added jgl 08 mar 2021 */
	{ MARKFREE (&n[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (n);
}
ACTION_END

PRIVATE void zmstartint(n)
verylong n;
{
  static verylong x = 0;
  long i;

  liperr = 0;
  if (!n || !(n[1] & 1) || ((zntop = n[0]) < 1)
      || ((zntop == 1) && (n[1] <= 1)))
    {
      liperr = LIP_BAD_MONT_MOD;
      /*zhalt("zero, or even, or negative modulus in zmstart");*/
      return;
    }
  if (n[zntop] >= (RADIX >> 1))
    zntop++;
  zsetlength(&x, (long) (zntop + 1));
  zsetlength(&zn, (long) (zntop + 1));
  zsetlength(&zr, (long) (zntop + 1));
  zsetlength(&zrr, (long) (zntop + 1));
  zsetlength(&zrrr, (long) (zntop + 1));
  zsetlength(&znm, (long) (zntop + 1));
  if (zcompare(n, zn))
    {
      zcopy(n, &zn);
      zadd(zn, bigone, &zrr);
      zrshift(zrr, (long) 1, &x);	/* x = (n+1)/2 */
      zrr[0] = 1;
      zrr[1] = NBITS;
      zexpmod(x, zrr, zn, &x);	/* x = 2^(-NBITS) %n    */
      for (i = x[0]; i; i--)
	x[i + 1] = x[i];
      x[1] = 0;
      x[0]++;
      zsubpos(x, bigone, &x);
      zquot(x, zn, &x, &zrr);
      zminv1 = x[1];	/* zminv1 = n[1]^(-1) modulo 2^NBITS */
      zminv2 = zminv1 >> NBITSH;
      zminv1 &= RADIXROOTM;
      for (i = 1; i <= zntop; i++)
	x[i] = 0;
      x[zntop + 1] = 1;
      x[0] = zntop + 1;
      zquot(x, zn, &zrr, &zr);
      zsqmod(zr, zn, &zrr);
      zmulmod(zr, zrr, zn, &zrrr);
      zsubmod(zn, zr, zn, &znm);
    }
}

PRIVATE void zmkeep(n)
verylong n;
{
  if (znotinternal)
    zcopy(zn, &zoldzn);
  zmstartint(n);
}

PRIVATE void zmback()
{
  if (znotinternal)
    zmstartint(zoldzn);
}

PUBLIC void zmstart(n)
verylong n;
{
  zmstartint(n);
  znotinternal = 1;
}

DEF_ACTION_1ARG(zmstart) /* zmontstart : Int -> unit */
{
  zmstart ((verylong)EXTDATA(arg));
  if (liperr)
    return raiseLip (liperr);
  return tuple0;
}
ACTION_END

PUBLIC void zmfree()
{
  znotinternal = 0;
}

PUBLIC void ztom(a,bb)
verylong a;
verylong *bb;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in ztom");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if (a[0]<0 || zcompare(zn, a) < 0)
    {
      zmod(a, zn, bb);
      zmontmul(*bb, zrr, bb);
    }
  else
    zmontmul(a, zrr, bb);
}

DEF_ACTION_1ARG(ztom) /* ztomont : Int -> Int */
{
  verylong r = 0;

  ztom ((verylong)EXTDATA(arg), &r);
  if (liperr)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zmtoz(a,bb)
verylong a;
verylong *bb;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmtoz");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  zmontmul(a, bigone, bb);
}

DEF_ACTION_1ARG(zmtoz) /* zmonttoz : Int -> Int */
{
  verylong r = 0;

  zmtoz ((verylong)EXTDATA(arg), &r);
  if (liperr)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zmontadd(a,b,c)
verylong a;
verylong b;
verylong *c;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontadd");*/
      return;
    }
  zaddmod(a, b, zn, c);
}

DEF_ACTION_2ARGS(zmontadd) /* zmontadd : Int * Int -> Int */
{
  verylong c = 0;

  zmontadd ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  if (liperr)
    {
      if (c) /* added jgl 08 mar 2021 */
	{ MARKFREE (&c[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (c);
}
ACTION_END

PUBLIC void zmontsub(a,b,c)
verylong a;
verylong b;
verylong *c;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontsub");*/
      return;
    }
  zsubmod(a, b, zn, c);
}

DEF_ACTION_2ARGS(zmontsub) /* zmontsub : Int * Int -> Int */
{
  verylong c = 0;

  zmontsub ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  if (liperr)
    {
      if (c) /* added jgl 08 mar 2021 */
	{ MARKFREE (&c[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (c);
}
ACTION_END

PUBLIC void zsmontmul(a,d,c)
verylong a;
long d;
verylong *c;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zsmontmul");*/
      return;
    }
  zsmulmod(a, d, zn, c);
}

DEF_ACTION_2ARGS(zsmontmul) /* zsmontmul : Int * int -> Int */
{
  verylong b = 0;

  zsmontmul ((verylong)EXTDATA(arg1), IVAL(arg2), &b);
  if (liperr!=0)
    {
      if (b) /* added jgl 08 mar 2021 */
	{ MARKFREE (&b[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (b);
}
ACTION_END

PUBLIC void zmontmul(a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  register long i;
  register long j;
  verylong c = *cc;
  static verylong x = 0;
  verylong px;
  verylong pc;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontmul");*/
      return;
    }
  if (ALLOCATE && (!a || !b || (a[0]==1 && !a[1]) || (b[0]==1 && !b[1])))
    {
      z_zero(cc);
      return;
    }
  zsetlength(&x, (i = (zntop << 1) + 1));
  zsetlength(&c, zntop);
  if (a == *cc) a = c;
  if (b == *cc) b = c;
  *cc = c;
  if (!kar_mem_initialized)
    {
      kar_mem_initialized = 1;
      for (j = (5 * MAXDEPTH) - 1; j >= 0; j--)
	kar_mem[j] = (verylong) 0;
    }
  if (*a <= *b)
    kar_mul(b, a, &x, (long) 0);
  else
    kar_mul(a, b, &x, (long) 0);
  for (; i > x[0]; i--)
    x[i] = (long) 0;
  px = &x[1];
  for (i = zntop; i > 0; i--)
    zmmulp(px++, zn);
  pc = &c[1];
  for (i = zntop; i > 0; i--)
    *pc++ = *px++;
  i = zntop;
  while ((i > 1) && (!(*--pc)))
    i--;
  c[0] = i;
  if (zcompare(c, zn) >= 0)
    zsubpos(c, zn, &c);
  *cc = c;
}

DEF_ACTION_2ARGS(zmontmul) /* zmontmul : Int * Int -> Int */
{
  verylong c = 0;

  zmontmul ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  if (liperr)
    {
      if (c) /* added jgl 08 mar 2021 */
	{ MARKFREE (&c[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (c);
}
ACTION_END

PUBLIC void zmontsq(a,cc)
verylong a;
verylong *cc;
{
  register long i;
  register long j;
  verylong c = *cc;
  static verylong x = 0;
  verylong px;
  verylong pc;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontsq");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(cc);
      return;
    }
  zsetlength(&x, (i = (zntop << 1) + 1));
  zsetlength(&c, zntop);
  if (a == *cc) a = c;
  *cc = c;
  if (!kar_mem_initialized)
    {
      kar_mem_initialized = 1;
      for (j = (5 * MAXDEPTH) - 1; j >= 0; j--)
	kar_mem[j] = (verylong) 0;
    }
  kar_sq(a, &x, (long) 0);
  for (; i > x[0]; i--)
    x[i] = (long) 0;
  px = &x[1];
  for (i = zntop; i > 0; i--)
    zmmulp(px++, zn);
  pc = &c[1];
  for (i = zntop; i > 0; i--)
    *pc++ = *px++;
  i = zntop;
  while ((i > 1) && (!(*--pc)))
    i--;
  c[0] = i;
  if (zcompare(c, zn) >= 0)
    zsubpos(c, zn, &c);
  *cc = c;
}

DEF_ACTION_1ARG(zmontsqr) /* zmontsqr : Int -> Int */
{
  verylong b = 0;

  zmontsq ((verylong)EXTDATA(arg), &b);
  if (liperr)
    {
      if (b) /* added jgl 08 mar 2021 */
	{ MARKFREE (&b[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (b);
}
ACTION_END

PUBLIC void zmontdiv(a,b,c)
verylong a;
verylong b;
verylong *c;
{
  static verylong g = 0;
  static verylong na = 0;
  static verylong nb = 0;
  static verylong r = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontdiv");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(c);
      return;
    }
  if (ALLOCATE && (!b || (b[0]==1 && !b[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zmontdiv");*/
      return;
    }
  zgcd(a, b, &g);
  if ((g[1] == 1) && (g[0] == 1))
    {
      if (zinv(b, zn, &g))
	{
	  /*zhalt("undefined quotient in zmontdiv");*/
	  zcopy(g,c);
	  return;
	}
      else
	zmontmul(g, a, &g);
    }
  else
    {
      zquot(a, g, &na, &r);
      zquot(b, g, &nb, &r);
      if (zinv(nb, zn, &g))
	{
	  /*zhalt("undefined quotient in zmontdiv");*/
	  zcopy(g,c);
	  return;
	}
      else
	zmontmul(g, na, &g);
    }
  zmontmul(g, zrrr, c);
}

DEF_ACTION_2ARGS(zmontdiv) /* zmontdiv : Int * Int -> Int */
{
  verylong r = 0;

  zmontdiv ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &r);
  if (liperr!=0)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zmontinv(a,c)
verylong a;
verylong *c;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontinv");*/
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("division by zero in zmontinv");*/
      return;
    }
  if (zinv(a, zn, c))
    {
      /*zhalt("undefined inverse in zmontinv");*/
      return;
    }
  zmontmul(*c, zrrr, c);
}

DEF_ACTION_1ARG(zmontinv) /* zmontinv : Int -> Int */
{
  verylong r = 0;

  zmontinv ((verylong)EXTDATA(arg), &r);
  if (liperr!=0)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zrstarts(s)
long s;
{
	register long i;
	register long ones;
	register long res;
	PRIVATE verylong three = 0;

	zintoz(s, &zseed);
	if (!three)
	{
		zintoz((long) 3, &three);
		ones = 107 / NBITS;
		res = 107 % NBITS;
		zsetlength(&zranp, ones + 1);
		for (i = ones; i; i--)
			zranp[i] = RADIXM;
		if (res)
			zranp[++ones] = (RADIXM >> (NBITS - res));
		zranp[0] = ones;
	}
	zsexpmod(three, (long) 121, zranp, &zprroot);
}

PUBLIC void zrstart(s)
verylong s;
{
  if (!zseed)
    zrstarts((long) 7157891);
  zmod(s, zranp, &zseed);
}

PUBLIC long zrandom(b)
long b;
{
  static verylong zb = 0;
  static verylong bound = 0;

  if (b <= 0)
    return (0);
  if (!zseed)
    zrstarts((long) 7157891);	/* guess how random 7157891
				 * is */
  zintoz(b, &zb);
  zmod(zranp, zb, &bound);
  zsubpos(zranp, bound, &bound);
  do
    {
      zmulmod(zseed, zprroot, zranp, &zseed);
    } while (zcompare(zseed, bound) >= 0);	/* I agree, this is sick */
  zmod(zseed, zb, &zb);
  return (ztoint(zb));
}

PUBLIC long zinvodds(n,p)
long n;
long p;
{
  register long n1 = n;
  register long n2 = p;
  register long preg = p;
  register long m1 = 1;
  register long m2 = 0;

  liperr = 0;
  /* m1*n == n1 mod p,  m2*n == n2 mod p */
  /* n2 is odd, n1 is odd after initialization */
  while (!(n1 & 1))
    {
      n1 >>= 1;
      if (m1 & 1)
	m1 += preg;
      m1 >>= 1;
    }
  if (n1 == 1)
    return (m1);
  while (n1 != n2)
    {
      if (n1 > n2)
	{
	  n1 -= n2;
	  m1 -= m2;
	  if (m1 < 0)
	    m1 += preg;
	  do
	    {
	      n1 >>= 1;
	      if (m1 & 1)
		m1 += preg;
	      m1 >>= 1;
	    } while (!(n1 & 1));
	  if (n1 == 1)
	    return (m1);
	}
      else
	{
	  n2 -= n1;
	  m2 -= m1;
	  if (m2 < 0)
	    m2 += preg;
	  do
	    {
	      n2 >>= 1;
	      if (m2 & 1)
		m2 += preg;
	      m2 >>= 1;
	    } while (!(n2 & 1));
	  if (n2 == 1)
	    return (m2);
	}
    }
  liperr = LIP_NOT_COPRIME;
  /*zhalt("arguments not coprime in zinvodds");*/
  return (0);
}

PUBLIC long zinvs(n,p)
long n;
long p;
{
  register long q = n;
  register long r;
  register long nq;
  register long nr;
  register long u = 1;
  register long w;
  register long nw;
  register long par = 0;

  if (p < 0)
    p = -p;
  if (n < 0)
    n = -n;
  if (n > p)
    {
      w = 0;
      r = p;
    }
  else
    r = p - (w = p / n) * n;
  nr = r;
  while (nr != 0)
    {
      if ((nr = q - r) < r)
	nq = 1;
      else if ((nr -= r) < r)
	nq = 2;
      else
	nr = q - (nq = q / r) * r;
      nw = nq * w + u;
      u = w;
      w = nw;
      q = r;
      r = nr;
      par = 1 - par;
    }
  if (par == 0)
    return (u);
  else
    return (p - u);
}

PRIVATE long zxxeucl(ain,nin,invv,uu)
verylong ain;
verylong nin;
verylong *invv;
verylong *uu;
{
  static verylong a = 0;
  static verylong n = 0;
  static verylong q = 0;
  static verylong w = 0;
  static verylong x = 0;
  static verylong y = 0;
  static verylong z = 0;
  verylong inv = *invv;
  verylong u = *uu;
  long diff;
  long ilo;
  long sa;
  long sn;
  long temp;
  long e;
  long fast;
  long parity;
  long gotthem;
  verylong pin;
  verylong p;
  long i;
  long try11;
  long try12;
  long try21;
  long try22;
  long got11 = 0;
  long got12 = 0;
  long got21 = 0;
  long got22 = 0;
  EXTENDED hi;
  EXTENDED lo;
  EXTENDED dt;
  EXTENDED fhi;
  EXTENDED flo;
  EXTENDED num;
  EXTENDED den;

#ifndef START
  if (fudge < 0)
    zstart();
#endif
  zsetlength(&a, (e = (ain[-1] > nin[-1] ? ain[-1] : nin[-1])));
  zsetlength(&n, e);
  zsetlength(&q, e);
  zsetlength(&w, e);
  zsetlength(&x, e);
  zsetlength(&y, e);
  zsetlength(&z, e);
  zsetlength(&inv, e);
  *invv = inv;
  zsetlength(&u, e);
  *uu = u;
  fhi = 1.0 + epsilon;
  flo = 1.0 - epsilon;
  pin = &ain[0];
  p = &a[0];
  for (i = (*pin); i >= 0; i--)
    *p++ = *pin++;
  pin = &nin[0];
  p = &n[0];
  for (i = (*pin); i >= 0; i--)
    *p++ = *pin++;
  inv[0] = 1;
  inv[1] = 1;
  w[0] = 1;
  w[1] = 0;
  while (n[0] > 1 || n[1] > 0)
    {
      gotthem = 0;
      sa = a[0];
      sn = n[0];
      diff = sa - sn;
      if (!diff || diff == 1)
	{
	  sa = a[0];
	  p = &a[sa];
	  num = (EXTENDED) (*p) * fradix;
	  if (sa > 1)
	    num += (*(--p));
	  num *= fradix;
	  if (sa > 2)
	    num += (*(p - 1));
	  sn = n[0];
	  p = &n[sn];
	  den = (EXTENDED) (*p) * fradix;
	  if (sn > 1)
	    den += (*(--p));
	  den *= fradix;
	  if (sn > 2)
	    den += (*(p - 1));
	  hi = fhi * (num + 1.0) / den;
	  lo = flo * num / (den + 1.0);
	  if (diff > 0)
	    {
	      hi *= fradix;
	      lo *= fradix;
	    }
	  try11 = 1;
	  try12 = 0;
	  try21 = 0;
	  try22 = 1;
	  parity = 1;
	  fast = 1;
	  while (fast > 0)
	    {
	      parity = 1 - parity;
	      if (hi >= fradix)
		fast = 0;
	      else
		{
		  ilo = (long)lo;
		  if (!ilo || ilo < (long)hi)
		    fast = 0;
		  else
		    {
		      dt = lo;
		      lo = flo / (hi - ilo);
		      if (dt > ilo)
			hi = fhi / (dt - ilo);
		      else
			hi = fradix;
		      temp = try11;
		      try11 = try21;
		      if ((RADIX - temp) / ilo < try21)
			fast = 0;
		      else
			try21 = temp + ilo * try21;
		      temp = try12;
		      try12 = try22;
		      if ((RADIX - temp) / ilo < try22)
			fast = 0;
		      else
			try22 = temp + ilo * try22;
		      if ((fast > 0) && (parity > 0))
			{
			  gotthem = 1;
			  got11 = try11;
			  got12 = try12;
			  got21 = try21;
			  got22 = try22;
			}
		    }
		}
	    }
	}
      if (gotthem)
	{
	  zsmul(inv, got11, &x);
	  zsmul(w, got12, &y);
	  zsmul(inv, got21, &z);
	  zsmul(w, got22, &w);
	  zadd(x, y, &inv);
	  zadd(z, w, &w);
	  zsmul(a, got11, &x);
	  zsmul(n, got12, &y);
	  zsmul(a, got21, &z);
	  zsmul(n, got22, &n);
	  zsub(x, y, &a);
	  zsub(n, z, &n);
	}
      else
	{
	  zquot(a, n, &q, &a);
	  ztimes(q, w, &x);
	  zadd(inv, x, &inv);
	  if (a[0] > 1 || a[1] > 0)
	    {
	      zquot(n, a, &q, &n);
	      ztimes(q, inv, &x);
	      zadd(w, x, &w);
	    }
	  else
	    {
	      p = &a[0];
	      pin = &n[0];
	      for (i = (*pin); i >= 0; i--)
		*p++ = *pin++;
	      n[0] = 1;
	      n[1] = 0;
	      zsub(nin, w, &inv);
	    }
	}
    }
  if ((a[0] == 1) && (a[1] == 1))
    e = 0;
  else
    e = 1;
  p = &u[0];
  pin = &a[0];
  for (i = (*pin); i >= 0; i--)
    *p++ = *pin++;
  *invv = inv;
  *uu = u;
  return (e);
}

PUBLIC long zinv(ain,nin,invv)
verylong ain;
verylong nin;
verylong *invv;
{
  static verylong u = 0;

  liperr = 0;
  if ((!nin) || (!ain) || (ain[0] < 0) || (nin[0] < 0))
    {
      liperr = LIP_ZERO_OR_NEG;
      /*zhalt("zero or negative argument(s) in zinv");*/
      return (0);
    }
  if (!(zxxeucl(ain, nin, invv, &u)))
    return (0);
  zcopy(u, invv);
  return (1);
}

PUBLIC void zexteucl(a,xa,b,xb,d)
verylong a;
verylong *xa;
verylong b;
verylong *xb;
verylong *d;
{
  static verylong modcon = 0;
  register long agrb;
  register long anegative;
  register long bnegative;

  liperr = 0;
  if (((!a) || (a[0]==1 && !a[1])) && ((!b) || (b[0]==1 && !b[1])))
    {
      liperr = LIP_ZERO;
      /*zhalt("zero argument(s) in zexteucl");*/
      return;
    }
  if ((anegative = (a[0] < 0)))
    a[0] = -a[0];
  if ((bnegative = (b[0] < 0)))
    b[0] = -b[0];
  if (!a[1] && (a[0] == 1))
    {
      z_zero(xa);
      z_one(xb);
      zcopy(b, d);
      goto done;
    }
  if ((!b[1] && (b[0] == 1)) || !(agrb = zcompare(a, b)))
    {
      z_one(xa);
      z_zero(xb);
      zcopy(a, d);
      goto done;
    }
  if (agrb > 0)
    {
      zxxeucl(a, b, xa, d);
      ztimes(a, *xa, xb);
      zsub(*d, *xb, xb);
      zquot(*xb, b, xb, &modcon);
    }
  else
    {
      zxxeucl(b, a, xb, d);
      ztimes(b, *xb, xa);
      zsub(*d, *xa, xa);
      zquot(*xa, a, xa, &modcon);
    }
  if ((modcon[1]) || (modcon[0] != 1))
    {
      liperr = LIP_NON_ZERO_REM_BUG;
      /*zhalt("non-zero remainder in zexteucl   BUG");*/
      return;
    }
done:
  if (anegative)
    {
      a[0] = -a[0];
      znegate(xa);
      if (a==b)
	znegate (xb);
    }
  if (bnegative)
    {
      b[0] = -b[0];
      znegate(xb);
    }
}

DEF_ACTION_2ARGS(zexteucl) /* zbezout : Int * Int -> (Int * Int) * Int */
{
  verylong xa = 0;
  verylong xb = 0;
  verylong d = 0;
  ExprPtr ex, exa, exb, exab, exd;
  ENTER;

  zexteucl ((verylong)EXTDATA(arg1), &xa, (verylong)EXTDATA(arg2), &xb, &d);
  if (liperr)
    {
      if (xa) /* added jgl 08 mar 2021 */
	{ MARKFREE (&xa[-1]); } /*@@@*/
      if (xb) /* added jgl 08 mar 2021 */
	{ MARKFREE (&xb[-1]); } /*@@@*/
      if (d) /* added jgl 08 mar 2021 */
	{ MARKFREE (&d[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  exa = lipMake (xa);
  (void) KEEP (exa);
  exb = lipMake (xb);
  (void) KEEP (exb);
  exab = Ktuple2 (exa, exb);
  (void) KEEP (exab);
  exd = lipMake (d);
  (void) KEEP (exd);
  ex = Ktuple2 (exab, exd);
  QUIT;
  return ex;
}
ACTION_END

PUBLIC void zchirem(aa,maa,bb,mbb,d)
verylong aa;
verylong maa;
verylong bb;
verylong mbb;
verylong *d;
{
  static verylong u = 0;
  static verylong v = 0;
  static verylong w = 0;
  verylong a;
  verylong b;
  verylong ma;
  verylong mb;
  register long agrb;

  liperr = 0;
  if ((!maa) || (!mbb))
    {
      liperr = LIP_ZERO;
      /*zhalt("zero argument(s) in zchirem");*/
      return;
    }
  if ((!aa) || (!bb) || (!aa[1] && aa[0] == 1) || (!bb[1] && bb[0] == 1) ||
      (aa[0] < 0) || (bb[0] < 0))
    {
      liperr = LIP_ZERO_OR_NEG;
      /*zhalt("zero or negative argument(s) in zchirem");*/
      return;
    }
  if ((agrb = aa[0]) < bb[0])
    agrb = bb[0];
  zsetlength(&u, agrb);
  zsetlength(&v, agrb);
  zsetlength(&w, agrb);
  if (!(agrb = zcompare(aa, bb)))
    {
      zsub(maa, mbb, &u);
      zmod(u, aa, &w);
      if (w[1] || w[0] != 1)
	{
	  liperr = LIP_SAME_MODULI;
	  /*zhalt("same moduli with different remainders in zchirem");*/
	  return;
	}
      zcopy(maa, d);
      return;
    }
  else if (agrb > 0)
    {
      a = aa;
      b = bb;
      ma = maa;
      mb = mbb;
    }
  else
    {
      a = bb;
      b = aa;
      ma = mbb;
      mb = maa;
    }
  if (zxxeucl(a, b, &u, &w))
    {
      liperr = LIP_NOT_COPRIME;
      /*zhalt("moduli not coprime in zchirem");*/
      return;
    }
  zmod(ma, a, &v);
  zsub(mb, v, &w);
  zmulmod(w, u, b, &w);
  ztimes(w, a, &u);
  zadd(v, u, d);
}

DEF_ACTION_4ARGS(zchirem) /* zchirem : Int * Int * Int * Int -> Int */
{
  verylong d = 0;

  zchirem ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), (verylong)EXTDATA(arg4), &d);
  if (liperr!=0)
    {
      if (d) /* added jgl 08 mar 2021 */
	{ MARKFREE (&d[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (d);
}
ACTION_END

PUBLIC void zmontexp(a,e,bb)
verylong a;
verylong e;
verylong *bb;
{
  register long i;
  register long j;
  register long k = 0;
  verylong b = *bb;
  static verylong loca = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      zcopy(zr,bb);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);
  if ((i == 1) && (!(e[1])))
    zcopy(zr,&b);
  else
    {
      zcopy(a, &loca);
      zcopy(loca, &b);
      for (; i; i--)
	{
	  j = e[i];
	  if (!k)
	    {
	      k = 1;
	      while ((k << 1) <= j)
		k <<= 1;
	    }
	  while (k >>= 1)
	    {
	      zmontsq(b, &b);
	      if (j & k)
		zmontmul(loca, b, &b);
	    }
	  k = RADIX;
	}
    }
  if (e[0] < 0)
    {
      if (zinv(b, zn, &b))
	{
	  liperr = LIP_NEGATIVE_EXP;
	  /*zhalt("undefined quotient in zmontexp");*/
	  return;
	}
      zmontmul(b, zrrr, &b);
    }
  *bb = b;
}

DEF_ACTION_2ARGS(zmontexp) /* zmontexp : Int * Int -> Int */
{
  verylong bb = 0;

  zmontexp ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &bb);
  if (liperr)
    {
      if (bb)
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC void zsexpmod(a,e,n,bb)
verylong a;
long e;
verylong n;
verylong *bb;
{
  static verylong le = 0;

  zintoz(e, &le);
  zexpmod(a, le, n, bb);
}

DEF_ACTION_3ARGS(zsexpmod) /* zsmexp : Int * int * Int -> Int */
{
  verylong bb = 0;

  zsexpmod ((verylong)EXTDATA(arg1), IVAL(arg2),
	    (verylong)EXTDATA(arg3), &bb);
  if (liperr!=0)
    {
      if (bb) /* added jgl 08 mar 2021 */
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC void zexpmod(a,e,n,bb)
verylong a;
verylong e;
verylong n;
verylong *bb;
{
  register long i;
  register long j;
  register long k = 0;
  verylong b = *bb;
  static verylong loca = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      z_one(bb);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);
  if ((i == 1) && (!(e[1])))
    z_one(&b);
  else
    {
      zmod(a, n, &loca);
      zcopy(loca, &b);
      for (; i; i--)
	{
	  j = e[i];
	  if (!k)
	    {
	      k = 1;
	      while ((k << 1) <= j)
		k <<= 1;
	    }
	  while (k >>= 1)
	    {
	      zsqmod(b, n, &b);
	      if (j & k)
		zmulmod(loca, b, n, &b);
	    }
	  k = RADIX;
	}
    }
  if (e[0] < 0 && zinv(b, n, &b))
    {
      liperr = LIP_NEGATIVE_EXP;
      /*zhalt("undefined quotient in zexpmod");*/
    }
  *bb = b;
}

DEF_ACTION_3ARGS(zexpmod) /* zmexp : Int * Int * Int -> Int */
{
  verylong bb = 0;

  zexpmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
	   (verylong)EXTDATA(arg3), &bb);
  if (liperr!=0)
    {
      if (bb)
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC void z2expmod(e,n,bb)
verylong e;
verylong n;
verylong *bb;
{
  register long i;
  register long j;
  register long k = 0;
  verylong b = *bb;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in z2expmod");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      z_one(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);
  if ((i == 1) && (!(e[1])))
    z_one(&b);
  else
    {
      if ((j = n[0]) < 0)
	j = -j;
      zsetlength(&b, j);
      *bb = b;
      zintoz((long) 2, &b);
      for (; i; i--)
	{
	  j = e[i];
	  if (!k)
	    {
	      k = 1;
	      while ((k << 1) <= j)
		k <<= 1;
	    }
	  while (k >>= 1)
	    {
	      zsqmod(b, n, &b);
	      if (j & k)
		{
		  z2mul(b, &b);
		  if (zcompare(b, n) >= 0)
		    zsubpos(b, n, &b);
		}
	    }
	  k = RADIX;
	}
    }
  if (e[0] < 0 && zinv(b, n, &b))
    {
      liperr = LIP_NEGATIVE_EXP;
      /*zhalt("undefined quotient in z2expmod");*/
    }
  *bb = b;
}

DEF_ACTION_2ARGS(z2expmod) /* zm2exp  : Int * Int -> Int */
{
  verylong bb = 0;

  z2expmod ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &bb);
  if (liperr)
    {
      if (bb)
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC long zdefault_m(l)
long l;
{
  if (l < 2)
    return (2);
  if (l < 3)
		return (3);
  if (l < 7)
    return (4);
  if (l < 16)
    return (5);
  if (l < 35)
    return (6);
  if (l < 75)
    return (7);
  if (l < 160)
    return (8);
  if (l < 340)
    return (9);
  return (10);
}

/*
DEF_ACTION_1ARG(zdefault_m)
{
  return Kint (zdefault_m (IVAL (arg)));
}
ACTION_END
*/

PRIVATE void mont_init_odd_power(a,a_sq,m,odd_a_power)
verylong a;
verylong *a_sq;
long m;
verylong **odd_a_power;
{
  /* odd power setter for montgomery m-ary exponentiation */
  /* initialize odd_a_power with a^i mod n for odd positive i < 2^m */
  /* a on input is montgomery number */
  register long i;
  register long length = (1L << (m - 1));

  if (!(*odd_a_power))
    {
      *odd_a_power = (verylong *)MALLOC((size_t)(length * sizeof(verylong)));
      for (i = 0; i < length; i++)
	(*odd_a_power)[i] = (verylong) 0;
    }
  zcopy(a, &((*odd_a_power)[0]));
  zmontsq((*odd_a_power)[0], a_sq);
}

PUBLIC void zmontexp_m_ary(a,e,bb,m)
verylong a;
verylong e;
verylong *bb;
long m;
{
  register long i;
  register long ei;
  static verylong a_sq = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp_m_ary");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      zcopy(zr,bb);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);
  if (i == 1)
    {
      zmontexp(a, e, bb);
      return;
    }
  if (!exp_odd_powers)
    {
      exp_odd_powers = (verylong **)MALLOC((size_t)(NBITS * sizeof(verylong *)));
      for (ei = 0; ei < NBITS; ei++)
	exp_odd_powers[ei] = (verylong *) 0;
    }
  if (m <= 1)
    m = zdefault_m(i);
  else if (m >= NBITS)
    m = (NBITS - 1);
  {
    register long k = 0;
    register long od = 0;
    register long odlen = 0;
    register long left = 0;
    register long zeros = 0;
    register long rem = 0;
    register long first = 1;
    register long reach = 1;
    verylong **loc_pow = &(exp_odd_powers[m]);

    mont_init_odd_power(a, &a_sq, m, loc_pow);
    for (; i > 0; i--)
      {
	ei = e[i];
	if (!k)
	  {
	    k = 1;
	    while (k <= ei)
	      k <<= 1;
	  }
	while (k >>= 1)
	  {
	    if (zeros)
	      {
		if (ei & k)
		  {
		    odlen = od = 1;
		    rem = zeros;
		    left = zeros = 0;
		  }
		else
		  zeros++;
	      }
	    else if (!od)
	      {
		if (ei & k)
		  {
		    odlen = od = 1;
		    left = zeros = 0;
		  }
		else
		  zeros = 1;
	      }
	    else
	      {
		left++;
		if (odlen + left == m)
		  {
		    if (ei & k)
		      {
			od <<= left;
			odlen += left;
			left = 0;
			od++;
			od >>= 1;
			for (; reach <= od; reach++)
			  zmontmul((*loc_pow)[reach - 1], a_sq, &((*loc_pow)[reach]));
			if (first)
			  {
			    zcopy((*loc_pow)[od], bb);
			    first = rem = odlen = od = 0;
			  }
			else
			  {
			    for (rem = rem + odlen; rem; rem--)
			      zmontsq(*bb, bb);
			    zmontmul(*bb, (*loc_pow)[od], bb);
			    odlen = od = 0;
			  }
		      }
		    else
		      {
			if (first)
			  {
			    if (od == 1)
			      {
				zcopy(a_sq, bb);
				left--;
			      }
			    else
			      {
				od >>= 1;
				for (; reach <= od; reach++)
				  zmontmul((*loc_pow)[reach - 1], a_sq, &((*loc_pow)[reach]));
				zcopy((*loc_pow)[od], bb);
			      }
			    zeros = left;
			    first = left = odlen = od = rem = 0;
			  }
			else
			  {
			    for (rem = rem + odlen; rem; rem--)
			      zmontsq(*bb, bb);
			    od >>= 1;
			    for (; reach <= od; reach++)
			      zmontmul((*loc_pow)[reach - 1], a_sq, &((*loc_pow)[reach]));
			    zmontmul(*bb, (*loc_pow)[od], bb);
			    zeros = left;
			    left = odlen = od = 0;
			  }
		      }
		  }
		else if (ei & k)
		  {
		    od <<= left;
		    odlen += left;
		    left = 0;
		    od++;
		  }
	      }
	  }
	k = RADIX;
      }
    if (od)
      {
	if (first)
	  {
	    if (od == 1)
	      {
		zcopy(a_sq, bb);
		left--;
	      }
	    else
	      {
		od >>= 1;
		for (; reach < od; reach++)
		  zmontmul((*loc_pow)[reach - 1], a_sq, &((*loc_pow)[reach]));
		zmontmul((*loc_pow)[od - 1], a_sq, bb);
	      }
	  }
	else
	  {
	    for (rem = rem + odlen; rem; rem--)
	      zmontsq(*bb, bb);
	    od >>= 1;
	    for (; reach <= od; reach++)
	      zmontmul((*loc_pow)[reach - 1], a_sq, &((*loc_pow)[reach]));
	    zmontmul(*bb, (*loc_pow)[od], bb);
	  }
      }
    if (left || zeros)
      {
	for (rem = zeros + left; rem; rem--)
	  zmontsq(*bb, bb);
      }
  }
  
  if (e[0] < 0)
    {
      if (zinv(*bb, zn, bb))
	{
	  liperr = LIP_NEGATIVE_EXP;
	  /*zhalt("undefined quotient in zmontexp_m_ary");*/
	  return;
	}
      zmontmul(*bb, zrrr, bb);
    }
}

DEF_ACTION_2ARGS(zmontexp_m_ary) /* zmontexpm : Int * Int -> Int */
{
  verylong bb = 0;

  zmontexp_m_ary ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &bb, 1);
  if (liperr)
    {
      if (bb)
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PRIVATE void init_odd_power(a,n,a_sq,m,odd_a_power)
verylong a;
verylong n;
verylong *a_sq;
long m;
verylong **odd_a_power;
{
  /* odd power setter for m-ary exponentiation */
  /* initialize odd_a_power with a^i mod n for odd positive i < 2^m */
  /* a on input is not necessarily reduced mod n */
  register long i;
  register long length = (1L << (m - 1));

  if (!(*odd_a_power))
    {
      *odd_a_power = (verylong *)MALLOC((size_t)(length * sizeof(verylong)));
      for (i = 0; i < length; i++)
	(*odd_a_power)[i] = (verylong) 0;
    }
  zmod(a, n, &((*odd_a_power)[0]));
  zsqmod((*odd_a_power)[0], n, a_sq);
}

PUBLIC void zexpmod_m_ary(a,e,n,bb,m)
verylong a;
verylong e;
verylong n;
verylong *bb;
long m;
{
  register long i;
  register long ei;
  static verylong a_sq = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod_m_ary");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      z_one(bb);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);
  if (i == 1)
    {
      zexpmod(a, e, n, bb);
      return;
    }
  if (!exp_odd_powers)
    {
      exp_odd_powers = (verylong **)MALLOC((size_t)(NBITS * sizeof(verylong *)));
      for (ei = 0; ei < NBITS; ei++)
	exp_odd_powers[ei] = (verylong *) 0;
    }
  if (m <= 1)
    m = zdefault_m(i);
  else if (m >= NBITS)
    m = (NBITS - 1);
  {
    register long k = 0;
    register long od = 0;
    register long odlen = 0;
    register long left = 0;
    register long zeros = 0;
    register long rem = 0;
    register long first = 1;
    register long reach = 1;
    verylong **loc_pow = &(exp_odd_powers[m]);

    init_odd_power(a, n, &a_sq, m, loc_pow);
    for (; i > 0; i--)
      {
	ei = e[i];
	if (!k)
	  {
	    k = 1;
	    while (k <= ei)
	      k <<= 1;
	  }
	while (k >>= 1)
	  {
	    if (zeros)
	      {
		if (ei & k)
		  {
		    odlen = od = 1;
		    rem = zeros;
		    left = zeros = 0;
		  }
		else
		  zeros++;
	      }
	    else if (!od)
	      {
		if (ei & k)
		  {
		    odlen = od = 1;
		    left = zeros = 0;
		  }
		else
		  zeros = 1;
	      }
	    else
	      {
		left++;
		if (odlen + left == m)
		  {
		    if (ei & k)
		      {
			od <<= left;
			odlen += left;
			left = 0;
			od++;
			od >>= 1;
			for (; reach <= od; reach++)
			  zmulmod((*loc_pow)[reach - 1], a_sq, n, &((*loc_pow)[reach]));
			if (first)
			  {
			    zcopy((*loc_pow)[od], bb);
			    first = rem = odlen = od = 0;
			  }
			else
			  {
			    for (rem = rem + odlen; rem; rem--)
			      zsqmod(*bb, n, bb);
			    zmulmod(*bb, (*loc_pow)[od], n, bb);
			    odlen = od = 0;
			  }
		      }
		    else
		      {
			if (first)
			  {
			    if (od == 1)
			      {
				zcopy(a_sq, bb);
				left--;
			      }
			    else
			      {
				od >>= 1;
				for (; reach <= od; reach++)
				  zmulmod((*loc_pow)[reach - 1], a_sq, n, &((*loc_pow)[reach]));
				zcopy((*loc_pow)[od], bb);
			      }
			    zeros = left;
			    first = left = odlen = od = rem = 0;
			  }
			else
			  {
			    for (rem = rem + odlen; rem; rem--)
			      zsqmod(*bb, n, bb);
			    od >>= 1;
			    for (; reach <= od; reach++)
			      zmulmod((*loc_pow)[reach - 1], a_sq, n, &((*loc_pow)[reach]));
			    zmulmod(*bb, (*loc_pow)[od], n, bb);
			    zeros = left;
			    left = odlen = od = 0;
			  }
		      }
		  }
		else if (ei & k)
		  {
		    od <<= left;
		    odlen += left;
		    left = 0;
		    od++;
		  }
	      }
	  }
	k = RADIX;
      }
    if (od)
      {
	if (first)
	  {
	    if (od == 1)
	      {
		zcopy(a_sq, bb);
		left--;
	      }
	    else
	      {
		od >>= 1;
		for (; reach < od; reach++)
		  zmulmod((*loc_pow)[reach - 1], a_sq, n, &((*loc_pow)[reach]));
		zmulmod((*loc_pow)[od - 1], a_sq, n, bb);
	      }
	  }
	else
	  {
	    for (rem = rem + odlen; rem; rem--)
	      zsqmod(*bb, n, bb);
	    od >>= 1;
	    for (; reach <= od; reach++)
	      zmulmod((*loc_pow)[reach - 1], a_sq, n, &((*loc_pow)[reach]));
	    zmulmod(*bb, (*loc_pow)[od], n, bb);
	  }
      }
    if (left || zeros)
      {
	for (rem = zeros + left; rem; rem--)
	  zsqmod(*bb, n, bb);
      }
  }
  
  if (e[0] < 0 && zinv(*bb, n, bb))
    {
      liperr = LIP_NEGATIVE_EXP;
      /*zhalt("undefined result in zexpmod_m_ary");*/
    }
}

DEF_ACTION_3ARGS(zexpmod_m_ary) /* zmexpm : Int * Int * Int -> Int */
{
  verylong bb = 0;

  zexpmod_m_ary ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
		 (verylong)EXTDATA(arg3), &bb, 1);
  if (liperr!=0)
    {
      if (bb)
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC void zsexp(a,e,bb)
verylong a;
long e;
verylong *bb;
{
  static verylong le = 0;

  zintoz(e, &le);
  z_exp(a, le, bb);
}

DEF_ACTION_2ARGS(zsexp) /* zsexp : Int * int -> Int */
{
  verylong bb = 0;

  zsexp ((verylong)EXTDATA(arg1), IVAL(arg2), &bb);
  if (liperr)
    {
      if (bb) /* added jgl 08 mar 2021 */
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC void z_exp(a,e,bb)
verylong a;
verylong e;
verylong *bb;
{
  register long i;
  register long j;
  register long k = 0;
  long sa;
  verylong b = (*bb);
  static verylong temp = 0;

  liperr = 0;
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      z_one(bb);
      return;
    }
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      z_zero(bb);
      return;
    }
  if ((e[0] == 1) && (!(e[1])))
    z_one(bb);
  else if ((a[1] == 1) && (a[0] == 1))
    z_one(bb);
  else if ((a[1] == 1) && (a[0] == -1))
    {
      if (e[1] & 1)
	zintoz((long) (-1), bb);
      else
	z_one(bb);
    }
  else if (e[0] < 0)
    {
      liperr = LIP_NEGATIVE_EXP;
      /*zhalt("negative exponent in z_exp");*/
      return;
    }
  else if (!(a[1]) && (a[0] == 1))
    z_zero(bb);
  else
    {
      if ((sa = a[0]) < 0)
	sa = (-sa);
      i = (long) ((sa - 1 + log10((EXTENDED) (a[sa])) / LOG10RAD) * ztoint(e) + 4);
      zsetlength(&b, i);
      zsetlength(&temp, i);
      for (i = sa; i >= 0; i--)
	b[i] = a[i];
      for (i = e[0]; i; i--)
	{
	  j = e[i];
	  if (!k)
	    {
	      k = 1;
	      while ((k << 1) <= j)
		k <<= 1;
	    }
	  while (k >>= 1)
	    {
	      zsqin(&b);
	      if (j & k)
		zmulin(a, &b);
	    }
	  k = RADIX;
	}
      *bb = b;
    }
}

DEF_ACTION_2ARGS(zexp) /* zexp : Int * Int -> Int */
{
  verylong bb = 0;

  z_exp ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &bb);
  if (liperr)
    {
      if (bb) /* added jgl 08 mar 2021 */
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC long zexpmods(a,e,n)
long a;
long e;
long n;
{
  if (!e)
    return (1);
  else
    {
      register long aa = a % n;
      register long ipow2 = 1;
      long b = aa;

      if (e<0) e = -e;
      while ((ipow2 << 1) <= e)
	ipow2 <<= 1;
      while (ipow2 >>= 1)
	{
	  b = zmulmods(b, b, n);
	  if (e & ipow2)
	    b = zmulmods(b, aa, n);
	}
      return (b);
    }
}

DEF_ACTION_3ARGS(zexpmods) /* zmsexp : int * int * int -> int */
{
  INT m = IVAL(arg3);

  if (m==0)
    return raiseLip (LIP_ZERO_MOD);
  return Kint (zexpmods (IVAL(arg1), IVAL(arg2), m));
}
ACTION_END

PUBLIC void zexpmod_doub1(xx1,ee1,xx2,ee2,n,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong n;
verylong *r;
{
  register long bit11, bit21;
  register long i,j,mask,block;
  static verylong tab[2][2];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod_doub1");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zexpmod_doub1");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<2;i++) for (j=0;j<2;j++)
      tab[i][j] = 0;
    non_ini = 0;
  }
#define MOD_INITPAR \
        if (zcompare(ee1,ee2)>=0) { \
		zcopy(xx1,&(tab[1][0])); \
		zcopy(xx2,&(tab[0][1])); \
                zcopy(ee1,&e1); \
                zcopy(ee1,&e2); /* create space for e2 */ \
                zcopy(ee2,&e2); \
        } else { \
		zcopy(xx2,&(tab[1][0])); \
		zcopy(xx1,&(tab[0][1])); \
                zcopy(ee2,&e2); /* create space for e2 */ \
                zcopy(ee1,&e2); \
                zcopy(ee2,&e1); \
        } \
        if (!(zscompare(e1,0))) { \
		zintoz(1,r); \
                return; \
        } \
        for (i=e2[0]+1;i<=e1[0];i++) e2[i] = 0;

	MOD_INITPAR

	  zmulmod(tab[1][0],tab[0][1],n,&(tab[1][1]));

	block = z2log(e1)-1;
        mask = ( 1L<<(block % NBITS) );
        block = (block / NBITS) +1;
        /*bit11 = 1;*/
        bit21 = (e2[block]&mask) ? 1 : 0;
	zcopy(tab[1][bit21],r);

#define MOD_FIRST \
	mask >>= 1; \
	if (!mask) { \
		block --; \
		if (!block) return; \
		mask  = ( 1L <<(NBITS-1) ); \
	} \
	bit11 = (e1[block]&mask) ? 1 : 0; \
	bit21 = (e2[block]&mask) ? 1 : 0; 

	MOD_FIRST
	  for (;;) {
	    zsqmod(*r,n,r);
	    if (bit11 || bit21) 
	      zmulmod(tab[bit11][bit21],*r,n,r);
	    MOD_FIRST
	      }
}

PUBLIC void zexpmod_doub2(xx1,ee1,xx2,ee2,n,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong n;
verylong *r;
{
  register long i,j,mask,block;
  register long bit11, bit12, bit21, bit22;
  static verylong tab[4][4];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod_doub2");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zexpmod_doub2");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<4;i++) for (j=0;j<4;j++)
      tab[i][j] = 0;
    non_ini = 0;
  }

  MOD_INITPAR

    zsqmod(tab[1][0],n,&(tab[2][0]));
  zsqmod(tab[0][1],n,&(tab[0][2]));
  zmulmod(tab[2][0],tab[1][0],n,&(tab[3][0]));
  zmulmod(tab[0][2],tab[0][1],n,&(tab[0][3]));
  for (j=1;j<4;j++) {
    for (i=1;i<4;i++) if ((i&1) | (j&1))
      zmulmod(tab[j][0],tab[0][i],n,&(tab[j][i]));
  }

  block = z2log(e1)-1;
  mask = ( 1L<<(block % NBITS) );
  block = (block / NBITS) +1;
  bit11 = 1;
  bit21 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      zcopy(tab[1][bit21],r);
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit12 = (e1[block]&mask) ? 1 : 0;
  bit22 = (e2[block]&mask) ? 1 : 0;
  if (bit12 || bit22) {
    zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
  } else {
    zcopy(tab[bit11][bit21],r);
    zsqmod(*r,n,r);
  }

#define MOD_SECOND \
	mask >>= 1; \
	if (!mask) { \
		block--; \
		if (!block) { \
			zsqmod(*r,n,r); \
			if (bit11||bit21) \
				zmulmod(tab[bit11][bit21],*r,n,r); \
			return; \
		} \
		mask = ( 1L<<(NBITS-1) ); \
	} \
        bit12 = (e1[block]&mask) ? 1 : 0; \
        bit22 = (e2[block]&mask) ? 1 : 0;

	MOD_FIRST
	  MOD_SECOND
	  for (;;) {
	    zsqmod(*r,n,r);
	    if (bit11 || bit21) {
	      if (bit12 || bit22) {
		zsqmod(*r,n,r);
		zmulmod(tab[(bit11<<1)+bit12][(bit21<<1)+bit22],*r,n,r);
	      } else {
		zmulmod(tab[bit11][bit21],*r,n,r);
		zsqmod(*r,n,r);
	      }
	      MOD_FIRST
		} else {
		  bit11 = bit12;
		  bit21 = bit22;
		}
	    MOD_SECOND
	      }
}

PUBLIC void zexpmod_doub3(xx1,ee1,xx2,ee2,n,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong n;
verylong *r;
{
  register long i,j, mask, block;
  register long bit11, bit12, bit13, bit21, bit22, bit23;
  static verylong tab[8][8];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod_doub3");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zexpmod_doub3");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<8;i++) for (j=0;j<8;j++)
      tab[i][j] = 0;
    non_ini = 0;
  }

  MOD_INITPAR

    zsqmod(tab[1][0],n,&(tab[2][0]));
  zsqmod(tab[0][1],n,&(tab[0][2]));
  for (i=3;i<8;i+=2) zmulmod(tab[2][0],tab[i-2][0],n,&(tab[i][0]));
  for (i=3;i<8;i+=2) zmulmod(tab[0][2],tab[0][i-2],n,&(tab[0][i]));
  for (i=1;i<8;i++) zmulmod(tab[0][1],tab[1][i-1],n,&(tab[1][i]));
  for (i=1;i<8;i+=2) zmulmod(tab[0][i],tab[2][0],n,&(tab[2][i]));
  for (i=1;i<8;i++) zmulmod(tab[0][1],tab[3][i-1],n,&(tab[3][i]));
  for (j=4;j<8;j++) {
    zmulmod(tab[1][0],tab[j-1][1],n,&(tab[j][1]));
    for (i=3;i<8;i+=2) zmulmod(tab[0][2],tab[j][i-2],n,&(tab[j][i]));
    j++;
    for (i=1;i<8;i++) zmulmod(tab[0][1],tab[j][i-1],n,&(tab[j][i]));
  }
  block = z2log(e1)-1;
  mask = ( 1L<<(block % NBITS) );
  block = (block / NBITS) +1;
  bit11 = 1;
  bit21 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      zcopy(tab[1][bit21],r);
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit12 = (e1[block]&mask) ? 1 : 0;
  bit22 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      if (bit12 || bit22) {
	zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
      } else {
	zcopy(tab[bit11][bit21],r);
	zsqmod(*r,n,r);
      }
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit13 = (e1[block]&mask) ? 1 : 0;
  bit23 = (e2[block]&mask) ? 1 : 0;
  if (bit13 || bit23) {
    zcopy(tab[4+(bit12<<1)+bit13][(bit21<<2)+(bit22<<1)+bit23],r);
  } else {
    if (bit12 || bit22) {
      zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
    } else {
      zcopy(tab[1][bit21],r);
      zsqmod(*r,n,r);
    }
    zsqmod(*r,n,r);
  }

#define MOD_THIRD \
	mask >>= 1; \
	if (!mask) { \
		block --; \
                if (!block) { \
			zsqmod(*r,n,r); \
			if (bit11 || bit21) { \
				if (bit12 || bit22) { \
				  zsqmod(*r,n,r); \
				  zmulmod(tab[(bit11<<1)+bit12][(bit21<<1)+bit22],*r,n,r); \
				} else { \
				  zmulmod(tab[bit11][bit21],*r,n,r); \
				  zsqmod(*r,n,r); \
				} \
			} else { \
				zsqmod(*r,n,r); \
				if (bit12 || bit22) { \
					zmulmod(tab[bit12][bit22],*r,n,r); \
				} \
			} \
			return; \
		} \
		mask = ( 1L<<(NBITS-1) ); \
	} \
	bit13 = (e1[block]&mask) ? 1 : 0; \
        bit23 = (e2[block]&mask) ? 1 : 0;

	MOD_FIRST
	  MOD_SECOND
	  MOD_THIRD

	  for (;;) {
	    zsqmod(*r,n,r);
	    if (bit11 || bit21) {
	      if (bit13 || bit23) {
		zsqmod(*r,n,r);
		zsqmod(*r,n,r);
		zmulmod(tab[(bit11<<2)+(bit12<<1)+bit13]
			[(bit21<<2)+(bit22<<1)+bit23],*r,n,r);
	      } else {
		if (bit12 || bit22) {
		  zsqmod(*r,n,r);
		  zmulmod(tab[(bit11<<1)+bit12]
			  [(bit21<<1)+bit22],*r,n,r);
		} else {
		  zmulmod(tab[bit11][bit21],*r,n,r);
		  zsqmod(*r,n,r);
		}
		zsqmod(*r,n,r);
	      }
	      MOD_FIRST
		MOD_SECOND
		} else {
		  bit11 = bit12;
		  bit21 = bit22;
		  bit12 = bit13;
		  bit22 = bit23;
		}
	    MOD_THIRD
	      }

}

PUBLIC void zexpmod_doub(xx1,ee1,xx2,ee2,n,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong n;
verylong *r;
{
  liperr = 0;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      liperr = LIP_ZERO_MOD;
      /*zhalt("modulus zero in zexpmod_doub");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zexpmod_doub");*/
    zintoz(0,r);
    return;
  }
  if (zscompare(ee1,1L<<NBITS)<0 && zscompare(ee2,1L<<NBITS)<0)
    zexpmod_doub1(xx1,ee1,xx2,ee2,n,r);
  else if (ee1[0]<10 && ee2[0]<10)
    zexpmod_doub2(xx1,ee1,xx2,ee2,n,r);
  else
    zexpmod_doub3(xx1,ee1,xx2,ee2,n,r);
}

DEF_ACTION_5ARGS(zexpmod_doub) /* zmexp2 : Int * Int * Int * Int * Int -> Int */
{
  verylong r = 0;

  zexpmod_doub ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
		(verylong)EXTDATA(arg3), (verylong)EXTDATA(arg4),
		(verylong)EXTDATA(arg5), &r);
  if (liperr)
    {
      if (r)
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void zmontexp_doub1(xx1,ee1,xx2,ee2,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong *r;
{
  register long bit11, bit21;
  register long i,j,mask,block;
  static verylong tab[2][2];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp_doub1");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zmontexp_doub1");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<2;i++) for (j=0;j<2;j++)
			tab[i][j] = 0;
    non_ini = 0;
  }
#define MONT_INITPAR \
        if (zcompare(ee1,ee2)>=0) { \
		zcopy(xx1,&(tab[1][0])); \
		zcopy(xx2,&(tab[0][1])); \
                zcopy(ee1,&e1); \
                zcopy(ee1,&e2); /* create space for e2 */ \
                zcopy(ee2,&e2); \
        } else { \
		zcopy(xx2,&(tab[1][0])); \
		zcopy(xx1,&(tab[0][1])); \
                zcopy(ee2,&e2); /* create space for e2 */ \
                zcopy(ee1,&e2); \
                zcopy(ee2,&e1); \
        } \
        if (!(zscompare(e1,0))) { \
		zcopy(zr,r); \
                return; \
        } \
        for (i=e2[0]+1;i<=e1[0];i++) e2[i] = 0;

	MONT_INITPAR

	  zmontmul(tab[1][0],tab[0][1],&(tab[1][1]));

	block = z2log(e1)-1;
        mask = ( 1L<<(block % NBITS) );
        block = (block / NBITS) +1;
        /*bit11 = 1;*/
        bit21 = (e2[block]&mask) ? 1 : 0;
	zcopy(tab[1][bit21],r);

#define MONT_FIRST \
	mask >>= 1; \
	if (!mask) { \
		block --; \
		if (!block) return; \
		mask  = ( 1L <<(NBITS-1) ); \
	} \
	bit11 = (e1[block]&mask) ? 1 : 0; \
	bit21 = (e2[block]&mask) ? 1 : 0;

	MONT_FIRST
	  for (;;) {
	    zmontsq(*r,r);
	    if (bit11 || bit21) 
	      zmontmul(tab[bit11][bit21],*r,r);
	    MONT_FIRST
	      }
}

PUBLIC void zmontexp_doub2(xx1,ee1,xx2,ee2,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong *r;
{
  register long i,j,mask,block;
  register long bit11, bit12, bit21, bit22;
  static verylong tab[4][4];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp_doub2");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zmontexp_doub2");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<4;i++) for (j=0;j<4;j++)
      tab[i][j] = 0;
    non_ini = 0;
  }

  MONT_INITPAR

    zmontsq(tab[1][0],&(tab[2][0]));
  zmontsq(tab[0][1],&(tab[0][2]));
  zmontmul(tab[2][0],tab[1][0],&(tab[3][0]));
  zmontmul(tab[0][2],tab[0][1],&(tab[0][3]));
  for (j=1;j<4;j++) {
    for (i=1;i<4;i++) if ((i&1) | (j&1))
      zmontmul(tab[j][0],tab[0][i],&(tab[j][i]));
  }

  block = z2log(e1)-1;
  mask = ( 1L<<(block % NBITS) );
  block = (block / NBITS) +1;
  bit11 = 1;
  bit21 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      zcopy(tab[1][bit21],r);
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit12 = (e1[block]&mask) ? 1 : 0;
  bit22 = (e2[block]&mask) ? 1 : 0;
  if (bit12 || bit22) {
    zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
  } else {
    zcopy(tab[bit11][bit21],r);
    zmontsq(*r,r);
  }

#define MONT_SECOND \
	mask >>= 1; \
	if (!mask) { \
		block--; \
		if (!block) { \
			zmontsq(*r,r); \
			if (bit11||bit21) \
				zmontmul(tab[bit11][bit21],*r,r); \
			return; \
		} \
		mask = ( 1L<<(NBITS-1) ); \
	} \
        bit12 = (e1[block]&mask) ? 1 : 0; \
        bit22 = (e2[block]&mask) ? 1 : 0;

	MONT_FIRST
	  MONT_SECOND
	  for (;;) {
	    zmontsq(*r,r);
	    if (bit11 || bit21) {
	      if (bit12 || bit22) {
		zmontsq(*r,r);
		zmontmul(tab[(bit11<<1)+bit12][(bit21<<1)+bit22],*r,r);
	      } else {
		zmontmul(tab[bit11][bit21],*r,r);
		zmontsq(*r,r);
	      }
	      MONT_FIRST
		} else {
		  bit11 = bit12;
		  bit21 = bit22;
		}
	    MONT_SECOND
	      }
}

PUBLIC void zmontexp_doub3(xx1,ee1,xx2,ee2,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong *r;
{
  register long i,j, mask, block;
  register long bit11, bit12, bit13, bit21, bit22, bit23;
  static verylong tab[8][8];
  static long non_ini = 1;
  static verylong e1 = 0;
  static verylong e2 = 0;

  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp_doub3");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zmontexp_doub3");*/
    zintoz(0,r);
    return;
  }
  if (non_ini) {
    for (i=0;i<8;i++) for (j=0;j<8;j++)
      tab[i][j] = 0;
    non_ini = 0;
  }

  MONT_INITPAR

    zmontsq(tab[1][0],&(tab[2][0]));
  zmontsq(tab[0][1],&(tab[0][2]));
  for (i=3;i<8;i+=2) zmontmul(tab[2][0],tab[i-2][0],&(tab[i][0]));
  for (i=3;i<8;i+=2) zmontmul(tab[0][2],tab[0][i-2],&(tab[0][i]));
  for (i=1;i<8;i++) zmontmul(tab[0][1],tab[1][i-1],&(tab[1][i]));
  for (i=1;i<8;i+=2) zmontmul(tab[0][i],tab[2][0],&(tab[2][i]));
  for (i=1;i<8;i++) zmontmul(tab[0][1],tab[3][i-1],&(tab[3][i]));
  for (j=4;j<8;j++) {
    zmontmul(tab[1][0],tab[j-1][1],&(tab[j][1]));
    for (i=3;i<8;i+=2) zmontmul(tab[0][2],tab[j][i-2],&(tab[j][i]));
    j++;
    for (i=1;i<8;i++) zmontmul(tab[0][1],tab[j][i-1],&(tab[j][i]));
  }
  block = z2log(e1)-1;
  mask = ( 1L<<(block % NBITS) );
  block = (block / NBITS) +1;
  bit11 = 1;
  bit21 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      zcopy(tab[1][bit21],r);
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit12 = (e1[block]&mask) ? 1 : 0;
  bit22 = (e2[block]&mask) ? 1 : 0;
  mask >>= 1;
  if (!mask) {
    block --;
    if (!block) {
      if (bit12 || bit22) {
	zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
      } else {
	zcopy(tab[bit11][bit21],r);
	zmontsq(*r,r);
      }
      return;
    }
    mask = ( 1L<<(NBITS-1) );
  }
  bit13 = (e1[block]&mask) ? 1 : 0;
  bit23 = (e2[block]&mask) ? 1 : 0;
  if (bit13 || bit23) {
    zcopy(tab[4+(bit12<<1)+bit13][(bit21<<2)+(bit22<<1)+bit23],r);
  } else {
    if (bit12 || bit22) {
      zcopy(tab[2+bit12][(bit21<<1)+bit22],r);
    } else {
      zcopy(tab[1][bit21],r);
      zmontsq(*r,r);
    }
    zmontsq(*r,r);
  }
  
#define MONT_THIRD \
	mask >>= 1; \
	if (!mask) { \
		block --; \
                if (!block) { \
			zmontsq(*r,r); \
			if (bit11 || bit21) { \
				if (bit12 || bit22) { \
				  zmontsq(*r,r); \
				  zmontmul(tab[(bit11<<1)+bit12][(bit21<<1)+bit22],*r,r); \
				} else { \
				  zmontmul(tab[bit11][bit21],*r,r); \
				  zmontsq(*r,r); \
				} \
			} else { \
				zmontsq(*r,r); \
				if (bit12 || bit22) { \
					zmontmul(tab[bit12][bit22],*r,r); \
				} \
			} \
			return; \
		} \
		mask = ( 1L<<(NBITS-1) ); \
	} \
	bit13 = (e1[block]&mask) ? 1 : 0; \
        bit23 = (e2[block]&mask) ? 1 : 0;

	MONT_FIRST
	  MONT_SECOND
	  MONT_THIRD

	  for (;;) {
	    zmontsq(*r,r);
	    if (bit11 || bit21) {
	      if (bit13 || bit23) {
		zmontsq(*r,r);
		zmontsq(*r,r);
		zmontmul(tab[(bit11<<2)+(bit12<<1)+bit13]
			 [(bit21<<2)+(bit22<<1)+bit23],*r,r);
	      } else {
		if (bit12 || bit22) {
		  zmontsq(*r,r);
		  zmontmul(tab[(bit11<<1)+bit12]
			   [(bit21<<1)+bit22],*r,r);
		} else {
		  zmontmul(tab[bit11][bit21],*r,r);
		  zmontsq(*r,r);
		}
		zmontsq(*r,r);
	      }
	      MONT_FIRST
		MONT_SECOND
		} else {
		  bit11 = bit12;
		  bit21 = bit22;
		  bit12 = bit13;
		  bit22 = bit23;
		}
	    MONT_THIRD
	      }

}

PUBLIC void zmontexp_doub(xx1,ee1,xx2,ee2,r)
verylong xx1;
verylong ee1;
verylong xx2;
verylong ee2;
verylong *r;
{
  liperr = 0;
  if (!zn)
    {
      liperr = LIP_UNDEF_MONT_MOD;
      /*zhalt("undefined Montgomery modulus in zmontexp_doub");*/
      return;
    }
  if ((zscompare(ee1,0)<0) || (zscompare(ee2,0)<0)) {
    liperr = LIP_NEGATIVE_EXP;
    /*zhalt("negative exponent in zmontexp_doub");*/
    zintoz(0,r);
    return;
  }
  if (zscompare(ee1,1L<<NBITS)<0 && zscompare(ee2,1L<<NBITS)<0)
    zmontexp_doub1(xx1,ee1,xx2,ee2,r);
  else if (ee1[0]<10 && ee2[0]<10)
    zmontexp_doub2(xx1,ee1,xx2,ee2,r);
  else
    zmontexp_doub3(xx1,ee1,xx2,ee2,r);
}

DEF_ACTION_4ARGS(zmontexp_doub) /* zmontexp2 : Int * Int * Int * Int -> Int */
{
  verylong r = 0;

  zmontexp_doub ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2),
		 (verylong)EXTDATA(arg3), (verylong)EXTDATA(arg4),
		 &r);
  if (liperr)
    {
      if (r)
	{ MARKFREE (&r[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (r);
}
ACTION_END

PUBLIC void z2mul(n,rres)
verylong n;
verylong *rres;
{
  register long sn;
  register long i;
  register long carry = 0;
  verylong res = *rres;

  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      z_zero(rres);
      return;
    }
  if ((sn = n[0]) < 0)
    sn = -sn;
  zsetlength(&res, sn + 1);
  if (n == *rres) n = res;
  *rres = res;
  for (i = 1; i <= sn; i++)
    {
      if ((res[i] = (n[i] << 1) + carry) >= RADIX)
	{
	  res[i] -= RADIX;
	  carry = 1;
	}
      else
	carry = 0;
    }
  if (carry)
    res[++sn] = 1;
  if (n[0] < 0)
    res[0] = -sn;
  else
    res[0] = sn;
}

DEF_ACTION_1ARG(z2mul) /* zlsl1 : Int -> Int */
{
  verylong rr = 0;

  z2mul ((verylong)EXTDATA(arg), &rr);
  return lipMake (rr);
}
ACTION_END

PUBLIC long z2div(n,rres)
verylong n;
verylong *rres;
{
  register long sn;
  register long i;
  register long result;
  verylong res = *rres;

  if ((!n) || ((!n[1]) && (n[0] == 1)))
    {
      z_zero(rres);
      return (0);
    }
  if ((sn = n[0]) < 0)
    sn = -sn;
  zsetlength(&res, sn);
  if (n == *rres) n = res;
  *rres = res;
  result = n[1] & 1;
  for (i = 1; i < sn; i++)
    {
      res[i] = (n[i] >> 1);
      if (n[i + 1] & 1)
	res[i] += (RADIX >> 1);
    }
  if ((res[sn] = (n[sn] >> 1)))
    res[0] = n[0];
  else if (sn == 1)
    {
      res[0] = 1;
    }
  else if (n[0] < 0)
    res[0] = -sn + 1;
  else
    res[0] = sn - 1;
  return (result);
}

DEF_ACTION_1ARG(z2div) /* zasr1 : Int -> Int */
{
  verylong rr = 0;

  (void) z2div ((verylong)EXTDATA(arg), &rr);
  return lipMake (rr);
}
ACTION_END

PUBLIC long z2mod(n)
verylong n;
{
  if (!n)
    return (0);
  return (n[1] & 1);
}

DEF_ACTION_1ARG(z2mod) /* zodd : Int -> bool */
{
  return z2mod ((verylong)EXTDATA(arg))?trueValue:falseValue;
}
ACTION_END

PUBLIC void zlshift(n,k,rres)
verylong n;
long k;
verylong *rres;
{
  register long big;
  register long small;
  register long sn;
  register long i;
  register long cosmall;
  verylong res = *rres;

  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      z_zero(rres);
      return;
    }
  if (!k)
    {
      if (n != *rres)
	zcopy(n, rres);
      return;
    }
  if (k < 0)
    {
      zrshift(n, -k, rres);
      return;
    }
  if (k == 1)
    {
      z2mul(n, rres);
      return;
    }
  if ((sn = n[0]) < 0)
    sn = -sn;
  i = sn + (big = k / NBITS);
  if ((small = k - big * NBITS))
    {
      zsetlength(&res, i + 1);
      if (n == *rres) n = res;
      *rres = res;
      res[i + 1] = n[sn] >> (cosmall = NBITS - small);
      for (i = sn; i > 1; i--)
	res[i + big] = ((n[i] << small) & RADIXM) + (n[i - 1] >> cosmall);
      res[big + 1] = (n[1] << small) & RADIXM;
      for (i = big; i; i--)
	res[i] = 0;
      if (res[sn + big + 1])
	big++;
    }
  else
    {
      zsetlength(&res, i);
      if (n == *rres) n = res;
      *rres = res;
      for (i = sn; i; i--)
	res[i + big] = n[i];
      for (i = big; i; i--)
	res[i] = 0;
    }
  if (n[0] > 0)
    res[0] = n[0] + big;
  else
    res[0] = n[0] - big;
}

DEF_ACTION_2ARGS(zlshift) /* zasl : Int * int -> Int */
{
  verylong rr = 0;

  zlshift ((verylong)EXTDATA(arg1), IVAL(arg2), &rr);
  return lipMake (rr);
}
ACTION_END

PUBLIC void zrshift(n,k,rres)
verylong n;
long k;
verylong *rres;
{
  register long big;
  register long small;
  register long sn;
  register long i;
  register long cosmall;
  verylong res = *rres;

  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      z_zero(rres);
      return;
    }
  if (!k)
    {
      if (n != *rres)
	zcopy(n, rres);
      return;
    }
  if (k < 0)
    {
      zlshift(n, -k, rres);
      return;
    }
  if (k == 1)
    {
      z2div(n, rres);
      return;
    }
  big = k / NBITS;
  small = k - big * NBITS;
  if ((sn = n[0]) < 0)
    sn = -sn;
  if ((big >= sn) || ((big == sn - 1) && (!(n[sn] >> small))))
    {
      z_zero(rres);
      return;
    }
  sn -= big;
  zsetlength(&res, sn);
  if (n == *rres) n = res;
  *rres = res;
  if (small)
    {
      cosmall = NBITS - small;
      for (i = 1; i < sn; i++)
	res[i] = (n[i + big] >> small) +
	  ((n[i + big + 1] << cosmall) & RADIXM);
      if (!(res[sn] = (n[sn + big] >> small)))
	sn--;
    }
  else
    for (i = 1; i <= sn; i++)
      res[i] = n[i + big];
  if (n[0] > 0)
    res[0] = sn;
  else
    res[0] = -sn;
}

DEF_ACTION_2ARGS(zrshift) /* zasr : Int * int -> Int */
{
  verylong rr = 0;

  zrshift ((verylong)EXTDATA(arg1), IVAL(arg2), &rr);
  return lipMake (rr);
}
ACTION_END

PUBLIC long z_makeodd(nn,mm)
verylong *nn;
verylong *mm;
{
  verylong n = *nn;
  register long i;
  long shift = 1;

  if (!n || (!n[1] && (n[0] == 1)))
    {
      if (n != *mm)
	zcopy(n,mm);
      return (-1);
    }
  while (!(n[shift]))
    shift++;
  i = n[shift];
  shift = NBITS * (shift - 1);
  while (!(i & 1))
    {
      shift++;
      i >>= 1;
    }
  zrshift(n, shift, mm);
  return (shift);
}

PRIVATE long zmakeodd(nn)
verylong *nn;
{
  return z_makeodd(nn,nn);
}

DEF_ACTION_1ARG(z_makeodd) /* zmakeodd : Int -> int * Int */
{
  verylong mm = 0;
  long k;
  ExprPtr mmx, kx, x;
  ENTER;

  k = z_makeodd ((verylong *)&EXTDATA(arg), &mm);
  if (k<0)
    {
      if (mm)
	{ MARKFREE (&mm[-1]); } /*@@@*/
      return raiseLip (LIP_ZERO);
    }
  mmx = lipMake (mm);
  (void) KEEP (mmx);
  kx = KINT (k);
  x = Ktuple2 (kx, mmx);
  QUIT;
  return x;
}
ACTION_END

PUBLIC long zcomposite(mm,t,firstbase,modify_m)
verylong *mm;
long t;
long firstbase;
int modify_m;
{
  register long sm = 0;
  register long i;
  register long j;
  register long s;
  register long loc_rand = 0;
  verylong m = *mm;
  static verylong u = 0;
  static verylong m1 = 0;
  static verylong a = 0;

  if (!m || m[0] < 0)
    return (1);
  if ((m[0] == 1) && (m[1] <= RADIXROOT))
    {
      sm = m[1];
      if (sm < 2)
	return (1);
      if (sm == 2)
	return (0);
      if (!(sm & 1))
	return (1);
      i = 3;
      while (i * i <= sm)
	{
	  if (!(sm % i))
	    return (1);
	  i++;
	  i++;
	}
      return (0);
    }
  if (!(m[1] & 1))
    return (1);
  zsetlength(&u, (i = m[0]));
  zsetlength(&m1, i);
  zsetlength(&a, i);
  zsubpos(m, bigone, &m1);
  zcopy(m1, &u);
  s = zmakeodd(&u) - 1;
  if (firstbase < 0)
    firstbase = -firstbase;
  if (!firstbase && (sm = m[1]) <= RADIXROOT)
    sm = RADIXM - 3;
  if (t < 0)
    i = -t;
  else
    i = t;
  for (; i > 0; i--)
    {
      if (sm)
	{
	  zintoz((loc_rand = (3 + zrandom(sm)) & RADIXM), &a);
	  zexpmod(a, u, m, &a);
	}
      else
	{
	  if (firstbase == 2)
	    z2expmod(u, m, &a);
	  else
	    {
	      zintoz(firstbase, &a);
	      zexpmod(a, u, m, &a);
	    }
	  if ((sm = m[1]) <= RADIXROOT)
	    sm = RADIXM - 3;
	}
      if ((a[0] != 1 || a[1] != 1) && zcompare(m1, a))
	{
	  for (j = s; j; j--)
	    {
	      zsqmod(a, m, &a);
	      if (!zcompare(a, m1))
		goto nexti;
	    }
	  if (t > 0)
	    return (1);
	  else
	    {
	      zsqmod(a, m, &a);
	      zsubmod(a, bigone, m, &a);
	      zintoz(loc_rand, &u);
	      zmulmod(a, u, m, &a);
	      if (!a[1] && a[0] == 1
		  && modify_m /* added jgl 08 mar 2021 */)
		{
		  zcopy(a, &m);
		  *mm = m; /* added jgl 08 mar 2021 */
		  return (-1);
		}
	      zgcd(a, m, &u);
	      if (u[1] == 1 && u[0] == 1)
		return (1);
	      if (modify_m) /* added jgl 08 mar 2021; before
			       we only had 'zcopy(u, &m)' */
		{
		  zcopy(u, &m);
		  *mm = m;
		}
	      return (-2);
	    }
	}
nexti:		;
    }
  return (0);
}

PUBLIC long zprime(n,t,firstbase)
verylong n;
long t;
long firstbase;
{
  if (t < 0)
    t = -t;
  return (1 - zcomposite(&n, t, firstbase, 0)); /* added ',0' jgl 08 mar 2021 */
}

PRIVATE void zsmexp(a,e,bb)
long a;
verylong e;
verylong *bb;
{
  /* mixed arithmetic exponentiation, only for in zmcomposite */
  /* with mixed montgomery and ordinary arithmetic */
  register long i;
  register long j;
  register long k = 0;
  verylong b = *bb;

  if (ALLOCATE && (!zn || (zn[0]==1 && !zn[1])))
    {
      /*zhalt("wrong call to zsmexp   BUG");*/
      return;
    }
  if (ALLOCATE && (!e || (e[0]==1 && !e[1])))
    {
      zcopy(zr, bb);
      return;
    }
  if (ALLOCATE && !a)
    {
      z_zero(bb);
      return;
    }
  if ((i = e[0]) < 0)
    i = (-i);	/* negative exponent made positive ... */
  if ((i == 1) && (!(e[1])))
    zcopy(zr, &b);
  else
    {
      zintoz(a, &b);
      ztom(b, &b);
      for (; i; i--)
	{
	  j = e[i];
	  if (!k)
	    {
	      k = 1;
	      while ((k << 1) <= j)
		k <<= 1;
	    }
	  while (k >>= 1)
	    {
	      zmontsq(b, &b);
	      if (j & k)
		{
		  zsmul(b, a, &b);
		  zmod(b, zn, &b);
		}
	    }
	  k = RADIX;
	}
    }
  *bb = b;
}

PUBLIC long zmcomposite(m,t)
verylong m;
long t;
{
  register long sm = 0;
  register long i;
  register long j;
  register long s;
  static verylong u = 0;
  static verylong m1 = 0;
  static verylong a = 0;

  if (!m || m[0] < 0)
    return (1);
  if ((m[0] == 1) && (m[1] <= RADIXROOT))
    {
      sm = m[1];
      if (sm < 2)
	return (1);
      if (sm == 2)
	return (0);
      if (!(sm & 1))
	return (1);
      i = 3;
      while (i * i <= sm)
	{
	  if (!(sm % i))
	    return (1);
	  i++;
	  i++;
	}
      return (0);
    }
  if (!(m[1] & 1))
    return (1);
  zsetlength(&u, (i = m[0]));
  zsetlength(&m1, i);
  zsetlength(&a, i);
  zmkeep(m);
  zsubpos(m, zr, &m1);	/* zr is montgomery-one, m1 is
			 * montgomery-(n-1) */
  zsubpos(m, bigone, &u);
  s = zmakeodd(&u) - 1;
  if ((sm = m[1]) <= RADIXROOT)
    sm = RADIXM - 3;
  if (t < 0)
    t = -t;
  for (i = t; i > 0; i--)
    {
      zsmexp((3 + zrandom(sm)) & RADIXM, u, &a);
      if (zcompare(zr, a) && zcompare(m1, a))
	{
	  for (j = s; j; j--)
	    {
	      zmontsq(a, &a);
	      if (!zcompare(a, m1))
		goto nexti;
	    }
	  zmback();
	  return (1);
	}
    nexti:		;
    }
  zmback();
  return (0);
}

PUBLIC long zsqrts(n)
long n;
{
  register long a;
  register long ndiva;
  register long newa;
  static verylong ln=0;
  static verylong rr=0;
  static verylong dif=0;

  if (n <= 0)
    return (0);
  if (n <= 3)
    return (1);
  if (n <= 8)
    return (2);
  if (n >= RADIX)
    {
      zintoz(n,&ln);
      z_sqrt(ln,&rr,&dif);
      return(ztoint(rr));
    }
  newa = 3L << (2 * (NBITSH - 1));
  a = 1L << NBITSH;
  while (!(n & newa))
    {
      newa >>= 2;
      a >>= 1;
    }
  while (1)
    {
      newa = ((ndiva = n / a) + a) / 2;
      if (newa - ndiva <= 1)
	{
	  if (newa * newa <= n)
	    return (newa);
	  else
	    return (ndiva);
	}
      a = newa;
    }
}

DEF_ACTION_1ARG(zsqrts) /* sqrt : int -> int */
{
  INT n = IVAL(arg);

  if (n<0)
    return raiseLip (LIP_NEGATIVE_SQRT);
  return Kint (zsqrts (n));
}
ACTION_END

PUBLIC long z_sqrt(n_in,rr,ddif)
verylong n_in;
verylong *rr;
verylong *ddif;
{
  static verylong n = 0;
  static verylong a = 0;
  static verylong ndiva = 0;
  static verylong diff = 0;
  register long i;
  verylong dif = *ddif;
  verylong r = *rr;

  liperr = 0;
  if (ALLOCATE && (!n_in || (n_in[0]==1 && !n_in[1])))
    {
      z_zero(rr);
      z_zero(ddif);
      return (1);
    }
  if ((i = n_in[0]) < 0)
    {
      liperr = LIP_NEGATIVE_SQRT;
      /*zhalt("negative argument in z_sqrt");*/
      return (0);
    }
  zcopy(n_in,&n);
  zsetlength(&a, i);
  zsetlength(&ndiva, i);
  zsetlength(&diff, i);
  if (i == 1)
    {
      zintoz(i = zsqrts(n[1]), &r);
      zintoz(i * i, &diff);
      goto done;
    }
  a[(a[0] = (i + 1) / 2)] = zsqrts(n[i]) + 1;
  if (!(i & 1))
    a[a[0]] <<= NBITSH;
  if (a[a[0]] & RADIX)
    {
      a[a[0]] = 0;
      a[0]++;
      a[a[0]] = 1;
    }
  for (i = a[0] - 1; i; i--)
    a[i] = 0;
  while (1)
    {
      zquot(n, a, &ndiva, &r);
      zadd(a, ndiva, &r);
      zrshift(r, (long) 1, &r);
      if (zcompare(r, ndiva) <= 0)
	{
	  zsq(r, &diff);
	  goto done;
	}
      zsubpos(r, ndiva, &diff);
      if ((diff[0] == 1) && (diff[1] <= 1))
	{
	  zsq(r, &diff);
	  if (zcompare(diff, n) > 0)
	    {
	      zcopy(ndiva, &r);
	      zsq(r, &diff);
	    }
	  goto done;
	}
      zcopy(r, &a);
    }
done:
  *rr = r;
  zsubpos(n, diff, &dif);
  *ddif = dif;
  if (!(dif[1]) && (dif[0] == 1))
    return (1);
  return (0);
}

DEF_ACTION_1ARG(zsqrt) /* zsqrt : Int -> Int * Int */
{
  verylong r = 0;
  verylong dif = 0;
  ExprPtr rx, dx, x;
  ENTER;

  (void) z_sqrt ((verylong)EXTDATA(arg), &r, &dif);
  if (liperr)
    {
      if (r) /* added jgl 08 mar 2021 */
	{ MARKFREE (&r[-1]); } /*@@@*/
      if (dif) /* added jgl 08 mar 2021 */
	{ MARKFREE (&dif[-1]); } /*@@@*/
    return raiseLip (liperr);
    }
  rx = lipMake (r);
  (void) KEEP (rx);
  dx = lipMake (dif);
  (void) KEEP (dx);
  x = Ktuple2 (rx, dx);
  QUIT;
  return x;
}
ACTION_END

PRIVATE void spgSweep (extdata, doext)
INOUT char *extdata;
IN CONST doExtDesc *doext;
{
  switch (doext->xtag)
    {
    case X_SAVE:
      {
	struct smallprimegen *spg = (struct smallprimegen *)extdata;

	outInt ((INT)spg->primbnd);
	if (errno) return;
	outInt ((INT)spg->pindex);
	if (errno) return;
	outInt ((INT)spg->pshift);
	if (errno) return;
	outInt ((INT)spg->lastp);
	if (errno) return;
	outChars ((CHAR *)spg->lowsieve, 2*spg->primbnd);
	break;
      }
    case X_MARK:
    case X_MARKDATA:
      break;
    case X_FREE:
      MARKFREE (extdata);
      break;
    }
}

PRIVATE ExprPtr spgLoad (desc,shared,sharedref,load)
IN CONST extDesc *desc;
IN Boolean shared;
INOUT ExprPtr sharedref;
IN CONST loadAction load;
{
  INT pb, pbnd;
  struct smallprimegen *spg;
  ExprPtr e;

  pbnd = pb = inInt ();
  if (errno) return NIL;
  if (pbnd > primbnd)
    pbnd = primbnd;
  spg = (struct smallprimegen *)MALLOC (sizeof (struct smallprimegen) + 2*pb);
  spg->primbnd = pbnd;
  spg->primbndsqr = 2*pbnd*(2*pbnd+1);
  spg->lowsieve = ((char *)spg)+sizeof(struct smallprimegen);
  spg->movesieve = spg->lowsieve + pb;
  spg->pindex = inInt ();
  if (errno)
    {
    err:
      MARKFREE (spg);
      return NIL;
    }
  spg->pshift = inInt ();
  if (errno) goto err;
  spg->lastp = inInt ();
  if (errno) goto err;
  inChars ((CHAR *)spg->lowsieve, 2*pb);
  if (errno) goto err;
  e = MakeExternal (desc, (char *)spg);
  (void) KEEP (e);
  return e;
}

PUBLIC extDesc spgExtDesc = {
  (CHAR *)"smallprimegen",
  { 's','p','g',' ' }, NULL,
  NULL, /* no hash function, smallprimegens are not shared */
  NULL, /* no comparison function, smallprimegens are not shared */
  NULL, /* no print function */
  spgSweep,
  spgLoad
};

PUBLIC struct smallprimegen *new_spg P((INT prmbnd))
{
  register long i;
  register long j;
  register long jstep;
  register long jstart;
  register long ibnd;
  register char *p;
  struct smallprimegen *spg;
  INT prmbndsqr;

  if (prmbnd > primbnd)
    prmbnd = primbnd;
  if (prmbnd < 2L)
    prmbnd = 2L;
  prmbndsqr = 2*prmbnd*(2*prmbnd+1);
  spg = (struct smallprimegen *)MALLOC (sizeof(struct smallprimegen) + 2*prmbnd);
  spg->primbnd = prmbnd;
  spg->primbndsqr = prmbndsqr;
  spg->lowsieve = ((char *)spg)+sizeof(struct smallprimegen);
  spg->movesieve = spg->lowsieve + prmbnd;
  spg->pindex = -1;
  spg->pshift = 0;
  spg->lastp = 0;
  p = &spg->lowsieve[0];
  for (i = prmbnd; i; i--)
    *p++ = 1;
  jstep = 1;
  jstart = -1;
  ibnd = (zsqrts((long) (2 * prmbnd + 1)) - 3) / 2;
  for (i = 0; i <= ibnd; i++)
    {
      jstart += 2 * ((jstep += 2) - 1);
      if (spg->lowsieve[i])
	for (j = jstart; j < prmbnd; j += jstep)
	  spg->lowsieve[j] = 0;
    }
  return spg;
}

PUBLIC void zpstart(spg)
IN struct smallprimegen *spg;
{
  spg->lastp = 0;
  spg->pshift = 0;
  spg->pindex = -1;
}

PUBLIC void zpstart2(spg)
IN struct smallprimegen *spg;
{
  spg->lastp = 0;
  spg->pshift = -1;
}

PRIVATE void zpshift(spg)
IN struct smallprimegen *spg;
{
  /* auxiliary routine for prime generator */
  register long i;
  register long j;
  register long jstep;
  register long jstart;
  register long ibound;
  register char *p;
  register char *pi;

  pi = &spg->movesieve[0];
  if (!spg->pshift)
    {
      p = &spg->lowsieve[0];
      for (i = spg->primbnd; i; i--)
	*pi++ = *p++;
    }
  else
    {
      for (i = spg->primbnd; i; i--)
	*pi++ = 1;
      jstep = 3;
      ibound = spg->pshift + 2 * spg->primbnd + 1;
      for (i = 0; jstep * jstep <= ibound; i++)
	{
	  if (spg->lowsieve[i])
	    {
	      if (!((jstart = (spg->pshift + 2) / jstep + 1) & 1))
		jstart++;
	      if (jstart <= jstep)
		jstart = jstep;
	      jstart = (long) (((unsigned long)(jstart * jstep - spg->pshift - 3)) >> 1);
	      for (j = jstart; j < spg->primbnd; j += jstep)
		spg->movesieve[j] = 0;
	    }
	  jstep += 2;
	}
    }
}

PUBLIC long zpnext(spg, ovf)
IN struct smallprimegen *spg;
OUT char *ovf;
{
  if (spg->pshift < 0)
    {
      zpstart(spg);
      return (spg->lastp = 2);
    }
  if (spg->pindex < 0)
    {
      spg->pindex = 0;
      zpshift(spg);
      return (spg->lastp = 3);
    }
  for (;;)
    {
      while ((++spg->pindex) < spg->primbnd)
	{
	  if (spg->movesieve[spg->pindex])
	    {
	      spg->lastp = spg->pshift + 2 * spg->pindex + 3;
	      if (spg->lastp<0 && ovf!=NULL)
		*ovf = 1;
	      return spg->lastp;
	    }
	}
      if ((spg->pshift += 2 * spg->primbnd) > spg->primbndsqr
	  || spg->pshift<0  /* overflow */)
	{
	  if (ovf!=NULL)
	    *ovf = 1;
	  zpstart(spg);
	  return (spg->lastp = 2);
	}
      zpshift(spg);
      spg->pindex = -1;
    }
}

PUBLIC long zp(spg)
IN struct smallprimegen *spg;
{
  return (spg->lastp);
}

PRIVATE INT zprimbnd P((INT maxprime))
{
  /* primbnd is the largest n such that 2n(2n+1) <= maxprime.
     Let a=maxprime, compute x such that 2x(2x+1)=a,
     i.e. 4x^2+2x-a=0.
     Reduced discriminant = 1+4a, x = (-1+sqrt(1+4a))/4.
     */
  INT prmbnd, prmbndsqr;

  prmbnd = (long)floor (0.25 * (sqrt (1.0 + 4.0*(EXTENDED)maxprime) - 1.0));
  while (prmbndsqr = 2*prmbnd*(2*prmbnd+1), prmbndsqr>=0 && prmbndsqr<=maxprime)
    prmbnd++;
  while (prmbndsqr = 2*prmbnd*(2*prmbnd+1), prmbndsqr<0 || prmbndsqr>maxprime)
    prmbnd--;
  return prmbnd;
}

PUBLIC void zpInit()
{
  /* primbnd is the largest n such that 2n(2n+1) <= MAX_SIGNED_LONG.
     Let a=MAX_SIGNED_LONG, compute x such that 2x(2x+1)=a,
     i.e. 4x^2+2x-a=0.
     Reduced discriminant = 1+4a, x = (-1+sqrt(1+4a))/4.
     */
  primbnd = zprimbnd (MAX_SIGNED_LONG);
}

DEF_PRIVATE_ACTION_0ARG(q_nextprime)
{
  struct smallprimegen *spg = (struct smallprimegen *)EXTDATA(ENV_UNDERLYING(env));
  long p;
  char ovf = 0;

  p = zpnext (spg, &ovf);
  if (ovf)
    return raiseLip (LIP_NOT_SMALL_PRIME);
  return Kint (p);
}
ACTION_END

DEF_ACTION_1ARG(nextprime) /* primes : int -> unit -> int */
{
  struct smallprimegen *spg;
  ExprPtr env2, f;
  INT maxprime = zprimbnd (IVAL(arg));

  spg = new_spg (maxprime);
  zpstart2 (spg);
  BEGINQUICKKEEP();
  env2 = MakeExternal (&spgExtDesc, (char *)spg);
  QUICKKEEP (env2);
  f = MakeCompiled (E_COMPILED_CLOSURE, ACTION_NAME(q_nextprime), env2);
  ENDQUICKKEEP();
  return f;
}
ACTION_END

PUBLIC void zgcd(mm1,mm2,rres)
verylong mm1;
verylong mm2;
verylong *rres;
{
  register long agrb;
  register long shibl;
  static verylong aa = 0;
  static verylong bb = 0;
  static verylong cc = 0;
  verylong a;
  verylong b;
  verylong c;
  verylong d;
  long m1negative;
  long m2negative;

  if (!mm1)
    {
      if (mm2 != *rres)
	zcopy(mm2,rres);
      zabs(rres);
      return;
    }
  if (!mm2)
    {
      if (mm1 != *rres)
	zcopy(mm1,rres);
      zabs(rres);
      return;
    }
  if (mm1 == mm2)
    {
      if (mm1 != *rres)
	zcopy(mm1, rres);
      zabs(rres);
      return;
    }
  if ((m1negative = (mm1[0] < 0)))
    mm1[0] = -mm1[0];
  if ((m2negative = (mm2[0] < 0)))
    mm2[0] = -mm2[0];
  if (!(mm1[1]) && (mm1[0] == 1))
    {
      d = mm2;
      goto done;
    }
  if (!(mm2[1]) && (mm2[0] == 1))
    {
      d = mm1;
      goto done;
    }
  if ((agrb = mm1[0]) < mm2[0])
    agrb = mm2[0];
  zsetlength(&aa, agrb);
  zsetlength(&bb, agrb);
  zsetlength(&cc, agrb);
  if (mm1[0] != mm2[0])
    {
      if (mm1[0] > mm2[0])
	{
	  zcopy(mm2, &aa);
	  zmod(mm1, aa, &bb);
	}
      else
	{
	  zcopy(mm1, &aa);
	  zmod(mm2, aa, &bb);
	}
      if (!(bb[1]) && (bb[0] == 1))
	{
	  d = aa;
	  goto done;
	}
    }
  else
    {
      zcopy(mm1, &aa);
      zcopy(mm2, &bb);
    }
  if ((agrb = zmakeodd(&aa)) < (shibl = zmakeodd(&bb)))
    shibl = agrb;
  if (!(agrb = zcompare(aa, bb)))
    {
      d = aa;
      goto endshift2;
    }
  else if (agrb < 0)
    {
      a = bb;
      b = aa;
    }
  else
    {
      a = aa;
      b = bb;
    }
  c = cc;
  zsubpos(a, b, &c);
  do
    { 
      zmakeodd(&c);
      /* invariant: a>b>0, a and b odd, a = b + c.2^k, c odd
       and a, b, c are distinct pointers */
      if (!(agrb = zcompare(b, c))) /* b==c */
	{
	  d = b;
	  goto endshift;
	}
      else if (agrb > 0)
	{ /* if b > c, new a is b, new b is c, and
	     we will store new results in new c=old a, so
	     as to keep all pointers a, b, c distinct */
	  d = a;
	  a = b;
	  b = c;
	  c = d;
	  /* changed jgl 09 mar 2021; used to be:
	     a = b; b = c; c = a;
	     which seems wrong */
	}
      else
	{ /* if b < c, swap a and c, so that new a is c,
	   new b is old b, and we will store new results in
	  new c=old a */
	  d = a;
	  a = c;
	  c = d;
	}
      zsubpos(a, b, &c);
    } while (c[1] || c[0] != 1);
  d = a; /* added jgl 09 mar 2021 */
endshift:
  aa = a; bb = b; cc = c; /* added jgl 09 mar 2021 */
endshift2:
  zlshift(d, shibl, &aa);
  d = aa;
done:
  if (m1negative)
    mm1[0] = -mm1[0];
  if (m2negative)
    mm2[0] = -mm2[0];
  zcopy(d, rres);
}

DEF_ACTION_2ARGS(zgcd) /* zgcd : Int * Int -> Int */
{
  verylong rres = 0;
  verylong mm1, mm2;

  mm1 = (verylong)EXTDATA(arg1);
  mm2 = (verylong)EXTDATA(arg2);
  if (mm1[0]==1 && !mm1[1] && mm2[0]==1 && !mm2[1])
    return raiseLip (LIP_ZERO);
  zgcd (mm1, mm2, &rres);
  return lipMake (rres);
}
ACTION_END

PUBLIC void zgcdeucl(mm1,mm2,rres)
verylong mm1;
verylong mm2;
verylong *rres;
{
  register long agrb;
  static verylong aa = 0;
  static verylong bb = 0;
  verylong a;
  long m1negative;
  long m2negative;

  if (!mm1)
    {
      if (mm2 != *rres)
	zcopy(mm2, rres);
      zabs(rres);
      return;
    }
  if (!mm2)
    {
      if (mm1 != *rres)
	zcopy(mm1, rres);
      zabs(rres);
      return;
    }
  if (mm1 == mm2)
    {
      if (mm1 != *rres)
	zcopy(mm1, rres);
      zabs(rres);
      return;
    }
  if ((m1negative = (mm1[0] < 0)))
    mm1[0] = -mm1[0];
  if ((m2negative = (mm2[0] < 0)))
    mm2[0] = -mm2[0];
  if (!(mm1[1]) && (mm1[0] == 1))
    {
      a = mm2;
      goto done;
    }
  if (!(mm2[1]) && (mm2[0] == 1))
    {
      a = mm1;
      goto done;
    }
  if ((agrb = mm1[0]) < mm2[0])
    agrb = mm2[0];
  zsetlength(&aa, agrb);
  zsetlength(&bb, agrb);
  zcopy(mm1, &aa);
  zcopy(mm2, &bb);
  while (bb[1] || (bb[0] != 1))
    {
      zmod(aa, bb, &aa);
      if (!(aa[1]) && (aa[0] == 1))
	{
	  a = bb;
	  goto done;
	}
      zmod(bb, aa, &bb);
    }
  a = aa;
done:
  if (m1negative)
    mm1[0] = -mm1[0];
  if (m2negative)
    mm2[0] = -mm2[0];
  zcopy(a, rres);
}

DEF_ACTION_2ARGS(zgcdeucl) /* zgcde : Int * Int -> Int */
{
  verylong rres = 0;
  verylong mm1, mm2;

  mm1 = (verylong)EXTDATA(arg1);
  mm2 = (verylong)EXTDATA(arg2);
  if (mm1[0]==1 && !mm1[1] && mm2[0]==1 && !mm2[1])
    return raiseLip (LIP_ZERO);
  zgcdeucl (mm1, mm2, &rres);
  return lipMake (rres);
}
ACTION_END

PUBLIC long zpollardrho(n,rres,ccof,t)
verylong n;
verylong *rres;
verylong *ccof;
long t;
{
  register long i;
  register long j = 10;
  static verylong xi = 0;
  static verylong x2i = 0;
  static verylong dif = 0;
  verylong res = *rres;
  verylong cof = *ccof;
  verylong adder = &glosho[1];

  liperr = 0;
  adder[1] = 1;
  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      z_zero(rres);
      z_zero(ccof);
      return (0);
    }
  if (!n[1] && (n[0] == 1))
    {
      z_zero(rres);
      z_zero(ccof);
      return (0);
    }
  if (n[0] < 0)
    {
      zintoz((long) (-1), rres);
      zcopy(n, ccof);
      return (1);
    }
  if (!(n[1] & 1))
    {
      zintoz((long) 2, rres);
      zrshift(n, (long) 1, ccof);
      if (((*ccof)[1] != 1) || ((*ccof)[0] != 1))
	return (1);
      return (0);
    }
  zsetlength(&xi, (i = n[0] + 1));
  zsetlength(&x2i, i);
  zsetlength(&dif, i);
  i = 0;
  zmkeep(n);
  zcopy(bigone, &dif);
  ztom(dif, &xi);
  zcopy(xi, &x2i);
  while (!t || i < t)
    {
      for (; i < j; i++)
	{
	  zmontsq(xi, &xi);
	  zadd(xi, adder, &xi);
	  zmontsq(x2i, &x2i);
	  zadd(x2i, adder, &x2i);
	  zmontsq(x2i, &x2i);
	  zadd(x2i, adder, &x2i);
	  zsub(xi, x2i, &res);
	  if (res[0] < 0)
	    res[0] = -res[0];
	  if ((res[1]) || (res[0] != 1))
	    zmontmul(dif, res, &dif);
	  else
	    {
	      zcopy(bigone, &dif);
	      ztom(dif, &xi);
	      zcopy(xi, &x2i);
	      if ((adder[1] += 2) > RADIX)
		{
		  zmback();
		  return (0);
		}
	    }
	}
      if ((dif[1]) || (dif[0] != 1))
	{
	  zgcd(n, dif, &res);
	  if (((res[1] != 1) || (res[0] != 1)) && zcompare(res, n))
	    {
	      zquot(n, res, &cof, &xi);
	      if (xi[1] || xi[0] != 1)
		{
		  *rres = res;
		  *ccof = cof;
		  liperr = LIP_POLLARD_RHO_BUG;
		  /*zhalt("wrong factor in zpollardrho   BUG");*/
		  zmback();
		  return (0);
		}
	      if (zcompare(res, cof) > 0)
		{
		  *rres = cof;
		  *ccof = res;
		}
	      else
		{
		  *rres = res;
		  *ccof = cof;
		}
	      zmback();
	      return (i + 1);
	    }
	}
      else
	zcopy(bigone, &dif);
      j = i + 6;
    }
  zmback();
  return (0);
}

DEF_ACTION_2ARGS(zpollardrho) /* zpollardrho : Int * int -> (Int * Int) option */
{
  verylong n = (verylong)EXTDATA(arg1);
  long t = IVAL(arg2);
  verylong rres = 0;
  verylong ccof = 0;
  ExprPtr r, c, rc;
  long res;

  res = zpollardrho (n, &rres, &ccof, t);
  if (liperr)
    {
      if (rres)
	{
	  MARKFREE(&rres[-1]); /*@@@*/
	}
      if (ccof)
	{
	  MARKFREE(&ccof[-1]); /*@@@*/
	}
      return raiseLip (liperr);
    }
  if (res)
    {
      ENTER;

      r = lipMake (rres);
      (void) KEEP (r);
      c = lipMake (ccof);
      (void) KEEP (c);
      rc = Ktuple2 (r, c);
      (void) KEEP (rc);
      rc = Kdata(0,optttm,rc); /* SOME (r, c) */
      QUIT;
    }
  else
    {
      if (rres)
	{
	  MARKFREE(&rres[-1]); /*@@@*/
	}
      if (ccof)
	{
	  MARKFREE(&ccof[-1]); /*@@@*/
	}
      rc = none;
    }
  return rc;
}
ACTION_END

PUBLIC long ztridiv(n,cof,b1,b2, ovfp)
verylong n;
verylong *cof;
long b1;
long b2;
char *ovfp;
{
  struct smallprimegen *spg;
  long lasttried;

  if (ALLOCATE && (!n || (n[0]==1 && !n[1])))
    {
      z_zero(cof);
      return (0);
    }
  if (b1<2)
    b1 = 2;
  spg = new_spg (b2);
  zpstart(spg);
  zpstart2(spg);
  lasttried = 0;
  *ovfp = 0;
  while (lasttried < b1)
    lasttried = zpnext(spg, ovfp);
  if (!*ovfp)
    while (lasttried <= b2)
      {
	if (!zsdiv(n, lasttried, cof))
	  goto done;
	lasttried = zpnext(spg, ovfp);
	if (*ovfp)
	  goto done;
    }
  *ovfp = 1;
done:
  MARKFREE (spg);
  return (lasttried);
}

DEF_ACTION_3ARGS(ztridiv) /* ztrydiv: Int * int * int -> (int * Int) option */
{
  verylong n;
  verylong cof = 0;
  long b1, b2, r;
  ExprPtr res, f, c;
  char ovf;

  n = (verylong)EXTDATA(arg1);
  b1 = IVAL(arg2);
  b2 = IVAL(arg3);
  r = ztridiv (n, &cof, b1, b2, &ovf);
  if (!ovf)
    {
      ENTER;
      f = KINT (r);
      c = lipMake (cof);
      (void) KEEP (c);
      res = Ktuple2 (f, c);
      (void) KEEP (res);
      res = Kdata(0,optttm,res); /* SOME (f, c) */
      QUIT;
    }
  else
    {
      if (cof!=0)
	{ MARKFREE(&cof[-1]); } /*@@@*/
      res = none;
    }
  return res;
}
ACTION_END

PUBLIC long zfwrite (a,linelen,str1,len1,str2,len2,doprint,printdata)
verylong a;
long linelen;
CHAR *str1;
INT len1;
CHAR *str2;
INT len2;
printFunc doprint;
char *printdata;
{
  static verylong out = 0;
  static verylong ca = 0;
  static long outsize = 0;
  static long div = 0;
  static long ldiv;
  register long i;
  long sa;
  long result;
  long zeros;
  long strlen1 = strlen((char *)str1);
  long strlen2 = strlen((char *)str2);

  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      (*doprint) ((CHAR *)"0", 1, printdata);
      return (1);
    }
  if (!div)
    {
      div = 10;
      ldiv = 1;
      while (div * 10 < RADIXROOT)
	{
	  div *= 10;
	  ldiv++;
	}
    }
  if (linelen && linelen < ldiv)
    linelen = LINE;
  if (!out)
    out = (verylong)MALLOC((size_t)((outsize = ((long)SIZE) << 1) * (size_t)SIZEOFLONG));
  (*doprint) (str1, len1, printdata);
  if ((sa = a[0]) < 0)
    {
      sa = -sa;
      (*doprint) ((CHAR *)"~", 1, printdata);
    }
  if (ldiv * outsize < LOG10RAD * sa)
    {
      MARKFREE((void*)out);
      out = (verylong) MALLOC((size_t)((outsize = (LOG10RAD * sa) / ldiv + 2)
				       * (size_t)SIZEOFLONG));
    }
  zsetlength(&ca, sa);
  for (i = (ca[0] = sa); i; i--)
    ca[i] = a[i];
  i = -1;
  do
    {
      if ((++i) >= outsize)
	{
	  verylong out2;

	  outsize += SIZE;
	  out2 = (verylong) REALLOC((void *)out,
				    (size_t)(outsize * (size_t)SIZEOFLONG));
	  if (out2==NULL)
	    {
	      out2 = (verylong) MALLOC ((size_t)(outsize * (size_t)SIZEOFLONG));
	      safememcpy (out2, out, (size_t)((outsize-SIZE) * (size_t)SIZEOFLONG));
	      MARKFREE (out);
	    }
	  out = out2;
	}
      out[i] = zsdiv(ca, div, &ca);
    } while (ca[1] || ca[0] != 1);
  sa = 0;
  result = out[i];
  do
    {
      sa++;
    } while (result /= 10);
  result = sa + i * ldiv;;
  {
    char buf[32];

    sprintf (buf, "%ld", out[i--]);
    (*doprint) ((CHAR *)buf, strlen(buf), printdata);
  }
  for (; i >= 0; i--)
    {
      if ((zeros = 10 * out[i]))
	{
	  while (zeros < div)
	    {
	      (*doprint) ((CHAR *)"0", 1, printdata);
	      zeros *= 10;
	    }
	}
      else
	{
	  for (zeros = ldiv - 1; zeros; zeros--)
	    (*doprint) ((CHAR *)"0", 1, printdata);
	}
      {
	char buf[32];

	sprintf (buf, "%ld", out[i]);
	(*doprint) ((CHAR *)buf, strlen(buf), printdata);
      }
      sa += ldiv;
      if (linelen && (sa > linelen - strlen1) && i)
	{
	  (*doprint) ((CHAR *)"\\\n", strlen ("\\\n"), printdata);
	  (*doprint) (str2, len2, printdata);
	  sa = 0;
	  strlen1 = strlen2;
	}
    }
  return (result);
}

PUBLIC long zsign(a)
verylong a;
{
  if (ALLOCATE && (!a || (a[0]==1 && !a[1])))
    {
      return (0);
    }
  if (a[0] < 0)
    return (-1);
  if (a[0] > 1)
    return (1);
  if (a[1])
    return (1);
  return (0);
}

DEF_ACTION_1ARG(zsign) /* zsign : Int -> int */
{
  return Kint (zsign ((verylong)EXTDATA(arg)));
}
ACTION_END

PRIVATE void zabs(pa)
verylong *pa;
{
  verylong a = *pa;

  if (!a)
    return;
  if (a[0] < 0)
    a[0] = (-a[0]);
}

PUBLIC void z_abs(a,paa)
verylong a, *paa;
{
  if (a!=*paa)
    zcopy(a,paa);
  zabs(paa);
}

DEF_ACTION_1ARG(zabs) /* zabs : Int -> Int */
{
  verylong a = (verylong)EXTDATA(arg);
  verylong res = 0;

  if (a[0]>=0)
    return arg;
  z_abs (a, &res);
  return lipMake (res);
}
ACTION_END

PUBLIC long z2logs(a)
long a;
{
  long i = 0;

  if (a < 0)
    a = -a;
  while (a>=256)
    i += 8, a >>= 8;
  if (a >=16)
    i += 4, a >>= 4;
  if (a >= 4)
    i += 2, a >>= 2;
  if (a >= 2)
    i += 2;
  else if (a >= 1)
    i++;
  return (i);
}

DEF_ACTION_1ARG(z2logs) /* zsnbits : int -> int */
{
  return Kint (z2logs (IVAL (arg)));
}
ACTION_END

PUBLIC long z2log(a)
verylong a;
{
  register long la;

  if (!a)
    return (0);
  la = (a[0] > 0 ? a[0] : -a[0]);
  return ( NBITS * (la - 1) + z2logs(a[la]) );
}

DEF_ACTION_1ARG(z2log) /* znbits : Int -> int */
{
  return Kint (z2log ((verylong)EXTDATA(arg)));
}
ACTION_END

PUBLIC EXTENDED zln(a)
verylong a;
{
/*	extern EXTENDED log(); */
  register long sa;

  liperr = 0;
  if ((!a) || (a[0] == 0))
    {
      liperr = LIP_ZERO;
      /*zhalt("non-positive argument in zln");*/
      return (0.0);
    }
  sa = a[0];
  if (sa<0)
    sa = -sa;
  if (sa == 1)
    return (log((EXTENDED) (a[1])));
  return ((sa - 2) * NBITS * log((EXTENDED) 2) +
	  log(((EXTENDED)RADIX) * a[sa] + a[sa - 1]));
}

DEF_ACTION_1ARG(zln) /* zlog : Int -> num */
{
  verylong a = (verylong)EXTDATA(arg);
  EXTENDED x = zln (a);
  ExprPtr e;

  if (liperr)
    return raiseLip (liperr);
  e = Kext (&x);
  if (a[0]<0)
    {
      BEGINQUICKKEEP();
      QUICKKEEP(e);
      e = Kcomplex (e, zpi);
      ENDQUICKKEEP();
    }
  return e;
}
ACTION_END

PUBLIC long zscompare(a,b)
verylong a;
long b;
{
  static verylong c = 0;

  zintoz(b, &c);
  return (zcompare(a, c));
}

PUBLIC void zswap(a,b)
verylong *a;
verylong *b;
{
  register verylong c;

  c = *a;
  *a = *b;
  *b = c;
}

PUBLIC long ziszero(a)
verylong a;
{
  if (!a) return (1);
  if (a[1]) return (0);
  if (a[0]==1) return (1);
  return (0);
}

PUBLIC long zismone(a)
verylong a;
{
  if (!a) return 0;
  if (a[1]!=1) return 0;
  if (a[0]==-1) return 1;
  return 0;
}

PRIVATE CHAR n1bits_in_byte[] = {
#include "n1bits.h"
};
  
PUBLIC long zweights(a)
long a;
{
  register long res;
  register CHAR *s = (CHAR *)&a;

#if SIZEOFLONG==1
  res = (long)n1bits_in_byte[(unsigned int)*s];
#else
#if SIZEOFLONG==2
  res = (long) (n1bits_in_byte[(unsigned int)s[0]] + n1bits_in_byte[(unsigned int)s[1]]);
#else
#if SIZEOFLONG==4
  res = (long) (n1bits_in_byte[(unsigned int)s[0]] + n1bits_in_byte[(unsigned int)s[1]] +
		n1bits_in_byte[(unsigned int)s[2]] + n1bits_in_byte[(unsigned int)s[3]]);
#else
  {
    int i;

    res = 0;
    for (i=0; i<SIZEOFLONG; i++)
      res += (long)n1bits_in_byte[(unsigned int)*s++];
  }
#endif
#endif
#endif
  return (res);
}

DEF_ACTION_1ARG(zweights) /* zsweight : int -> int */
{
  return Kint (zweights (IVAL (arg)));
}
ACTION_END

PUBLIC long zweight(a)
verylong a;
{
  register long i;
  register long res = 0;

  if (!a) return (0);
  i = a[0];
  if (i<0) i = -i;
  for (;i;i--)
    res += zweights(a[i]);
  return (res);
}

DEF_ACTION_1ARG(zweight) /* zweight : Int -> int */
{
  return Kint (zweight ((verylong)EXTDATA(arg)));
}
ACTION_END

PUBLIC MapPtr IntToSet (a)
verylong a;
{
  MapPtr e,ii;
  long i, n, x;
  INT j, k;
  ENTER;

  e = NewMap();
  n = a[0];
  if (n<0)
    {
      liperr = LIP_NEGATIVE_EXP;
      return e;
    }
  liperr = 0;
  for (i=1, j=0; i<=n; i++)
    for (k=0, x=a[i]; k<NBITS_IN_LIP_LONG; k++, j++, x>>=1)
      if (x&1)
	{
	  ii = KINT (j);
	  e =  SetAdd (ii, e);
	  KEEPONLY(e);
	}
  QUIT;
  return e;
}

DEF_ACTION_1ARG(IntToSet) /* setofInt : Int -> int set */
{
  ExprPtr e;

  e = IntToSet ((verylong)EXTDATA(arg));
  if (liperr)
    return raiseLip(liperr);
  return e;
}
ACTION_END

MAPSWEEPFUNC(do_maxint)
{
  INT a = IVAL(x);
  INT *n = MAPDATA(INT *);

  if (a > *n)
    *n = a;
  return false;
}

MAPSWEEPFUNC(do_setint)
{
  INT a = IVAL(x);

  if (a>=0)
    {
      verylong c = *MAPDATA(verylong *);
      INT i = a/NBITS;

      c[1+i] |= (1L << (a-i*NBITS));
    }
  return false;
}

PUBLIC void SetToInt (e,cc)
MapPtr e;
verylong *cc;
{
  INT n = -1;
  long i, len;
  verylong c;

  (void) MapSweep (e, do_maxint, &n);
  len = n/NBITS+1;
  zsetlength (cc, len);
  c = *cc;
  c[0] = len;
  for (i=1; i<=len; i++)
    c[i] = 0;
  (void) MapSweep (e, do_setint, cc);
}

DEF_ACTION_1ARG(SetToInt) /* Intofset : (int -m> 'a) -> Int */
{
  verylong c = 0;

  SetToInt (arg, &c);
  return lipMake (c);
}
ACTION_END

PUBLIC void znot(a,b) /* In two's complement, ~a = -a-1. */
verylong a;
verylong *b;
{
  zsadd (a, (long)1, b);
  znegate (b);
}

DEF_ACTION_1ARG(znot) /* znot : Int -> Int */
{
  verylong bb = 0;

  znot ((verylong)EXTDATA(arg), &bb);
  return lipMake (bb);
}
ACTION_END

PRIVATE void zand_pos_pos (a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa;
  register long sb;
  register long sm;

  if (ziszero(a) || ziszero(b)) {
    z_zero(cc);
    return;
  }
  sa = a[0];
  sb = b[0];
  sm = (sa > sb ? sb : sa );
  zsetlength(&c, sm);
  *cc = c;
  for (sa = 1; sa <= sm; sa ++) 
    c[sa] = a[sa] & b[sa];
  while ((sm > 1) && (!(c[sm])))
    sm --;
  c[0] = sm;
}

PRIVATE void zand_pos_neg (a, b, cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, i;

  if (ziszero (a))
    {
      z_zero (cc);
      return;
    }
  if (zismone (b))
    {
      zcopy (a, cc);
      return;
    }
  sa = a[0];
  sb = -b[0];
  if (sb>sa) sb = sa;
  zsetlength(&c, sa);
  *cc = c;
  /* We now convert b = - (b1 b2 ... bn) in two's complement into c:
     this is RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ... */
  c[1] = a[1] & (RADIX-b[1]);
  for (i = 2; i<=sb; i++)
    c[i] = a[i] & ~b[i]; /* RADIX-1-x = ~x, except for the high bits,
				       which do not matter anyway. */
  for (; i<=sa; i++)
    c[i] = a[i];
  while ((sa > 1) && (!(c[sa])))
    sa--;
  c[0] = sa;
}

PRIVATE void zand_neg_neg (a, b, cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, i;

  if (zismone (a))
    {
      zcopy (b, cc);
      return;
    }
  if (zismone (b))
    {
      zcopy (a, cc);
      return;
    }
  sa = -a[0];
  sb = -b[0];
  if (sa < sb)
    {
      verylong aux = a;
      a = b;
      b = aux;
      i = sa;
      sa = sb;
      sb = i;
    }
  zsetlength(&c, sa);
  *cc = c;
  /* We now convert a and b in two's complement into c:
     these are
     a = RADIX-a1, RADIX-1-a2, ..., RADIX-1-am, RADIX-1, RADIX-1, ...
     b = RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     Assuming n<=m,
     c = -(RADIX-(RADIX-a1 & RADIX-b1), RADIX-1-(RADIX-1-a2 & RADIX-1-b2), ...
     RADIX-1-(RADIX-1-an & RADIX-1-bn),
     an+1, ..., am, 0, 0, ...)
     */
  c[1] = RADIX-((RADIX-a[1]) & (RADIX-b[1]));
  for (i=2; i<=sb; i++)
    /* computing ci = RADIX-1-(RADIX-1-ai & RADIX-1-bi) amounts to computing
       ~(~ai & ~bi) [up to the high bits] = ai | bi. */
    c[i] = a[i] | b[i];
  for (; i<=sa; i++)
    c[i] = a[i];
  while ((sa > 1) && (!(c[sa])))
    sa--;
  c[0] = -sa;
}

PUBLIC void zand(a,b,cc)
verylong a,b;
verylong *cc;
{
  if (a[0]<0)
    if (b[0]<0)
      zand_neg_neg (a,b,cc);
    else zand_pos_neg (b,a,cc);
  else if (b[0]<0)
    zand_pos_neg (a,b,cc);
  else zand_pos_pos (a,b,cc);
}

DEF_ACTION_2ARGS(zand) /* zand : Int * Int -> Int */
{
  verylong c = 0;

  zand ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake(c);
}
ACTION_END

PRIVATE void zxor_pos_pos (a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa;
  register long sb;
  register long sm;
  register long la;
  register long i;

  if (ziszero(a)) {
    zcopy(b,cc);
    return;
  }
  if (ziszero(b)) {
    zcopy(a,cc);
    return;
  }
  sa = a[0];
  sb = b[0];
  if (sa > sb) {
    la = sa;
    sm = sb;
  } else {
    la = sb;
    sm = sa;
  }
  zsetlength(&c, la);
  *cc = c;
  for (i = 1; i <= sm; i ++)
    c[i] = a[i] ^ b[i];
  if (sa > sb)
    for (;i <= la; i++) c[i] = a[i];
  else
    for (;i <= la; i++) c[i] = b[i];
  while ((la > 1) && (!(c[la])))
    la--;
  c[0] = la;
}

PRIVATE void zxor_pos_neg (a,b,cc)
verylong a,b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, la, sm, i;

  if (ziszero(a))
    {
      zcopy (b, cc);
      return;
    }
  if (zismone(b))
    {
      znot (a, cc);
      return;
    }
  sa = a[0];
  sb = -b[0];
  if (sa > sb) {
    la = sa;
    sm = sb;
  } else {
    la = sb;
    sm = sa;
  }
  zsetlength(&c, la);
  *cc = c;
  /* Write b = - (b1 b2 ... bn) as
     RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     So a^b = a1^(RADIX-b1), a2^(RADIX-1-b2), ..., am^(RADIX-1-bm), RADIX-1-bm+1,
     ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     = -(RADIX-a1^(RADIX-b1), RADIX-1-a2^(RADIX-1-b2), ..., RADIX-1-am^(RADIX-1-bm),
     bm+1, ..., bn, 0, ...)
     */
  c[1] = RADIX - (a[1] ^ (RADIX-b[1]));
  for (i=2; i<=sm; i++)
    c[i] = a[i] ^ b[i]; /* = (RADIX-1)-a[i] ^ ((RADIX-1)-b[i]) = ~(a[i] ^ ~b[i])
			   up to the high bits, = a[i] ^ b[i]. */
  if (sa<sb)
    for (; i<=la; i++)
      c[i] = b[i];
  else for (; i<=la; i++)
    c[i] = a[i]; /* = RADIX-1-a[i]^(RADIX-1) = ~(a[i] ^ ~0) up to the high bits,
		    = a[i] ^ 0 = a[i]; */
  while ((la > 1) && (!(c[la])))
    la--;
  c[0] = -la;
}

PRIVATE void zxor_neg_neg (a,b,cc)
verylong a,b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, i;

  if (zismone(a))
    {
      znot (b,cc);
      return;
    }
  if (zismone(b))
    {
      znot (a,cc);
      return;
    }
  sa = -a[0];
  sb = -b[0];
  if (sa < sb)
    {
      verylong aux = a;
      a = b;
      b = aux;
      i = sa;
      sa = sb;
      sb = i;
    }
  zsetlength(&c, sa);
  *cc = c;
  /* We now convert a and b in two's complement into c:
     these are
     a = RADIX-a1, RADIX-1-a2, ..., RADIX-1-am, RADIX-1, RADIX-1, ...
     b = RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     Assuming n<=m,
     c = (RADIX-a1 ^ RADIX-b1), (RADIX-1-a2 ^ RADIX-1-b2), ...
     (RADIX-1-an ^ RADIX-1-bn),
     an+1, ..., am, 0, 0, ...
     */
  c[1] = (RADIX-a[1]) ^ (RADIX-b[1]);
  for (i=2; i<=sb; i++)
    /* computing ci = (RADIX-1-ai ^ RADIX-1-bi) amounts to computing
       (~ai ^ ~bi) [up to the high bits] = (ai ^ bi). */
    c[i] = a[i] ^ b[i];
  for (; i<=sa; i++)
    c[i] = a[i];
  while ((sa > 1) && (!(c[sa])))
    sa--;
  c[0] = sa;
}

PUBLIC void zxor(a,b,cc)
verylong a,b;
verylong *cc;
{
  if (a[0]<0)
    if (b[0]<0)
      zxor_neg_neg (a,b,cc);
    else zxor_pos_neg (b,a,cc);
  else if (b[0]<0)
    zxor_pos_neg (a,b,cc);
  else zxor_pos_pos (a,b,cc);
}

DEF_ACTION_2ARGS(zxor) /* zxor : Int * Int -> Int */
{
  verylong c = 0;

  zxor ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake(c);
}
ACTION_END

PRIVATE void zor_pos_pos (a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa;
  register long sb;
  register long sm;
  register long la;
  register long i;

  if (ziszero(a)) {
    zcopy(b,cc);
    return;
  }
  if (ziszero(b)) {
    zcopy(a,cc);
    return;
  }
  sa = a[0];
  sb = b[0];
  if (sa > sb) {
    la = sa;
    sm = sb;
  } else {
    la = sb;
    sm = sa;
  }
  zsetlength(&c, la);
  *cc = c;
  for (i = 1; i <= sm; i ++)
    c[i] = a[i] | b[i];
  if (sa > sb)
    for (;i <= la; i++) c[i] = a[i];
  else
    for (;i <= la; i++) c[i] = b[i];
  c[0] = la;
}

PRIVATE void zor_pos_neg (a,b,cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, i;

  if (ziszero(a) || zismone(b))
    {
      zcopy(b,cc);
      return;
    }
  sa = a[0];
  sb = -b[0];
  if (sa > sb) sa = sb;
  zsetlength(&c, sb);
  *cc = c;
  /* Write b = - (b1 b2 ... bn) as
     RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     So a|b = a1|(RADIX-b1), a2|(RADIX-1-b2), ..., am|(RADIX-1-bm), RADIX-1-bm+1,
     ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     = -(RADIX-a1|(RADIX-b1), RADIX-1-a2|(RADIX-1-b2), ..., RADIX-1-am|(RADIX-1-bm),
     bm+1, ..., bn, 0, ...)
     */
  c[1] = RADIX - (a[1] | (RADIX-b[1]));
  for (i=2; i<=sa; i++)
    c[i] = ~a[i] & b[i]; /* = (RADIX-1) - a[i] | ((RADIX-1)-b[i]) = ~(a[i] | ~b[i])
			    up to the high bits = ~a[i] & b[i]. */
  for (; i<=sb; i++)
    c[i] = b[i];
  while (sb>1 && !c[sb])
    sb--;
  c[0] = -sb;
}

PRIVATE void zor_neg_neg (a, b, cc)
verylong a;
verylong b;
verylong *cc;
{
  verylong c = *cc;
  register long sa, sb, i;

  if (zismone (a))
    {
      zcopy (a, cc);
      return;
    }
  if (zismone (b))
    {
      zcopy (b, cc);
      return;
    }
  sa = -a[0];
  sb = -b[0];
  if (sa < sb)
    {
      verylong aux = a;
      a = b;
      b = aux;
      i = sa;
      sa = sb;
      sb = i;
    }
  zsetlength(&c, sb);
  *cc = c;
  /* We now convert a and b in two's complement into c:
     these are
     a = RADIX-a1, RADIX-1-a2, ..., RADIX-1-am, RADIX-1, RADIX-1, ...
     b = RADIX-b1, RADIX-1-b2, ..., RADIX-1-bn, RADIX-1, RADIX-1, ...
     Assuming n<=m,
     c = -(RADIX-(RADIX-a1 | RADIX-b1), RADIX-1-(RADIX-1-a2 | RADIX-1-b2), ...
     RADIX-1-(RADIX-1-an | RADIX-1-bn), 0, 0, ...)
     */
  c[1] = RADIX-((RADIX-a[1]) | (RADIX-b[1]));
  for (i=2; i<=sb; i++)
    /* computing ci = RADIX-1-(RADIX-1-ai | RADIX-1-bi) amounts to computing
       ~(~ai | ~bi) [up to the high bits] = ai | bi. */
    c[i] = a[i] & b[i];
  while ((sb > 1) && (!(c[sb])))
    sb--;
  c[0] = -sb;
}

PUBLIC void zor(a,b,cc)
verylong a,b;
verylong *cc;
{
  if (a[0]<0)
    if (b[0]<0)
      zor_neg_neg (a,b,cc);
    else zor_pos_neg (b,a,cc);
  else if (b[0]<0)
    zor_pos_neg (a,b,cc);
  else zor_pos_pos (a,b,cc);
}

DEF_ACTION_2ARGS(zor) /* zor : Int * Int -> Int */
{
  verylong c = 0;

  zor ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake(c);
}
ACTION_END

PRIVATE void zlowbits_pos (a,b,cc)
verylong a;
long b;
verylong *cc;
{
  verylong c = *cc;
  register long bl;
  register long wh;
  register long sa;

  if (ziszero(a) || (b<=0)) {
    zintoz(0,cc);
    return;
  }
  bl = ((b+NBITS-1)/NBITS);
  wh = b - NBITS*bl+NBITS;
  sa = a[0];
  if (sa < bl) {
    zcopy(a,cc);
    return;
  }
  zsetlength(&c, bl);
  *cc = c;
  for (sa=1;sa<bl;sa++)
    c[sa] = a[sa];
  c[bl] = a[bl]&((1L<<wh)-1);
  while ((bl>1) && (!c[bl]))
    bl --;
  c[0] = bl;
}

PRIVATE void zlowbits_neg (a,b,cc)
verylong a;
long b;
verylong *cc;
{
  verylong c = *cc;
  register long bl;
  register long wh;
  register long sa, sm, i;

  if (b<=0)
    {
      zintoz (0, cc);
      return;
    }
  bl = ((b+NBITS-1)/NBITS);
  wh = b - NBITS*bl+NBITS;
  zsetlength (&c, bl);
  *cc = c;
  /* a = RADIX-a1, RADIX-1-a2, ..., RADIX-1-an, RADIX-1, ..., RADIX-1
   */
  if (bl==1)
    c[1] = (RADIX-a[1]) & ((1L<<wh)-1);
  else
    {
      c[1] = RADIX-a[1];
      sa = a[0];
      sm = bl-1;
      if (sm>sa) sm = sa;
      for (i=2; i<=sm; i++)
	c[i] = (RADIX-1)-a[i];
      for (; i<bl; i++)
	c[i] = RADIX-1;
      c[bl] = (bl<=sa)?(~a[bl] & ((1L<<wh)-1)):((1L<<wh)-1);
      while ((bl>1) && (!c[bl]))
	bl--;
    }  
  c[0] = bl;
}

PUBLIC void zlowbits(a,b,cc)
verylong a;
long b;
verylong *cc;
{
  if (a[0]<0)
    zlowbits_neg (a,b,cc);
  else zlowbits_pos (a,b,cc);
}

DEF_ACTION_2ARGS(zlowbits) /* zlowbits : Int * int -> Int */
{
  verylong c = 0;

  zlowbits ((verylong)EXTDATA(arg1), IVAL(arg2), &c);
  return lipMake (c);
}
ACTION_END

PUBLIC long zshighbits(a,b)
verylong a;
long b;
{
  long l;
  static verylong aa=0;

  if (ziszero(a) || (b<0))
    return(0);
  if (b>NBITS)
    b = NBITS;
  zcopy(a,&aa);
  zabs(&aa);
  l=z2log(aa);
  if (l > b)
    zrshift(aa,l-b,&aa);
  return (ztoint(aa));
}

PUBLIC void zhighbits(a,b,c)
verylong a;
long b;
verylong *c;
{
  long l;

  if (ziszero(a) || (b<0)) {
    z_zero(c);
    return;
  }
  zcopy(a,c);
  zabs(c);
  l = z2log(*c);
  if (l > b)
    zrshift(*c,l-b,c);
}

DEF_ACTION_2ARGS(zhighbits) /* zhighbits : Int * int -> Int */
{
  verylong c = 0;

  zhighbits ((verylong)EXTDATA(arg1), IVAL(arg2), &c);
  return lipMake (c);
}
ACTION_END

#ifdef UNUSED
/* appears to be code for concatenation of bit sequences */
PUBLIC void zcat(a,b,c)
verylong a;
verylong b;
verylong *c;
{
  static verylong aux = 0; 

  if (ziszero(a)) {
    zcopy(b,c);
    return;
  }
  if (ziszero(b)) {
    /*zlshift(a,1,c);*/
    zcopy(a,c);
    return;
  }
  zlshift(a,z2log(b),&aux);
  zor (aux, b, c);
}

DEF_ACTION_2ARGS(zcat) /* zcat : Int * Int -> Int */
{
  verylong c = 0;

  zcat ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2), &c);
  return lipMake (c);
}
ACTION_END
#endif

PRIVATE long zbit_pos (a,p)
verylong a;
long p;
{
  register long bl;
  register long wh;
  register long sa;

  if (ziszero(a)) return (0);
  if (p < 0) return 0; /* p = -p; */
  bl = (p/NBITS);
  wh = 1L << (p - NBITS*bl);
  bl++;
  sa = a[0];
  if (sa < bl) return (0);
  if (a[bl] & wh) return (1);
  return (0);
}

PRIVATE long zbit_neg (a,p)
verylong a;
long p;
{
  register long bl;
  register long wh;
  register long sa;

  if (zismone(a)) return 1;
  if (p < 0) return 0; /* p = -p; */
  bl = (p/NBITS);
  wh = 1L << (p - NBITS*bl);
  bl++;
  sa = -a[0];
  if (sa < bl) return 1;
  if (bl==1)
    {
      if ((RADIX-a[1]) & wh)
	return 1;
    }
  else
    {
      if (~a[bl] & wh) return 1;
    }
  return (0);
}

PUBLIC long zbit(a,p)
verylong a;
long p;
{
  if (a[0]<0)
    return zbit_neg (a,p);
  else return zbit_pos (a,p);
}

DEF_ACTION_2ARGS(zbit) /* zbit : Int * int -> bool */
{
  return zbit ((verylong)EXTDATA(arg1), IVAL(arg2))?trueValue:falseValue;
}
ACTION_END

PRIVATE void zsetbit_pos(a,b)
verylong *a;
long b;
{
  register long bl;
  register long wh;
  register long sa;

  bl = (b/NBITS);
  wh = 1L << (b - NBITS*bl);
  bl++;
  sa = (*a)[0];
  if (sa >= bl)
    (*a)[bl] |= wh;
  else
    {
      zsetlength(a,bl);
      sa++;
      for (;sa<=bl;sa++) (*a)[sa]=0;
      (*a)[0] = bl;
      (*a)[bl] |= wh;
    }
}

PRIVATE void zsetbit_neg (a,b)
verylong *a;
long b;
{
  register long bl, wh, sa;

  bl = b/NBITS;
  wh = 1L << (b-NBITS*bl);
  bl++;
  sa = -(*a)[0];
  if (sa<bl)
    ; /* nop */
  else
    {
      if (bl==1) /* a = RADIX-a0, RADIX-1-a1, ..., RADIX-1-an, RADIX-1, RADIX-1, ... */
	(*a)[1] = RADIX-((RADIX-(*a)[1]) | wh);
      else
	(*a)[bl] &= ~wh; /* = (RADIX-1)-(((RADIX-1)-(*a)[bl]) | wh)
			    = ~(~(*a)[bl] | wh) up to the high bits = (*a)[bl] & ~wh. */
      while (sa>1 && !(*a)[sa])
	sa--;
      (*a)[0] = -sa;
    }
}

PUBLIC void zsetbit(a,b)
verylong *a;
long b;
{
  if (b<0)
    return;
  if ((*a)[0]<0)
    zsetbit_neg (a,b);
  else zsetbit_pos (a,b);
}

DEF_ACTION_2ARGS(zsetbit) /* zaddbit : Int * int -> Int */
{
  verylong c = 0;

  zcopy ((verylong)EXTDATA(arg1), &c);
  zsetbit (&c, IVAL(arg2));
  return lipMake (c);
}
ACTION_END

PRIVATE void zclrbit_pos (a,b)
verylong *a;
long b;
{
  register long bl, wh, sa;

  bl = b/NBITS;
  wh = 1L << (b-NBITS*bl);
  bl++;
  sa = (*a)[0];
  if (sa<bl)
    ; /* nop */
  else
    {
      (*a)[bl] &= ~wh;
      while (sa>1 && !(*a)[sa])
	sa--;
      (*a)[0] = sa;
    }
}

PRIVATE void zclrbit_neg(a,b)
verylong *a;
long b;
{
  register long bl;
  register long wh;
  register long sa;

  bl = (b/NBITS);
  wh = 1L << (b - NBITS*bl);
  bl++;
  sa = -(*a)[0];
  if (bl==1)
    (*a)[1] = RADIX-((RADIX-(*a)[1]) & ~wh);
  else if (sa >= bl)
    (*a)[bl] |= wh; /* (*a)[bl] = (RADIX-1)-(((RADIX-1)-(*a)[bl]) & ~wh
		       = ~(~(*a)[bl] & ~wh) up to the high bits = (*a)[bl] | wh. */
  else
    {
      zsetlength(a,bl);
      sa++;
      for (;sa<=bl;sa++) (*a)[sa]=0;
      (*a)[0] = -bl;
      (*a)[bl] |= wh;
    }
}

PUBLIC void zclrbit(a,b)
verylong *a;
long b;
{
  if (b<0)
    return;
  if ((*a)[0]<0)
    zclrbit_neg (a,b);
  else zclrbit_pos (a,b);
}

DEF_ACTION_2ARGS(zclrbit) /* zrembit : Int * int -> Int */
{
  verylong c = 0;

  zcopy ((verylong)EXTDATA(arg1), &c);
  zclrbit (&c, IVAL(arg2));
  return lipMake (c);
}
ACTION_END

PRIVATE CHAR bitrev_in_byte[] = {
#include "bitrev.h"
};

PRIVATE void zrev2(a,out)
CHAR a[2];
CHAR out[2];
{
  out[0] = bitrev_in_byte[(unsigned int)a[1]];
  out[1] = bitrev_in_byte[(unsigned int)a[0]];
}

PRIVATE void zrev4(a,out)
CHAR a[4];
CHAR out[4];
{
  zrev2(&a[0], &out[2]);
  zrev2(&a[2], &out[0]);
}

PRIVATE unsigned long zrevlong(aa)
long aa;
{
  unsigned long res;

#if SIZEOFLONG==1
  res = (unsigned long)bitrev_in_byte[(unsigned int)aa];
#elsif SIZEOFLONG==2
  zrev2 ((CHAR *)&aa, (CHAR *)&res);
#elsif SIZEOFLONG==4
  zrev4 ((CHAR *)&aa, (CHAR *)&res);
#else
  {
    int i = SIZEOFLONG;
    CHAR *ap = (CHAR *)&aa, *rp = ((CHAR *)&res)+SIZEOFLONG;

    while (i>=4)
      {
	zrev4 (ap, &rp[-4]);
	ap += 4;
	rp -= 4;
	i -= 4;
      }
    if (i>=2)
      {
	zrev2 (ap, &rp[-2]);
	ap += 2;
	rp -= 2;
	i -= 2;
      }
    if (i>=1)
      {
	rp[-1] = bitrev_in_byte[(unsigned int)ap[0]];
      }
  }
#endif
  return res;
}

PUBLIC long zreverses(aa,nbits)
long aa;
long nbits;
{
  register unsigned long res = zrevlong ((unsigned long)aa);

  if (nbits<0)
    nbits = 0;
  else if (nbits>BITSOFLONG)
    nbits = BITSOFLONG;
  return (long)(res >> (BITSOFLONG-nbits));
}

DEF_ACTION_2ARGS(zreverses) /* zsreverse : int * int -> int */
{
  return Kint (zreverses (IVAL(arg1), IVAL(arg2)));
}
ACTION_END

PRIVATE void zreverse_pos(a,b,nbits)
verylong a;
verylong *b;
long nbits;
{
  register long sa;
  register long i;
  register long sr;
  register long *bp;

  if (ziszero(a)) {
    z_zero(b);
    return;
  }
  sa = a[0];
  sr = (nbits+NBITS-1)/NBITS;
  if (sa>sr) sa = sr;
  zsetlength(b,sr);
  bp = &(*b)[sr];
  *bp = (long)(zrevlong (a[1]) >> (BITSOFLONG-NBITS_IN_LIP_LONG));
  for (i=2; i<=sa; i++)
    *--bp = (long)(zrevlong (a[i]) >> (BITSOFLONG-NBITS_IN_LIP_LONG));
  for (; i<=sr; i++)
    *--bp = 0;
  while (sr>1 && !(*b)[sr])
    sr--;
  (*b)[0] = sr;
  i = sr*NBITS_IN_LIP_LONG-nbits;
  if (i)
    zrshift (*b, i, b);
}

PRIVATE void zreverse_neg(a,b,nbits)
verylong a;
verylong *b;
long nbits;
{
  register long sa;
  register long i;
  register long sr;
  register long *bp;

  sa = -a[0];
  sr = (nbits+NBITS-1)/NBITS;
  if (sa>sr) sa = sr;
  zsetlength(b,sr);
  if (sr==1)
    {
      (*b)[1] = zreverses (RADIX-a[1], nbits);
      (*b)[0] = 1;
    }
  else
    {
      bp = &(*b)[sr];
      *bp = (long)(zrevlong (RADIX-a[1]) >> (BITSOFLONG-NBITS_IN_LIP_LONG));
      for (i=2; i<=sa; i++)
	*--bp = (long)(zrevlong (~a[i]) >> (BITSOFLONG-NBITS_IN_LIP_LONG));
      for (; i<=sr; i++)
	*--bp = RADIX-1;
      while (sr>1 && !(*b)[sr])
	sr--;
      (*b)[0] = sr;
      i = sr*NBITS_IN_LIP_LONG-nbits;
      if (i)
	zrshift (*b, i, b);
    }
}

PUBLIC void zreverse(a,b,nbits)
verylong a;
verylong *b;
long nbits;
{
  if (nbits<=0)
    z_zero (b);
  else if (a[0]<0)
    zreverse_neg (a,b,nbits);
  else zreverse_pos (a,b,nbits);
}

DEF_ACTION_2ARGS(zreverse) /* zreverse : Int * int -> Int */
{
  verylong b = 0;

  zreverse ((verylong)EXTDATA(arg1), &b, IVAL(arg2));
  return lipMake (b);
}
ACTION_END

PUBLIC long zjacobi(a,n)
verylong a;
verylong n;
{
  long unit = 1;
  long i;
  verylong newn = 0;
  verylong newa = 0;
  Boolean mya = false, myn = false;

  liperr = 0;
  if (!n || zsign(n) <= 0)
    {
      liperr = LIP_ZERO_OR_NEG;
      /*zhalt("non-positive second argument in zjacobi");*/
      return (0);
    }
  if ((n[1] & 1)==0)
    {
      liperr = LIP_EVEN;
      return 0;
    }
  if (!a || !zsign(a)) /* J_n(0)=0. */
    {
      return (0);
    }
  else if (a[0] == 1 && a[1] == 1) /* J_n(1) = 1. */
    {
      return (1);
    }
  else if (n[0] == 1 && n[1] == 1) /* J_1(a)=1 if a<>0. */
    {
      return (1);
    }
#ifdef I_DONT_UNDERSTAND_THIS /* is this a form of the Kronecker symbol? */
  else if ((!(a[1] & 1)) && (!(n[1] & 1))) /* J_2n (2a) = 0. */
    {
      return (0);
    }
  else if (!(n[1] & 1)) /* If a odd, J_{2^i.n} (a) = -1 if |a|<>1 mod 2^{min (3, i)}
			   otherwise = J_n (a). (why? !!!) */
    {
      zcopy(n, &newn);
      if (myn)
	{ MARKFREE(&n[-1]); } /*@@@*/
      n = newn;
      myn = true;
      i = zmakeodd(&n);
      i = (i > 3 ? 8 : 1L << i);
      if (a[1] % i != 1)
	{
	  MARKFREE (&n[-1]); /*@@@*/
	  return (-1);
	}
      goto again;
      /*return (zjacobi(a, temp));*/
    }
#endif
again: /* Here we compute J_n(a) for n odd. */
  if (a[0]==1)
    {
      if (a[1]==0) /* J_n(0) = 0. */
	{
	  unit = 0;
	ret:
	  if (mya)
	    { MARKFREE(&a[-1]); } /*@@@*/
	  if (myn)
	    { MARKFREE(&n[-1]); } /*@@@*/
	  return unit;
	}
      if (a[1]==1) /* J_n(1) = 1. */
	goto ret;
    }
  if (a[0] < 0)
    {
      newa = 0;
      zcopy(a, &newa);
      if (mya)
	{ MARKFREE(&a[-1]); } /*@@@*/
      a = newa;
      mya = true;
      a[0] = -a[0];
      /* check (n-1)/2 parity */
      if (n[1] & 2)
	unit = -unit;
      goto again;
      /*return (zjacobi(temp, n) * unit);*/
    }
  else if (!(a[1] & 1))
    {
      /* check (n*n-1)/8 parity */
      i = n[1] & 7;
      if (i==3 || i==5)
	unit = -unit;
      newa = 0;
      zcopy(a, &newa);
      if (mya)
	{ MARKFREE(&a[-1]); } /*@@@*/
      a = newa;
      mya = true;
      zrshift (a, (long)1, &a);
      goto again;
      /*return (zjacobi(temp, n) * unit);*/
    }
  else
    { /* If a and n are odd, J_n(a) = (-1)^{(a-1)(n-1)/4} J_a{n mod a}. */
      /* check ((a-1)*(n-1))/4 parity */
      if ((a[1] & 2) && (n[1] & 2))
	unit = -unit;
      newn = 0;
      zmod (n, a, &newn);
      if (myn)
	{ MARKFREE(&n[-1]); } /*@@@*/
      n = a;
      myn = mya;
      a = newn;
      mya = true;
      goto again;
    }
}

DEF_ACTION_2ARGS(zjacobi) /* zjacobi : Int * Int -> int */
{
  long res;

  res = zjacobi ((verylong)EXTDATA(arg1), (verylong)EXTDATA(arg2));
  if (liperr)
    return raiseLip (liperr);
  return Kint (res);
}
ACTION_END

PRIVATE void zrootnewton(a,b,n)
verylong a;
verylong *b;
long n;
{
  static verylong c = 0;
  static verylong d = 0;
  static verylong t0 = 0;
  static verylong t1 = 0;
  static verylong t2 = 0;
  long nbits, bit, i;

  if (!a || (a[0]==1 && !a[1]))
    {
      zintoz ((long) 0, b);
      return;
    }
  zintoz((long) 1, b);
  /* nbits = (z2log(a) + n - 1) / n;
     2^nbits would be just above nth-root (a); we approximate the initial value
     one bit at a time, initializing nbits to the value above minus 1: */
  nbits = (z2log(a) - 1) / n;
  if (nbits==0)
    {
      return;
    }
  zlshift(*b, nbits, b);

  /* Make the approximation better, one bit at a time.  This odd-looking
     termination criteria makes large nth get better initial approximation,
     which avoids slow convergence for such values.  This is taken from GMP 3.1.1. */
  bit = nbits - 1;
  for (i = 1; (n >> i) != 0; i++)
    {
      zsetbit (b, bit);
      zrshift (*b, bit, &t0);
      zsexp (t0, n, &t1);
      zlshift (t1, bit*n, &t1);
      if (zcompare (a, t1)<0)
	zclrbit (b, bit); /* clear bit */
      bit--;			/* check/set next bit */
      if (bit < 0)
	return;
    }
  zsetbit (b, bit);
  zintoz ((long)1, &t2);
  zlshift (t2, bit, &t2);
  zadd (*b, t2, b);

#if DEBUG
  /* Check that the starting approximation is >= than the root.  */
  zsexp (*b, n, &t1);
  if (zcompare (a, t1)>=0)
    abort ();
#endif
  zsadd (*b, (long)1, b);

  do
    {
      zsexp(*b, n - 1, &c);
      zquot(a, c, &c, &d);
      zsub(*b, c, &c);
      zsdiv(c, n, &c);
      zsub(*b, c, b);
      /* b := b - c
	 where c = [(b-[a/b^{n-1}])/n].
	 Since b>=nth-root (a), a/b^{n-1} = ab/b^n <= b, so [a/b^{n-1}] <= b,
	 hence c>=0.
	 Stops when c==0, ie when b-[a/b^{n-1}] <= n-1.
	 In particular, b-a/b^{n-1} <= n-1.
	 So b = nth-root (a) + eps, entailing:
	 b-a/b^{n-1} = nth-root (a) + eps - nth-root (a) / (1+eps/a)^{n-1}
	 = eps + (n-1)eps/a - n(n-1)eps^2/2a^2 + (n+1)n(n-1)eps^3/6a^3 - ...
	 At first order, means that eps is about <=a(n-1)/(a+n-1)
       */
    } while (zsign(c) > 0);
  zsexp(*b, n, &c);
  if (zcompare(c, a) > 0)
    zsadd(*b, (long) (-1), b);
}

PUBLIC void zroot(a_in,n,b)
verylong a_in;
long n;
verylong *b;
{
  static verylong a = 0;
  long signa = zsign(a_in);

  liperr = 0;
  zcopy(a_in,&a);
  if ((signa < 0) && (n & 1))
    a[0] = -a[0];
  if ((signa > 0 && n > 1) || (signa < 0 && (n & 1)))
    zrootnewton(a, b, n);
  else if (n == 1 || (signa == 0 && n > 0))
    zcopy(a, b);
  else if (n == 0)
    {
      liperr = LIP_ZERO;
      /*zhalt("dth root with d=0 in zroot");*/
      return;
    }
  else if ((signa < 0) && (!(n & 1)))
    {
      liperr = LIP_NEGATIVE_SQRT;
      /*zhalt("dth root with even d of negative number in zroot");*/
      return;
    }
  else if (signa == 0 && n < 0)
    {
      liperr = LIP_DIV_BY_ZERO;
      /*zhalt("dth root with d<0 of zero in zroot");*/
      return;
    }
  else
    zintoz((long) 0, b);
  if (signa < 0)
    {
      if (((*b)[1]) || ((*b)[0] > 1))
	(*b)[0] = -(*b)[0];
    }
}

DEF_ACTION_2ARGS(zroot) /* zroot : Int * int -> Int */
{
  verylong bb = 0;

  zroot ((verylong)EXTDATA(arg1), IVAL(arg2), &bb);
  if (liperr)
    {
      if (bb) /* added jgl 08 mar 2021 */
	{ MARKFREE (&bb[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  return lipMake (bb);
}
ACTION_END

PUBLIC long zjacobis(m,n)
long m;
long n;
{
  register long m2;
  register long m1;
  register long jcbi = 1;

  liperr = 0;
  if (n <= 0)
    {
      liperr = LIP_ZERO_OR_NEG;
      return 0;
    }
  if ((n&1)==0)
    {
      liperr = LIP_EVEN;
      return 0;
    }
  m2 = n;
  if (m==0)
    return 0;
  if (m==1)
    return 1;
  if (m2==1)
    return 1;
#ifdef I_DONT_UNDERSTAND_THIS
  if ((m&1)==0 && (m2&1)==0)
    return 0;
  if ((m2&1)==0)
    {
      long i;

      for (i=0; (m2&1)==0; i++, m2>>=1);
      i = (i > 3 ? 8 : 1L << i);
      if ((m>0 && m%i!=1) || (m<0 && (-m)%i!=1))
	return -1;
      if (m2==1)
	return 1;
      /*fallthrough*/
    }
#endif
  m1 = (m >= 0) ? (m % m2) : (m2 - ((-m) % m2));
  while (1)
    {
      switch (m1 & 15)
	{
	case 0:
	  if (m1 == 0)
	    return ((m2 == 1) ? jcbi : 0);
	  m1 >>= 4;	/* Divide numerator by 16 */
	  continue;	/* continue while loop */
	case 8:
	  m1 >>= 2;	/* Divide numerator by 4 */
	  continue;	/* continue while loop */
	case 4:
	  m1 >>= 2;	/* Divide numerator by 4 */
	  break;
	case 1:
	case 5:
	case 9:
	case 13:
	  break;
	case 2:
	case 10:
	  m1 >>= 1;
	  if ((m2 & 7) == 3 || (m2 & 7) == 5)
	    jcbi = -jcbi;
	  break;
	case 12:
	  m1 >>= 2;	/* Divide numerator by 4, */
	  /* fall through to 3 mod 4 cases */
	case 3:
	case 7:
	case 11:
	case 15:
	  if ((m2 & 2) != 0)	/* negate if  m1 == m2
				 * == 3 mod 4 */
	    jcbi = -jcbi;
	  break;
	case 6:
	case 14:
	  m1 >>= 1;
	  if ((m2 & 4) != 0)
	    jcbi = -jcbi;
	  break;
	}	/* end switch */
      {	/* m1 and m2 are odd; invert them */
	/* First divide m2 by 4, mod m1 */
	register long m3;
	m3 = ((((m2 + m1) & 3) == 0) ? m2 + m1 : m2 - m1) >> 2;
	if (m3 >= m1)
	  m3 %= m1;
	m2 = m1;
	m1 = m3;
      }
    }	/* end while(1) */
}

DEF_ACTION_2ARGS(zjacobis) /* zsjacobi : int * int -> int */
{
  long res;

  res = zjacobis (IVAL(arg1), IVAL(arg2));
  if (liperr)
    return raiseLip (liperr);
  return Kint (res);
}
ACTION_END

PUBLIC long zprobprime(a,number_of_tests)
verylong a;
long number_of_tests;
{
  long result;
  static verylong cofactor_a = 0;
  char ovf;

  if (!a || a[0] < 0)
    return (0);
  if ((a[0]==1) && (a[1]==2))
    return (1);
  if (!(a[1] & 1))
    return (0);
  if (a[0] == 1)
    {
      if ((a[1] == 3) || (a[1] == 5) || (a[1] == 7) || (a[1] == 11) )
	return (1);
      if (a[1] < 13) return (0);
      result = zsqrts(a[1]);
      if (ztridiv(a, &cofactor_a, (long) 3, result, &ovf), !ovf)
	return (0);
      return (1);
    }
  result = a[0] * 5 * NBITS;
  if (ztridiv(a, &cofactor_a, (long) 3, result, &ovf), !ovf)
    return (0);
  if (zcomposite(&a, (long) 1, (long) 2, 0)) /* added ',0' jgl 08 mar 2021 */
    return (0);
  result = !zmcomposite(a, number_of_tests);
  /*
   * if (!result) { printf("composite, but pseudo prime to the base 2:\n");
   * zwriteln(a); }
   */
  return (result);
}

DEF_ACTION_2ARGS(zprobprime) /* zprime : Int * int -> bool */
{
  return zprobprime ((verylong)EXTDATA(arg1), IVAL(arg2)) ? trueValue:falseValue;
}
ACTION_END

#ifdef HAS_PROTOTYPES
PUBLIC long zsrand (long bnd)
#else
PUBLIC long zsrand (bnd)
long bnd;
#endif
{
  long size, mask;
  unsigned long x;

  if (bnd==0)
    return randLong ();
  if (bnd<0)
    size = BITSOFLONG;
  else size = z2logs (bnd);
  mask = (1L << size)-1;
  do {
    x = randLong() & mask;
  } while (x>=(unsigned long)bnd);
  return (long)x;
}

DEF_ACTION_1ARG(zsrand) /* irand : int -> int */
{
  return Kint (zsrand (IVAL (arg)));
}
ACTION_END

PUBLIC void zrandoml(bitlength,a,gen)
long bitlength;
verylong *a;
generator *gen;
{
  register long i, n;
  register long neg = 0;
  ExprPtr in;

  if (bitlength == 0)
    {
      zintoz((long) 0, a);
      return;
    }
  if (bitlength < 0)
    {
      bitlength = -bitlength;
      neg = 1;
    }
  if (bitlength == 1)
    {
      zintoz((long) 1, a);
      return;
    }
  zintoz((long) 1, a);
  zlshift(*a, bitlength - 1, a);
  n = (*a)[0];
  if (n>1)
    switch (gen->type)
      {
      case 0:
	for (i = 1; i < n; i++)
	  (*a)[i] = (*gen->what.f)((long)RADIX);
	break;
      default:
	{
	  ENTER;

	  in = KINT((long)RADIX);
	  for (i=1; i<n; i++)
	    (*a)[i] = IVAL((*ACTION(gen->what.g)) (ENV_CONS(QENV (gen->what.g),1), in));
	  QUIT;
	}
      }
  switch (gen->type)
    {
    case 0: (*a)[n] += (*gen->what.f)((*a)[n]); break;
    default:
      {
	ENTER;

	in = KINT((*a)[n]);
	(*a)[n] += IVAL((*ACTION(gen->what.g)) (ENV_CONS(QENV (gen->what.g),1), in));
	QUIT;
      }
    }
  if (neg)
    (*a)[0] = -(*a)[0];
}

DEF_ACTION_2ARGS(zrandoml) /* zrandl : int * (int -> int) -> Int */
{
  verylong b = 0;
  generator gen;

  gen.type = 1;
  gen.what.g = arg2;
  zrandoml (IVAL(arg1), &b, &gen);
  /* This induces a memory leak if arg2 throws an exception;
     indeed, in this case b is not freed. */
  return lipMake (b);
}
ACTION_END

DEF_ACTION_1ARG(zrandoml1) /* zrandl1 : int -> Int */
{
  verylong b = 0;
  generator gen;

  gen.type = 0;
  gen.what.f = zsrand;
  zrandoml (IVAL(arg), &b, &gen);
  return lipMake (b);
}
ACTION_END

PUBLIC void zrand(bnd,a,gen)
verylong bnd;
verylong *a;
generator *gen;
{
  long b0, sb;
  register long i, n;
  ExprPtr in;

  if (bnd==NULL || (bnd[1]<=1 && (bnd[0]==1 || bnd[0]==-1)))
    {
      zintoz ((long)0, a);
      return;
    }
  b0 = bnd[0];
  if (b0<0)
    bnd[0] = -b0;
  sb = bnd[0];
  zsetlength (a, sb);
  do {
    n = sb;
    if (n>1)
      switch (gen->type)
	{
	case 0:
	  for (i = 1; i < n; i++)
	    (*a)[i] = (*gen->what.f)((long)RADIX);
	  break;
	default:
	  {
	    ENTER;

	    in = KINT((long)RADIX);
	    for (i=1; i<n; i++)
	      (*a)[i] = IVAL((*ACTION(gen->what.g)) (ENV_CONS(QENV (gen->what.g),1), in));
	    QUIT;
	  }
	}
    switch (gen->type)
      {
      case 0: (*a)[n] = (*gen->what.f) (bnd[n]+1); break;
      default:
	{
	  ENTER;

	  in = KINT(bnd[n]+1);
	  (*a)[n] = IVAL((*ACTION(gen->what.g)) (ENV_CONS(QENV (gen->what.g),1), in));
	  QUIT;
	}
      }
    while (n>1 && (*a)[n]==0)
      n--;
    (*a)[0] = n;
  } while (zcompare (*a, bnd)>=0);
  if (b0<0)
    (*a)[0] = -(*a)[0];
  bnd[0] = b0;
}

DEF_ACTION_2ARGS(z_rand) /* zrand : Int * (int -> int) -> Int */
{
  verylong a = 0;
  generator gen;

  gen.type = 1;
  gen.what.g = arg2;
  zrand ((verylong)EXTDATA(arg1), &a, &gen);
  /* This induces a memory leak if arg2 throws an exception;
     indeed, in this case a is not freed. */
  return lipMake (a);
}
ACTION_END

DEF_ACTION_1ARG(zrand1) /* zrand1 : Int -> Int */
{
  verylong a = 0;
  generator gen;

  gen.type = 0;
  gen.what.f = zsrand;
  zrand ((verylong)EXTDATA(arg), &a, &gen);
  return lipMake (a);
}
ACTION_END

PUBLIC void zrandomb(bnd,a,gen)
verylong bnd;
verylong *a;
generator *gen;
{
  static verylong big = 0;
  static verylong bnd_minus_one = 0;

  if (zscompare(bnd, (long) 1) <= 0)
    {
      zintoz((long) 0, a);
      return;
    }
  if (zscompare(bnd, (long) 2) == 0)
    {
      zintoz((long) 1, a);
      return;
    }
  zrandoml(z2log(bnd) + NBITS, &big, gen);
  zsadd(bnd, (long) (-1), &bnd_minus_one);
  zmod(big, bnd_minus_one, a);
  zsadd(*a, (long) 1, a);
}

PUBLIC long zrandomprime(bitlength,nbtests,a,gen)
long bitlength;
long nbtests;
verylong *a;
generator *gen;
{
  static verylong t = 0;
  static verylong bnd = 0;
  register long adder = 2;
  long three_mod_four = 0;

  if (bitlength < 0)
    {
      bitlength = - bitlength;
      three_mod_four = 1;
      adder = 4;
    }
  if (bitlength < 2)
    return (0);
  if (bitlength == 2)
    {
      if (three_mod_four)
	zintoz(3,a);
      else
	{
	  switch (gen->type)
	    {
	    case 0: zintoz ((*gen->what.f) ((long)2), a); break;
	    default:
	      {
		ExprPtr in;
		ENTER;

		in = KINT((long)2);
		zintoz (IVAL((*ACTION(gen->what.g)) (ENV_CONS(QENV (gen->what.g),1), in)), a);
		QUIT;
		break;
	      }
	    }
	  zsadd(*a, (long) 2, a);
	}
      return (1);
    }
  zintoz((long) 1, &bnd);
  zlshift(bnd, bitlength, &bnd);
  while (1)
    {
      if (three_mod_four)
	{
	  zrandoml(bitlength - 2, &t,gen);
	  zlshift(t, (long) 2, a);
	  zsadd(*a, (long) 3, a);
	}
      else
	{
	  zrandoml(bitlength - 1, &t,gen);
	  zlshift(t, (long) 1, a);
	  zsadd(*a, (long) 1, a);
	}
      while (zcompare(*a, bnd) < 0)
	{
	  if (zprobprime(*a, nbtests))
	    return (1);
	  zsadd(*a, adder, a);
	}
    }
}

DEF_ACTION_3ARGS(zrandomprime) /* zrandprime : int * int * (int -> int) -> Int */
{
  verylong a = 0;
  generator gen;

  gen.type = 1;
  gen.what.g = arg3;
  if (zrandomprime (IVAL(arg1), IVAL(arg2), &a, &gen)==0)
    {
      if (a) /* added jgl 08 mar 2021 */
	{ MARKFREE (&a[-1]); } /*@@@*/
      return raiseLip(LIP_PRIME_GEN_FAIL);
    }
  return lipMake (a);
}
ACTION_END

DEF_ACTION_2ARGS(zrandomprime1) /* zrandprime1 : int * int -> Int */
{
  verylong a = 0;
  generator gen;

  gen.type = 0;
  gen.what.f = zsrand;
  if (zrandomprime (IVAL(arg1), IVAL(arg2), &a, &gen)==0)
    {
      if (a) /* added jgl 08 mar 2021 */
	{ MARKFREE (&a[-1]); } /*@@@*/
      return raiseLip(LIP_PRIME_GEN_FAIL);
    }
  return lipMake (a);
}
ACTION_END

PUBLIC long zrandomqprime(lp,lq,nbtests,p,q,frac,gen)
long lp;
long lq;
long nbtests;
verylong *p;
verylong *q;
verylong *frac;
generator *gen;
{
  register long oddq = 1;
  register long cnt = 2 * lp;
  static verylong tpsizem = 0;
  static verylong tpsize = 0;
  static verylong lown = 0;
  static verylong upn = 0;
  static verylong twoq = 0;

  liperr = 0;
  if (lq < 0)
    {
      lq = z2log(*q);
      if ((!lq) || ((*q)[0] < 0))
	{
	  liperr = LIP_WRONG_Q;
	  /*zhalt("wrong q in zrandomqprime");*/
	  return (0);
	}
      if (((*q)[0] == 1) && ((*q)[1] == 1))
	{
	  if (!(zrandomprime(lp, nbtests, p, gen)))
	    return (0);
	  zcopy(*p, frac);
	  return (1);
	}
      if (-lq >= lp)
	return (0);
    }
  else if (!(zrandomprime(lq, nbtests, q, gen)))
    return (0);
  zintoz((long) 1, &tpsizem);
  zlshift(tpsizem, lp - 1, &tpsizem);
  zsadd(tpsizem, (long) (-1), &tpsizem);
  zintoz((long) 1, &tpsize);
  zlshift(tpsize, lp, &tpsize);
  zsadd(tpsize, (long) (-1), &tpsize);
  if ((*q)[1] & 1)
    zlshift(*q, (long) 1, &twoq);
  else
    {
      oddq = 0;
      zcopy(*q, &twoq);
    }
  zquot(tpsizem, twoq, &lown, frac);
  zquot(tpsize, twoq, &upn, frac);
  zsub(upn, lown, &upn);
  do
    {
      cnt--;
      if (!cnt)
	{
	  if (zscompare(upn, lp) <= 0)
	    return (0);
	}
      zrandomb(upn, p, gen);
      zadd(lown, *p, frac);
      ztimes(twoq, *frac, p);
      zsadd(*p, (long) 1, p);
    } while (!zprobprime(*p, nbtests));
  if (oddq)
    zlshift(*frac, (long) 1, frac);
  return (1);
}

DEF_ACTION_4ARGS(zrandompprime) /* zrandpprime : int * int * Int * (int -> int) -> Int * Int */
{
  /* args = (lp, nbtests, q, gen) */
  long res;
  generator gen;
  verylong p = 0;
  verylong frac = 0;

  gen.type = 1;
  gen.what.g = arg4;
  res = zrandomqprime (IVAL(arg1), (long)-1, IVAL(arg2), &p,
		       (verylong *)&EXTDATA(arg3), &frac, &gen);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (frac)
	{ MARKFREE(&frac[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr px, fracx, x;
    ENTER;

    px = lipMake (p);
    (void) KEEP(px);
    fracx = lipMake (frac);
    (void) KEEP(fracx);
    x = Ktuple2 (px, fracx);
    QUIT;
    return x;
  }
}
ACTION_END

DEF_ACTION_3ARGS(zrandompprime1) /* zrandpprime1 : int * int * Int -> Int * Int */
{
  /* args = (lp, nbtests, q) */
  long res;
  generator gen;
  verylong p = 0;
  verylong frac = 0;

  gen.type = 0;
  gen.what.f = zsrand;
  res = zrandomqprime (IVAL(arg1), (long)-1, IVAL(arg2), &p,
		       (verylong *)&EXTDATA(arg3), &frac, &gen);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (frac)
	{ MARKFREE(&frac[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr px, fracx, x;
    ENTER;

    px = lipMake (p);
    (void) KEEP(px);
    fracx = lipMake (frac);
    (void) KEEP(fracx);
    x = Ktuple2 (px, fracx);
    QUIT;
    return x;
  }
}
ACTION_END

PUBLIC long zrandomfprime(length,nbtests,frac,p,q,gen,spg)
long length;
long nbtests;
verylong frac;
verylong *p;
verylong *q;
generator *gen;
IN struct smallprimegen *spg;
{
  register long pp;
  register long qmp;
  register long bound;
  register long small_frac = 0;
  static verylong aux = 0;

  if ((zscompare(frac,2)<0) || (frac[1]&1))
    return (0);
  if (length < (NBITS>>1))
    {
      do
	{
	  if (!zrandomprime(length,nbtests,q,gen))
	    return (0);
	  ztimes(*q,frac,p);
	  zsadd(*p,1,p);
	} while (!zprobprime(*p,nbtests));
      return (1);
    }
  if (zscompare(frac,RADIXROOT) <= 0)
    small_frac = ztoint(frac);
  if ((bound = (z2log(frac)+length)*5) > RADIXROOT)
    bound = RADIXROOT;
  for (;;)
    {
      zrandoml(length-1,q,gen);
      zlshift(*q,1,q);
      zsadd(*q,1,q);
      zpstart(spg);
      while ((pp=zpnext(spg, NULL)) <= bound) 
	{
	  if (!(qmp = zsdiv(*q,pp,&aux)))
	    goto next_try;
	  if (small_frac)
	    qmp *= small_frac;
	  else
	    qmp *= zsdiv(frac,pp,&aux);
	  if (!((qmp+1)%pp))
	    goto next_try;
	}
      if (zcomposite(q,1,2,0)) /* added ',0' jgl 08 mar 2021 */
	goto next_try;
      ztimes(*q,frac,p);
      zsadd(*p,1,p);
      if (zcomposite(p,1,2,0)  /* added ',0' jgl 08 mar 2021 */
	  || zmcomposite(*q,nbtests)
	  || zmcomposite(*p,nbtests))
	goto next_try;
      return (1);
    next_try:;
    }
}

DEF_ACTION_4ARGS(zrandomfprime) /* zrandfprime : int * int * Int * (int -> int) -> Int * Int */
{
  /* args = (lq, nbtests, frac, gen) */
  generator gen;
  verylong p = 0;
  verylong q = 0;
  long res;
  struct smallprimegen *spg;

  gen.type = 1;
  gen.what.g = arg4;
  liperr = 0;
  spg = new_spg (1L << NPBITS);
  zpstart(spg);
  zpstart2(spg);
  res = zrandomfprime (IVAL(arg1), IVAL(arg2),
		       (verylong)EXTDATA(arg3), &p, &q, &gen, spg);
  MARKFREE(spg);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (q)
	{ MARKFREE(&q[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr px, qx, x;
    ENTER;

    px = lipMake (p);
    (void) KEEP(px);
    qx = lipMake (q);
    (void) KEEP(qx);
    x = Ktuple2 (px, qx);
    QUIT;
    return x;
  }
}
ACTION_END

DEF_ACTION_3ARGS(zrandomfprime1) /* zrandfprime1 : int * int * Int -> Int * Int */
{
  /* args = (lq, nbtests, frac) */
  generator gen;
  verylong p = 0;
  verylong q = 0;
  long res;
  struct smallprimegen *spg;

  gen.type = 0;
  gen.what.f = zsrand;
  liperr = 0;
  spg = new_spg (1L << NPBITS);
  zpstart(spg);
  zpstart2(spg);
  res = zrandomfprime (IVAL(arg1), IVAL(arg2),
		       (verylong)EXTDATA(arg3), &p, &q, &gen, spg);
  MARKFREE(spg);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (q)
	{ MARKFREE(&q[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr px, qx, x;
    ENTER;

    px = lipMake (p);
    (void) KEEP(px);
    qx = lipMake (q);
    (void) KEEP(qx);
    x = Ktuple2 (px, qx);
    QUIT;
    return x;
  }
}
ACTION_END

PUBLIC long zrandomgprime(lq,nbtests,small,p,q,g,gen,spg)
long lq;
long nbtests;
long small;
verylong *p;
verylong *q;
verylong *g;
generator *gen;
IN struct smallprimegen *spg;
{
  static verylong montone = 0;
  static verylong aux = 0;

  zintoz(2,&aux);
  if (!zrandomfprime(lq,nbtests,aux,p,q,gen,spg))
    return 0;
  zmkeep(*p);
  zintoz((long)1, &montone);
  ztom(montone,&montone);
  if (small)
    zcopy(montone,g);
  for (;;)
    {
      if (small)
	zaddmod(*g,montone,*p,g);
      else
	zrandomb(*p,g,gen);
      zmontsq(*g,&aux);
      if (zcompare(aux,montone))
	{
	  zmontexp_m_ary(*g,*q,&aux,0);
	  if (zcompare(aux,montone))
	    {
	      zmtoz(*g,g);
	      zmback();
	      return 1;
	    }
	}
    }
}

DEF_ACTION_4ARGS(zrandomgprime)
/* zrandgprime : int * int * bool * (int -> int) -> Int * Int * Int */
{
  /* args = (lq, nbtests, small, gen) */
  generator gen;
  verylong p = 0;
  verylong q = 0;
  verylong g = 0;
  long res;
  struct smallprimegen *spg;

  gen.type = 1;
  gen.what.g = arg4;
  liperr = 0;
  spg = new_spg (1L << NPBITS);
  zpstart(spg);
  zpstart2(spg);
  res = zrandomgprime (IVAL(arg1), IVAL(arg2), (long)(arg3==trueValue),
		       &p, &q, &g, &gen, spg);
  MARKFREE(spg);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (q)
	{ MARKFREE(&q[-1]); } /*@@@*/
      if (g)
	{ MARKFREE(&g[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr pgq[3], x;
    ENTER;

    pgq[0] = lipMake (p);
    (void) KEEP(pgq[0]);
    pgq[1] = lipMake (q);
    (void) KEEP(pgq[1]);
    pgq[2] = lipMake (g);
    (void) KEEP(pgq[2]);
    x = KtupleGnrl (3, pgq);
    QUIT;
    return x;
  }
}
ACTION_END

DEF_ACTION_3ARGS(zrandomgprime1) /* zrandgprime1 : int * int * bool -> Int * Int * Int */

{
  /* args = (lq, nbtests, small) */
  generator gen;
  verylong p = 0;
  verylong q = 0;
  verylong g = 0;
  long res;
  struct smallprimegen *spg;

  gen.type = 0;
  gen.what.f = zsrand;
  liperr = 0;
  spg = new_spg (1L << NPBITS);
  zpstart(spg);
  zpstart2(spg);
  res = zrandomgprime (IVAL(arg1), IVAL(arg2), (long)(arg3==trueValue),
		       &p, &q, &g, &gen, spg);
  MARKFREE(spg);
  if (liperr)
    {
    err:
      if (p)
	{ MARKFREE(&p[-1]); } /*@@@*/
      if (q)
	{ MARKFREE(&q[-1]); } /*@@@*/
      if (g)
	{ MARKFREE(&g[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_PRIME_GEN_FAIL;
      goto err;
    }
  {
    ExprPtr pgq[3], x;
    ENTER;

    pgq[0] = lipMake (p);
    (void) KEEP(pgq[0]);
    pgq[1] = lipMake (q);
    (void) KEEP(pgq[1]);
    pgq[2] = lipMake (g);
    (void) KEEP(pgq[2]);
    x = KtupleGnrl (3, pgq);
    QUIT;
    return x;
  }
}
ACTION_END

/*!!!*/

#define	MAXM		3000000
#define	PHASE1_BATCH	1
#define	MAXTEP		31
#define	MAXPWRP		12
#define	MAXRP		(1L << MAXPWRP)
#define	MINBOUND	100

PRIVATE verylong ecm_tex[MAXTEP];
PRIVATE verylong ecm_tey[MAXTEP];
PRIVATE verylong ecm_coef[MAXRP];
PRIVATE verylong ecm_power[MAXPWRP];
PRIVATE verylong ecm_eval[MAXPWRP];

PRIVATE long ph1set(n,rap,alpha,mu,x,f)
verylong n;
verylong *rap;
verylong *alpha;
verylong *mu;
verylong *x;
verylong *f;
{
  /* Set up elliptic curve */
  /* if 0, success, */
  /* if 1, n factored, factor in f */
  static verylong r = 0;
  static verylong s = 0;
  static verylong t1 = 0;
  static verylong t2 = 0;
  static verylong t3 = 0;
  static verylong u = 0;
  static verylong u2 = 0;
  static verylong u3 = 0;
  static verylong u4 = 0;
  generator gen;

  gen.type = 0;
  gen.what.f = zrandom;
  zrandomb(n,&s,&gen);
  zsqmod(s,n,&r);
  zsmulmod(s,6,n,&s);
  zsadd(r,6,&r);
  if (zinv(r, n, &t1))
    {
      zcopy(t1, f);
      return (1);
    }
  zmulmod(s, t1, n, &u);
  zsqmod(u, n, &u2);
  zmulmod(u, u2, n, &u3);
  zsqmod(u2, n, &u4);
  zsmul(u4, 3, &t1);
  zmod(t1, n, &t1);
  zsmul(u2, 6, &t2);
  zmod(t2, n, &t2);
  zaddmod(t1, t2, n, &t1);
  z_one(&t2);
  zsubmod(t2, t1, n, &t1);
  zsmul(u3, 4, &t2);
  zmod(t2, n, &t2);
  if (zinv(t2, n, &t3))
    {
      zcopy(t3, f);
      return (1);
    }
  zmulmod(t1, t3, n, alpha);
  zintoz(2, &t1);
  zaddmod(*alpha, t1, n, &t1);
  zintoz(4, &t2);
  if (zinv(t2, n, &t3))
    {
      zcopy(t3, f);
      return (1);
    }
  zmulmod(t1, t3, n, rap);
  zsmul(u, 3, &t1);
  zmod(t1, n, &t1);
  zmulmod(t1, t3, n, x);
  zintoz(3, &t1);
  if (zinv(t1, n, &t2))
    {
      zcopy(t2, f);
      return (1);
    }
  zmulmod(*alpha, t2, n, &t1);
  zsubmod(n, t1, n, mu);
  ztom(*rap, rap);
  ztom(*alpha, alpha);
  ztom(*mu, mu);
  ztom(*x, x);
  return (0);
}

PRIVATE long ph1exp(n,x,e,rap,f)
verylong n;
verylong *x;
verylong e;
verylong rap;
verylong *f;
{
  /* x = x^e on rap mod n */
  /* if 0, success */
  /* if 1, n factored, factor in f */
  static verylong x1 = 0;
  static verylong xn = 0;
  static verylong xn1 = 0;
  static verylong zzn = 0;
  static verylong zn1 = 0;
  static verylong s = 0;
  static verylong s1 = 0;
  static verylong d = 0;
  static verylong d1 = 0;
  static verylong t1 = 0;
  static verylong t2 = 0;
  static verylong t3 = 0;
  static verylong t4 = 0;
  static verylong longmask = 0;
  register long mask;
  register long maskcnt;
  register long eatmaskcnt;
  register long first = 1;

  if (zscompare(e,1) <= 0)
    return (0);
  zcopy(*x, &x1);
  zcopy(x1, &xn1);
  zcopy(zr, &zn1);
  zaddmod(xn1, zn1, n, &s1);
  zsubmod(xn1, zn1, n, &d1);
  zmontsq(s1, &t1);
  zmontsq(d1, &t2);
  zmontmul(t1, t2, &xn);
  zsubmod(t1, t2, n, &t3);
  zmontmul(rap, t3, &t4);
  zaddmod(t2, t4, n, &t4);
  zmontmul(t3, t4, &zzn);
  zintoz(1,&longmask);
  zlshift(longmask,z2log(e),&longmask);
  zsub(longmask,e,&e);
  maskcnt = longmask[0];
  mask = longmask[maskcnt] >> 1;
  if (!mask)
    {
      mask = (RADIX >> 2);
      maskcnt --;
    }
  else
    {
      mask >>= 1;
      if (!mask)
	{
	  mask = (RADIX >> 1);
	  maskcnt --;
	}
    }
  for (;maskcnt >0; maskcnt --)
    {
      if (maskcnt > e[0])
	eatmaskcnt = 0;
      else
	eatmaskcnt = e[maskcnt];
      while (mask)
	{
	  zaddmod(xn, zzn, n, &s);
	  zsubmod(xn, zzn, n, &d);
	  zaddmod(xn1, zn1, n, &s1);
	  zsubmod(xn1, zn1, n, &d1);
	  if (mask&eatmaskcnt)
	    {
	      if ((maskcnt ) || ((!maskcnt) && (!(mask & 1))))
		{
		  if (first)
		    {
		      zcopy(xn, &xn1);
		      zcopy(zzn, &zn1);
		    }
		  else
		    {
		      zmontsq(s1, &t1);
		      zmontsq(d1, &t2);
		      zmontmul(t1, t2, &xn1);
		      zsubmod(t1, t2, n, &t3);
		      zmontmul(rap, t3, &t4);
		      zaddmod(t2, t4, n, &t4);
		      zmontmul(t3, t4, &zn1);
		    }
		}
	      zmontmul(d1, s, &t1);
	      zmontmul(d, s1, &t2);
	      zaddmod(t1, t2, n, &t3);
	      zmontsq(t3, &xn);
	      zsubmod(t1, t2, n, &t3);
	      zmontsq(t3, &t3);
	      zmontmul(x1, t3, &zzn);
	    }
	  else
	    {
	      zmontsq(s, &t1);
	      zmontsq(d, &t2);
	      zmontmul(t1, t2, &xn);
	      zsubmod(t1, t2, n, &t3);
	      zmontmul(rap, t3, &t4);
	      zaddmod(t2, t4, n, &t4);
	      zmontmul(t3, t4, &zzn);
	      if ((maskcnt ) || ((!maskcnt) && (!(mask & 1))))
		{
		  zmontmul(d1, s, &t1);
		  zmontmul(d, s1, &t2);
		  zaddmod(t1, t2, n, &t3);
		  zmontsq(t3, &xn1);
		  zsubmod(t1, t2, n, &t3);
		  zmontsq(t3, &t3);
		  zmontmul(x1, t3, &zn1);
		}
	    }
	  mask >>= 1; first = 0;
	}
      mask = (RADIX >> 1);
    }
  if (zinv(zzn, n, f))
    return (1);
  zmontmul(*f, zrrr, x);
  zmontmul(*x, xn, x);
  return (0);
}

PRIVATE long ph1(n,x,m,rap,f,spg)
verylong n;
verylong *x;
long m;
verylong rap;
verylong *f;
IN struct smallprimegen *spg;
{
  /* x = x^B, B is product prime powers <=m	 */
  /* if 0, success				 */
  /* if 1, n factored, factor in f		 */
  static verylong e = 0;
  register long p;
  register long q;
  register long mp;

  zpstart2(spg);
  zintoz(1,&e);
  while ((p = zpnext(spg, NULL)) <= m)
    {
      mp = m / p;
      q = p;
      while (q <= mp)
	q *= p;
#if (PHASE1_BATCH == 1)
      zintoz(q,&e);
      if (ph1exp(n, x, e, rap, f) > 0)
	return (1);
#else
      zsmul(e,q,&e);
      iter_cnt ++;
      if (iter_cnt == PHASE1_BATCH)
	{
	  if (ph1exp(n, x, e, rap, f) > 0)
	    return (1);
	  iter_cnt = 0;
	  zintoz(1,&e);
	}
    }
  if (iter_cnt)
    {
      if (ph1exp(n, x, e, rap, f) > 0)
	return (1);
#endif
    }
  return (0);
}

PRIVATE long ph1toph2(n,x,y,ra,alpha,mu,f)
verylong n;
verylong *x;
verylong *y;
verylong *ra;
verylong alpha;
verylong mu;
verylong *f;
{
  /* tranform from first type of curve to second	 */
  /* x, alpha, mu input; x, y, ra output		 */
  /* if 0, success				 */
  /* if 1, n factored, factor in f		 */
  static verylong beta = 0;
  static verylong t = 0;

  zaddmod(*x, alpha, n, &t);
  zmontmul(*x, t, &t);
  zmontmul(*x, t, &t);
  zaddmod(*x, t, n, &beta);
  if (zinv(beta, n, y))
    {
      zcopy(*y, f);
      return (1);
    }
  zmontmul(*y, zrrr, y);
  zsubmod(*x, mu, n, &t);
  zmontmul(t, *y, x);
  zmontmul(alpha, mu, &t);
  zaddmod(t, zr, n, &t);
  zmontmul(*y, t, &t);
  zmontmul(*y, t, ra);
  return (0);
}

PRIVATE long ph2squ(n,x1s,y1s,x2,y2,ra,f)
verylong n;
verylong x1s;
verylong y1s;
verylong *x2;
verylong *y2;
verylong ra;
verylong *f;
{
  /* (x2,y2)=(x1s,y1s)^2 on y^2=x^3+ra*x-b 	 */
  /* infinity if x1s[0] < 0			 */
  /* if 0, success				 */
  /* if 1, n factored, factor in f		 */
  static verylong x1 = 0;
  static verylong yy1 = 0;
  static verylong t = 0;

  zcopy(x1s, &x1);
  zcopy(y1s, &yy1);
  if (x1s[0] < 0)
    {
      (*x2)[0] = -1;
      z_zero(y2);
      z_zero(f);
      return (1);
    }
  zaddmod(yy1, yy1, n, &t);
  if (zinv(t, n, y2))
    {
      zcopy(*y2, f);
      return (1);
    }
  zmontmul(*y2, zrrr, y2);
  zmontsq(x1,x2);
  zaddmod(*x2,*x2,n,&t);
  zaddmod(*x2,t,n,&t);
  zaddmod(t, ra, n, &t);
  zmontmul(t, *y2, y2);
  zmontsq(*y2, &t);
  zsubmod(t, x1, n, &t);
  zsubmod(t, x1, n, x2);
  zsubmod(x1, *x2, n, &t);
  zmontmul(*y2, t, &t);
  zsubmod(t, yy1, n, y2);
  return (0);
}

PRIVATE long ph2mul(n,x1s,y1s,x2s,y2s,x3,y3,ra,f)
verylong n;
verylong x1s;
verylong y1s;
verylong x2s;
verylong y2s;
verylong *x3;
verylong *y3;
verylong ra;
verylong *f;
{
  /* (x3,y3)=(x1s,y1s)*(x2s,y2s) on y^2=x^3+ra*x-b 	 */
  /* infinity if x1s[0] < 0 (or x2s[0]<0)			 */
  /* if 0, success					 */
  /* if 1, n factored, factor in f			 */
  static verylong x1 = 0;
  static verylong yy1 = 0;
  static verylong x2 = 0;
  static verylong y2 = 0;
  static verylong t = 0;

  zcopy(x1s, &x1);
  zcopy(y1s, &yy1);
  zcopy(x2s, &x2);
  zcopy(y2s, &y2);
  if (x1s[0] < 0)
    {
      zcopy(x2, x3);
      zcopy(y2, y3);
      zcopy(y2, f);
      return (1);
    }
  if (x2s[0] < 0)
    {
      zcopy(x1, x3);
      zcopy(yy1, y3);
      zcopy(yy1, f);
      return (1);
    }
  if (!(zcompare(x1, x2)))
    {
      zaddmod(yy1, y2, n, &t);
      if (!t[1] && t[0] == 1)
	{
	  (*x3)[0] = -1;
	  z_zero(y3);
	  z_zero(f);
	  return (1);
	}
      else
	return (ph2squ(n, x1, yy1, x3, y3, ra, f));
    }
  else
    {
      zsubmod(x1, x2, n, &t);
      if (zinv(t, n, y3))
	{
	  zcopy(*y3, f);
	  return (1);
	}
      zmontmul(*y3, zrrr, y3);
      zsubmod(yy1, y2, n, &t);
      zmontmul(t, *y3, y3);
      zmontsq(*y3, &t);
      zsubmod(t, x1, n, &t);
      zsubmod(t, x2, n, x3);
      zsubmod(x1, *x3, n, &t);
      zmontmul(*y3, t, &t);
      zsubmod(t, yy1, n, y3);
      return (0);
    }
}

PRIVATE long ph2exp(n,x1,yy1,x2,y2,e,ra,f)
verylong n;
verylong x1;
verylong yy1;
verylong *x2;
verylong *y2;
long e;
verylong ra;
verylong *f;
{
  /* (x2,y2)=(x1,yy1)^e on y^2=x^3+ra*x-b 	 */
  /* if 0, success			 */
  /* if 1, n factored, factor in f	 */
  static verylong x1s = 0;
  static verylong y1s = 0;
  static verylong x3 = 0;
  static verylong y3 = 0;
  register long i;
  register long two;
  register long three;
  long ee[NBITSH];

  if (e == 1)
    {
      zcopy(x1, x2);
      zcopy(yy1, y2);
      return (0);
    }
  zcopy(x1, &x1s);
  zcopy(yy1, &y1s);
  if (e == 2)
    return (ph2squ(n, x1s, y1s, x2, y2, ra, f));
  i = -1;
  two = 0;
  three = 0;
  while (e)
    {
      ee[++i] = (e & 3);
      e >>= 2;
      if (ee[i] == 3)
	three = 1;
    }
  if (three || ee[i] == 2)
    {
      if (ph2squ(n, x1s, y1s, x2, y2, ra, f))
	return (1);
      two = 1;
      if (three && ph2mul(n, x1s, y1s, *x2, *y2, &x3, &y3, ra, f))
	return (1);
    }
  if (ee[i] == 3)
    {
      zcopy(x3, x2);
      zcopy(y3, y2);
    }
  if (ee[i] == 1)
    {
      if (two)
	two = 0;
      else
	{
	  two = 1;
	  zcopy(x1s, x2);
	  zcopy(y1s, y2);
	}
    }
  for (i = i - 1; i >= 0; i--)
    {
      if (two)
	{
	  if (ph2squ(n, *x2, *y2, x2, y2, ra, f))
	    return (1);
	}
      else
	two = 1;
      switch (ee[i])
	{
	case 0:
	  {
	    if (ph2squ(n, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    break;
	  }
	case 1:
	  {
	    if (ph2squ(n, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    if (ph2mul(n, x1s, y1s, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    break;
	  }
	case 2:
	  {
	    if (ph2mul(n, x1s, y1s, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    if (ph2squ(n, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    break;
	  }
	case 3:
	  {
	    if (ph2squ(n, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    if (ph2mul(n, x3, y3, *x2, *y2, x2, y2, ra, f))
	      return (1);
	    break;
	  }
	}
    }
  return (0);
}

PRIVATE long ph2set(n,x,y,te,ra,f)
verylong n;
verylong x;
verylong y;
long te;
verylong ra;
verylong *f;
{
  /* set up table for table size te	 */
  /* if 0, success			 */
  /* if 1, n factored, factor in f	 */
  static verylong z = 0;
  register long i;
  register long j;
  long expi;
  long exp2;

  expi = RADIX / (te + 2);
  exp2 = 1 + zrandom(expi);
  expi = 1 + zrandom(expi);
  for (i = 0; i <= te; i++)
    {
      zcopy(x, &(ecm_tex[i]));
      zcopy(y, &(ecm_tey[i]));
      for (j = te; j > 0; j--)
	if (ph2exp(n, ecm_tex[i], ecm_tey[i], &(ecm_tex[i]), &(ecm_tey[i]), expi, ra, f))
	  return (1);
      expi += exp2;
    }
  for (j = 1; j <= te; j++)
    {
      for (i = te; i >= j; i--)
	{
	  zsubmod(n, ecm_tey[i - 1], n, &z);
	  if (ph2mul(n, ecm_tex[i], ecm_tey[i], ecm_tex[i - 1], z, &(ecm_tex[i]), &(ecm_tey[i]), ra, f))
	    return (1);
	}
    }
  return (0);
}

PRIVATE void ph2prec(n,n1,j)
verylong n;
long n1;
long j;
{
  static verylong mcn2j = 0;
  static verylong t = 0;
  register long i;
  register long n2;
  register long n2j;

  if (n1 > 1)
    {
      n2 = ((n1 - 1) >> 1);
      n2j = j + n2;
      zaddmod(ecm_coef[n2j], znm, n, &(ecm_coef[n2j]));
      zsubmod(n, ecm_coef[n2j], n, &mcn2j);
      n2j ++;
      for (i = 0; i < n2; i++)
	{
	  zmontmul(ecm_coef[i + n2j], mcn2j, &t);
	  zaddmod(ecm_coef[i + j], t, n, &(ecm_coef[i + j]));
	}
      ph2prec(n, n2, n2j);
      ph2prec(n, n2, j);
    }
}

PRIVATE void ph2eval(n,ind,n1,j,result)
verylong n;
long ind;
long n1;
long j;
verylong *result;
{
  register long n2;

  if (n1 == 1)
    zaddmod(ecm_coef[j], ecm_power[0], n, result);
  else
    {
      n2 = ((n1 - 1) >> 1);
      zaddmod(ecm_coef[n2 + j], ecm_power[ind], n, &(ecm_eval[ind]));
      ph2eval(n, ind - 1, n2, n2 + j + 1, result);
      zmontmul((ecm_eval[ind]), *result, &(ecm_eval[ind]));
      ph2eval(n, ind - 1, n2, j, result);
      zaddmod((ecm_eval[ind]), *result, n, result);
    }
}

PRIVATE long ph2(n,inx,iny,m,te,ra,f)
verylong n;
verylong inx;
verylong iny;
long m;
long te;
verylong ra;
verylong *f;
{
  /* does second phase for m 		 */
  /* if 0, no factor found		 */
  /* if 1, n factored, factor in f	 */
  static long non_initialized = 1;
  static verylong x = 0;
  static verylong y = 0;
  static verylong x1 = 0;
  static verylong x2 = 0;
  static verylong prod = 0;
  register long j;
  register long k;
  long r;
  long r2;
  long s;
  long ind;

  if (non_initialized)
    {
      for (j = 0; j < MAXRP; j++)
	ecm_coef[j] = 0;
      for (j = 0; j < MAXTEP; j++)
	ecm_tex[j] = ecm_tey[j] = 0;
      for (j = 0; j < MAXPWRP; j++)
	ecm_power[j] = 0;
      for (j = 0; j < MAXPWRP; j++)
	ecm_eval[j] = 0;
      non_initialized = 0;
    }
  zcopy(inx, &x);
  zcopy(iny, &y);
  if (ph2set(n, x, y, te, ra, f))
    return (1);
  if ((r = 100 * te) >= MAXRP)
    r = MAXRP - 1;
  r2 = 1;
  ind = 0;
  j = 3;
  while (j <= r)
    {
      ind++;
      r2 = j;
      j <<= 1;
      j++;
    }
  r = r2;
  s = m / r;
  while (r > 3 && s < 2 * r)
    {
      r >>= 1;
      ind--;
      s = m / r;
    }
  zcopy(zr, (&(ecm_coef[0])));
  for (j = 1; j <= r; j++)
    {
      for (k = 0; k < te; k++)
	{
	  if (ph2mul(n, ecm_tex[k], ecm_tey[k], ecm_tex[k + 1], ecm_tey[k + 1], &(ecm_tex[k]), &(ecm_tey[k]), ra, f))
	    return (1);
	}
      zsubmod(n, ecm_tex[0], n, &x1);
      zcopy(ecm_coef[j - 1], &ecm_coef[j]);
      for (k = j - 1; k > 0; k--)
	{
	  zmontmul(x1, ecm_coef[k], &x2);
	  zaddmod(x2, ecm_coef[k - 1], n, &(ecm_coef[k]));
	}
      zmontmul(x1, ecm_coef[0], &(ecm_coef[0]));
    }
  ph2prec(n, r, (long) 0);
  if (ph2set(n, x, y, te, ra, f))
    return (1);
  z_one(&prod);
  for (j = s; j > 0; j--)
    {
      for (k = 0; k < te; k++)
	{
	  if (ph2mul(n, ecm_tex[k], ecm_tey[k], ecm_tex[k + 1], ecm_tey[k + 1], &(ecm_tex[k]), &(ecm_tey[k]), ra, f))
	    return (1);
	}
      zcopy(ecm_tex[0], &(ecm_power[0]));
      for (k = 0; k < ind; k++)
	zmontsq(ecm_power[k], &(ecm_power[k + 1]));
      ph2eval(n, ind, r, (long) 0, &x2);
      zmontmul(x2, prod, &prod);
    }
  if (zinv(prod, n, f))
    return (1);
  return (0);
}

#define message(str,t) {}

PRIVATE long trial(n,m,f,info,spg)
verylong n;
long m;
verylong *f;
long info;
IN struct smallprimegen *spg;
{
  /* attempt to factor n, smoothness bound phase 1 is m	 */
  /* if 0, no success					 */
  /* if 1, n factored in setup, factor in f 		 */
  /* if 2, n factored in phase 1, factor in f 		 */
  /* if 3, n factored in ph1toph2, factor in f 		 */
  /* if 4, n factored in phase 1, factor in f 		 */
  static verylong x = 0;
  static verylong y = 0;
  static verylong rap = 0;
  static verylong alpha = 0;
  static verylong mu = 0;
  static verylong ra = 0;
  static long te;

  if (ph1set(n, &rap, &alpha, &mu, &x, f) > 0)
    return (1);
  if (m > MAXM)
    m = MAXM;
  if (ph1(n, &x, m, rap, f, spg))
    return (2);
  if (ph1toph2(n, &x, &y, &ra, alpha, mu, f))
    return (3);
  /*
   * if (m >= 4500000) te = 60; else
   */ if (m >= 1125000)
     te = 30;
   else if (m >= 720000)
     te = 24;
   else if (m >= 405000)
     te = 18;
   else if (m >= 180000)
     te = 12;
   else if (m >= 45000)
     te = 6;
   else if (m >= 11250)
     te = 3;
   else if (m >= 5000)
     te = 2;
   else
     te = 1;
   m *= 10;
   if (ph2(n, x, y, m, te, ra, f))
     return (4);
   return (0);
}

PUBLIC long zecm(n,f,uni,nb,bound,co,info,spg)
verylong n;
verylong *f;
long uni;
long nb;
long bound;
long co;
long info;
IN struct smallprimegen *spg;
{
  /* attempt to factor n, factor in f */
  /* uni is seed for random number generator */
  /* uni=0 means random generator started already */
  /* nb is number of curves */
  /* bound is bound for phase 1 of first curve */
  /* |co| is maximum number of probabilistic compositeness tests */
 /* info = 0: no timings or information */
  /* |info|=1: timing per curve, and some additional info */
  /* |info|>1: timing per phase and per curve */
  /* side effect: restart prime number generator, */
  /* zpnext() undefined after call */
  static verylong q = 0;
  static verylong r = 0;
  register long i;
  register long j;

  liperr = 0;
  if (info < 0)
    info = -info;
  if (bound < MINBOUND)
    bound = MINBOUND;
  if (uni)
    {
      if (uni < 0)
	uni = ((-uni) << 1);
      else
	uni = 2 * uni + 1;
      zrstarts(uni + 1);
    }
  if (!n || n[0] < 0 || (n[0] == 1 && n[1] <= RADIXROOT))
    {
      return (0);
    }
  if (!(n[1] & 1))
    {
      zintoz(2, f);
      return (1);
    }
  if (co)
    {
      if (co > 0)
	co = -co;
      zcopy(n, f);
      i = zcomposite(f, co, 0, 1); /* added ',1' jgl 08 mar 2021 */
      if (!i)
	{
	  return (-1);
	}
      if (i == -2)
	{
	  return (1);
	}
      if (i == -1)
	{
	  return (2);	/* factor in f^n-f */
	}
      if (i != 1)
	{
	  liperr = LIP_ECM_BUG;
	  /*zhalt("impossible return value of zcomposite in zecm   BUG");*/
	  return (0);
	}
    }

  z_one(f);
  zmkeep(n);

  for (i = nb; i > 0; i--)
    {
      j = trial(n, bound, f, info, spg);
      bound = (long) (1.02 * bound);
      if (j)
	{
	  if (((*f)[0] == 1 && (*f)[1] <= 1) || !zcompare(n, *f))
	    {
	      continue;
	    }
	  zquot(n, *f, &q, &r);
	  if (r[1] || r[0] != 1)
	    {
	      liperr = LIP_ECM_BUG;
	      /*
		zfwriteln(stderr,*f);
		zhalt("this is a wrong factor, found in zecm   BUG");
		*/
	      zmback();
	      return (0);
	    }
	  zmback();
	  return (1);
	}
    }
  zmback();
  return (0);
}

DEF_ACTION_4ARGS(zecm) /* zecm : Int * int * int * int -> (Int * bool) option */
{
  /* args = (n, curvebnd, phase1bnd, nbtests) */
  long res;
  verylong f = 0;
  struct smallprimegen *spg;

  spg = new_spg (1L << NPBITS);
  zpstart(spg);
  zpstart2(spg);
  liperr = 0;
  res = zecm ((verylong)EXTDATA(arg1), &f, randLong (), IVAL(arg2),
	      IVAL(arg3), IVAL(arg4), 0L, spg);
  MARKFREE(spg);
  if (liperr)
    {
    err:
      if (f)
	{ MARKFREE(&f[-1]); } /*@@@*/
      return raiseLip (liperr);
    }
  if (res==0)
    {
      liperr = LIP_FACTOR_FAIL;
      goto err;
    }
  if (res<0)
    return none;
  {
    ExprPtr fx, x;
    ENTER;

    fx = lipMake (f);
    (void) KEEP (fx);
    fx = Ktuple2 (fx, (res==1)?trueValue:falseValue);
    (void) KEEP (fx);
    x = Kdata (0, optttm, fx); /* SOME (f, bool) */
    QUIT;
    return x;
  }
}
ACTION_END

#endif
